/*
 * $Id$
 * 
 * Created on 02.08.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TimeZone;

import org.apache.log4j.Logger;

import ch.toe.famix.model.Model;
import ch.toe.famix.model.Package;
import ch.toe.famix.model.container.ClassContainer;
import ch.toe.famix.model.container.MethodContainer;
import ch.toe.famix.model.container.PackageContainer;

/**
 * @author tsa
 *
 * This method is the main FAMIX wrapper class. It contains a Model and a Class (which then itself
 * contains childs such as Attributes, Methods, and so on). 
 */
public class FAMIXInstance implements Visitable {

	// logging device
	private static Logger logger = Logger.getLogger(FAMIXInstance.class);
	
	/**
	 * Time of instance creation
	 */
	private final long instanceCreationStart;
	
	/**
	 * Time of finishing instance creation
	 */
	private long instanceCreationEnd = 0;
	
	/**
	 * Container reference
	 */
	private ClassContainer classContainer = new ClassContainer();
	
	/**
	 * Container reference
	 */
	private PackageContainer packageContainer = new PackageContainer();

	/**
	 * Container reference
	 */
	private MethodContainer methodContainer = new MethodContainer();
	
	/**
	 * This is the Model of the current FAMIX tree.
	 */
	private Model model;

	/**
	 * These are the top nodes of the instances.
	 */
	private Set<Package> topNodes = new LinkedHashSet<Package>();
	
	/**
	 * @param model
	 */
	public FAMIXInstance(Model model) {
		instanceCreationStart = System.currentTimeMillis();
		this.model = model;
		logger.debug("Instance created.");
	}
	
	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return "FAMIXInstance";
	}

	/**
	 * Looks for a Package with the given name in the collection. Adds new Package
	 * if none has been found.
	 * Will add parent Packages if given name is a unique name (separated by dots).
	 * 
	 * @param packagename
	 * @return the Package that has been added/found to/in the instance
	 */
	public Package addPackage(String packagename, Package parent) {
		// search for package in collection
		Package ret = this.getPackageContainer().findByName(packagename);
		// no package found in collection, add a new one
		if (ret == null) {
			// when no parent given, look for possible parent in packagename
			if (parent == null) {
				// if name contains dots, it's a uniqueName otherwise a simple name
				// package simply has no parent
				if (Package.isSimpleName(packagename)) {
					ret = new Package(packagename);
				} 
				// process all name parts
				else {
					String[] nameParts = Package.splitName(packagename);
					if (nameParts.length>1) {
						// process multiple parts
						String parentpackagename = packagename.substring(0, packagename.lastIndexOf("."));
						Package newparent = this.addPackage(parentpackagename, null);
						ret = new Package(nameParts[nameParts.length-1], newparent);
					} else if (nameParts.length == 1) {
						// somehow this is nevertheless a simple name, should have been caught on simple name check
						logger.warn("Packages not simple name showed up as simple name. Check package "+packagename);
						ret = new Package(packagename);
					} else {
						// now, this is weird. shouldn't happen.
						logger.fatal("Package name was not a simple name, but did not have any name parts! Discarding "+packagename);
					}
					
				}
			}
			// parent was given, create new package with given parent, add to collection
			else {
				ret = new Package(packagename, parent);
			}			
			if (ret != null) {
				if (this.getPackageContainer().add(ret))
					logger.debug("Created new package: "+ret.getName());			
			}
		}
		// package already created, returning it
		else  {
			logger.debug("Package "+ret.getName()+" already exists.");
		}
		this.addTopNodePackage(ret);
		return ret;
	}
	
	/**
	 * Adds given Package to top node packages. 
	 * 
	 * @param mypackage
	 */
	private void addTopNodePackage(Package mypackage) {
		if (mypackage != null) {
			// if any packets in topNode are childs of mypackage, remove them
			Iterator iter = topNodes.iterator();
			while (iter.hasNext()) {
				Package pkg = (Package)iter.next();
				if (pkg.getBelongsTo() != null && ((Package)pkg.getBelongsTo()).equals(mypackage) )
					iter.remove();
			}
			
			// check if mypackage has no parent or parent is already in topNodes
			if (mypackage.getBelongsTo() == null ||
					!( (mypackage.getBelongsTo() != null) && (topNodes.contains((Package)mypackage.getBelongsTo()))) ) {				
				if (!topPackageIsInTopNodes(mypackage)) {
					if (this.topNodes.add(mypackage))
						logger.debug("Adding Package "+mypackage.getName()+" as top node of instance.");					
				}
				else 
					logger.debug("Discarding Package "+mypackage.getName()+" as top node of instance.");
			}
			else {
				logger.debug("Discarding Package "+mypackage.getName()+" as top node of instance.");
			}
		}
	}

	/**
	 * Test a Package if its top node is in topNodes
	 * 
	 * @param pkg
	 * @return true if the given Packets top Package is already in topNodes, false otherwise
	 */
	private boolean topPackageIsInTopNodes(Package pkg) {
		// get top package name
		String[] nameParts = Package.splitName(pkg.getUniqueName());
		Iterator iter = topNodes.iterator();
		while (iter.hasNext()) {
			Package top = (Package)iter.next();
			if (top.getUniqueName().equals(nameParts[0]))
				return true;
		}
		return false;
	}
	
	/**
	 * This stops the time accounting for this instance creation and
	 * returns the duration for the creation (@see getInstanceCreationDuration())
	 * in miliseconds.
	 * 
	 * @return the time in ms since the creation of this instance
	 */
	public long stopInstanceCreation() {
		instanceCreationEnd = System.currentTimeMillis();
		return getInstanceCreationDuration();
	}

	/**
	 * Get the duration of the instance creation.
	 * 
	 * @return duration of instance creation. When negative time elapsed since creation start.
	 */
	public long getInstanceCreationDuration() {
		return instanceCreationEnd - instanceCreationStart;
	}

	/**
	 * Get the duration of the instance creation. Empty if not finished yet.
	 * 
	 * @return duration of instance creation. When empty, the creation has not yet ended.
	 */
	public String getInstanceCreationDurationString() {
		if (instanceCreationEnd == 0)
			return "";
		SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss.SS");
		dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
		return dateFormat.format(new Date(getInstanceCreationDuration()));
	}

	/**
	 * This guides the Visitor to see this instances model and the packages
	 */
	public void accept(Visitor v) {
		v.visit(this);
		model.accept(v);
		for (Iterator iter = topNodes.iterator(); iter.hasNext(); ) 
			((Package)iter.next()).accept(v);
		v.endVisit(this);
	}

	/**
	 * @return Returns the model.
	 */
	public Model getModel() {
		return model;
	}

	/**
	 * @return Returns the topNodes.
	 */
	public Set<Package> getTopNodes() {
		return topNodes;
	}

	/**
	 * @return Returns the classContainer.
	 */
	public ClassContainer getClassContainer() {
		return classContainer;
	}

	/**
	 * @return Returns the packageContainer.
	 */
	public PackageContainer getPackageContainer() {
		return packageContainer;
	}

	/**
	 * @return Returns the methodContainer.
	 */
	public MethodContainer getMethodContainer() {
		return methodContainer;
	}
}
