/*
 * $Id$
 * 
 * Created on 14.10.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix;

import java.util.ArrayList;

import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import patviz.javarsf.TypeParser;
import ch.toe.famix.model.Model;

public class FAMIXUtil {

	/**
	 * Convenience method, converting an array of ASTNodes into FAMIX instances
	 * 
	 * @param monitor
	 * @param nodes
	 * @return an array of FAMIXInstances, order/length of array might not match input array order/length
	 * @throws InterruptedException 
	 */
	public static FAMIXInstance[] convertToFamix(SubProgressMonitor monitor, ASTNode[] nodes) throws InterruptedException {
		monitor.beginTask("Convert nodes to FAMIX", nodes.length);
		ArrayList<FAMIXInstance> ret = new ArrayList<FAMIXInstance>();
		for (int i=0; i<nodes.length; i++) {
			if (nodes[i] instanceof MethodDeclaration) {
				ret.add(convertToFamix((MethodDeclaration)nodes[i]));				
			}
			if (nodes[i] instanceof TypeDeclaration) {
				ret.add(convertToFamix((TypeDeclaration)nodes[i]));				
			}
			if (monitor.isCanceled())
				throw new InterruptedException();
			monitor.worked(1);
		}
		monitor.done();
		return ret.toArray(new FAMIXInstance[ret.size()]);
	}

	/**
	 * Take a node and convert it using the appropriate method
	 * 
	 * @param monitor
	 * @param node
	 * @return converted FAMIX instance
	 */
	public static FAMIXInstance convertToFamix(SubProgressMonitor monitor, ASTNode node) {
		if (node instanceof MethodDeclaration)
			return FAMIXUtil.convertToFamix(monitor, (MethodDeclaration)node);
		if (node instanceof TypeDeclaration)
			return FAMIXUtil.convertToFamix(monitor, (TypeDeclaration)node);
		return null;
	}
	
	/**
	 * Converts a MethodDeclaration into its FAMIX representation
	 * 
	 * @param monitor
	 * @param method
	 * @return the FAMIXInstance
	 */
	private static FAMIXInstance convertToFamix(SubProgressMonitor monitor, MethodDeclaration method) {
		monitor.beginTask("Convert MethodDeclaration to FAMIX", 1);
		FAMIXInstance ret = convertToFamix(method);
		monitor.done();
		return ret;
	}

	/**
	 * Converts a TypeDeclaration into its FAMIX representation
	 * 
	 * @param monitor
	 * @param type
	 * @return the FAMIXInstance
	 */
	private static FAMIXInstance convertToFamix(SubProgressMonitor monitor, TypeDeclaration type) {
		monitor.beginTask("Convert TypeDeclaration to FAMIX", 1);
		FAMIXInstance ret = convertToFamix(type);
		monitor.done();
		return ret;
	}

	/**
	 * Converts a MethodDeclaration into its FAMIX representation 
	 * 
	 * @param method
	 * @return the FAMIXInstance
	 */
	private static FAMIXInstance convertToFamix(MethodDeclaration method) {
        Model model = new Model("Coogle", "Single MethodDeclaration", "3", "1.4");
        FAMIXInstance famix = new FAMIXInstance(model);
        TypeParser typeParser = new TypeParser(famix, null);
        typeParser.parseMethodDecl(method);
        return famix;
	}


	/**
	 * Converts a MethodDeclaration into its FAMIX representation 
	 * 
	 * @param type
	 * @return the FAMIXInstance
	 */
	private static FAMIXInstance convertToFamix(TypeDeclaration type) {
        Model model = new Model("Coogle", "Single TypeDeclaration", "3", "1.4");
        FAMIXInstance famix = new FAMIXInstance(model);
        TypeParser typeParser = new TypeParser(famix, null);
        typeParser.parse(type);
        return famix;
	}

	/**
	 * Hangle through AST and fetch me the MethodDeclaration with the given name
	 * which must be a child of the ASTNode somewhere.
	 * 
	 * @param node
	 * @return the last MethodDeclaration which matches the name 
	 */
	public static MethodDeclaration findMethodDeclaration(ASTNode node, String name) {
		MethodDeclarationASTVisitor visitor = new MethodDeclarationASTVisitor(name); 
		node.accept(visitor);
		return visitor.found;
	}

	/**
	 * Hangle through AST and fetch me the TypeDeclaration with the given name
	 * which must be a child of the ASTNode somewhere.
	 * 
	 * @param node
	 * @return the last TypeDeclaration which matches the name 
	 */
	public static TypeDeclaration findTypeDeclaration(ASTNode node, String name) {
		TypeDeclarationASTVisitor visitor = new TypeDeclarationASTVisitor(name); 
		node.accept(visitor);
		return visitor.found;
	}
}
