/*
 * $Id$
 * 
 * Created on 27.09.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix;

import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;

import ch.toe.famix.model.Access;
import ch.toe.famix.model.Attribute;
import ch.toe.famix.model.FormalParameter;
import ch.toe.famix.model.ImplicitVariable;
import ch.toe.famix.model.InheritanceDefinition;
import ch.toe.famix.model.Invocation;
import ch.toe.famix.model.LocalVariable;
import ch.toe.famix.model.Model;
import ch.toe.famix.model.TypeCast;
import ch.toe.famix.model.container.Claim;

public class PrintVisitor implements Visitor {

	/**
	 * Specifies the String to use for indentation
	 */
	private static final String INDENTATION = "  ";
	
	/**
	 * Saves the PrintWriter to write to
	 */
	private PrintWriter pw;

	/**
	 * At which level of indentation we are
	 */
	private int indentationLevel = -1;
	
	/**
	 * Default constructor
	 * 
	 * @param pw the writer for output
	 */
	public PrintVisitor(PrintWriter pw) {
		this.pw = pw;
	}

	/**
	 * Construct visitor with given start indentationLevel
	 * 
	 * @param pw
	 * @param indentationLevel
	 */
	public PrintVisitor(PrintWriter pw, int indentationLevel) {
		this.pw = pw;
		this.indentationLevel = indentationLevel;
	}

	/**
	 * Called by all visit methods
	 */
	private void visitStart() {
		indentationLevel++;
	}

	/**
	 * Called by all endVisit methods
	 */
	private void visitEnd() {
		indentationLevel--;
	}
	
	public void visitFAMIXInstance(FAMIXInstance o) {
		visitStart();
		print("FAMIXInstance");
		print("Instance creation duration: "+o.getInstanceCreationDurationString());
	}
	
	public void endVisitFAMIXInstance(FAMIXInstance o) {
		if (o.getClassContainer().getClaims().size() > 0) {
			print("The following Class Claims are unresolved:");
			indentationLevel++;
			for (Iterator iter = o.getClassContainer().getClaims().iterator(); iter.hasNext(); ) {
				Claim claim = (Claim)iter.next();
				print(claim.getClaimerName()+" -> "+claim.getClaimed());
			}
			indentationLevel--;
		}
		if (o.getMethodContainer().getClaims().size() > 0) {
			print("The following Method Claims are unresolved:");
			indentationLevel++;
			for (Iterator iter = o.getMethodContainer().getClaims().iterator(); iter.hasNext(); ) {
				Claim claim = (Claim)iter.next();
				print(claim.getClaimerName()+" -> "+claim.getClaimed());
			}			
			indentationLevel--;
		}
		print("Thank you for visiting. Goodbye.");
		visitEnd();
	}
	
	public void visitModel(Model o) {
		print(o.getExporterName()+" "+
				o.getExporterDate()+" "+o.getExporterTime()+
				" - Version "+o.getExporterVersion()			
			);
		print(o.getPublisherName()+" "+
				o.getParsedSystemName()+
				" - Level: "+o.getExtractionLevel()+
				" - ("+o.getSourceLanguage()+" "+o.getSourceDialect()+")"
			);
	}
	
	public void visitClass(ch.toe.famix.model.Class o) {
		visitStart();
		print((o.isPublic() ? "public " :"")+
				(o.isAbstract() ? "abstract " : "")+
				(o.isFinal() ? "final " : "")+
				(o.isInterface() ? "interface" : "class")+
				" "+o.getUniqueName()
			);
	}

	public void endVisitClass(ch.toe.famix.model.Class o) {
		visitEnd();
	}
	
	public void visitInheritanceDefinition(InheritanceDefinition o) {
		indentationLevel++;
		if (o.isExtending()) {
			if (o.getSuperclass() != null)
				print("extends "+o.getSuperclass().getUniqueName());
			else
				print("extends "+o.getSuperclassString());
		}			
		if (o.isImplementing()) {
			if (o.getSuperclass() != null)
				print("implements "+o.getSuperclass().getUniqueName());
			else
				print("implements "+o.getSuperclassString());			
		}
		indentationLevel--;
	}
	
	public void visitAttribute(Attribute o) {
		print(o.getAccessControlQualifier()+
				(o.isFinal() ? " final" : "")+
				" "+o.getUniqueName()
			);		
	}

	public void visitMethod(ch.toe.famix.model.Method o) {
		visitStart();
		print(o.getAccessControlQualifier()+" "+
				(o.isAbstract() ? "abstract " : "")+
				(o.isHasClassScope() ? "static " : "")+
				(o.isFinal() ? "final " : "")+
				(o.isSynchronized() ? "synchronized " : "")+
				(o.isConstructor() ?
						"" :
						" "+(o.getDeclaredReturnType() == null ? "void" : o.getDeclaredReturnType()))+
				" "+o.getSignature()
			);
	}

	public void endVisitMethod(ch.toe.famix.model.Method o) {
		visitEnd();
	}
	
	public void visitFormalParameter(FormalParameter o) {
		print((o.isFinal() ? "final " : "")+
				o.getDeclaredType()+" "+o.getName()
			);
	}

	public void visitLocalVariable(LocalVariable o) {
		print((o.isFinal() ? "final " : "")+
				o.getDeclaredType()+" "+o.getName()
			);
	}

	public void visitImplicitVariable(ImplicitVariable o) {
		print(o.getDeclaredType()+" "+o.getName());
	}
	
	public void visitAccess(Access o) {
		if (o.hasUnresolvedAccess())
			print(o.getAccessesString());
		else
			print(o.getAccesses().getUniqueName());
	}

	public void visitInvocation(Invocation o) {
		visitStart();
		if (o.hasUnresolvedInvokes())
			print(o.getInvokesString());
		else
			print(o.getInvokes().getUniqueName());
	}

	public void endVisitInvocation(Invocation o) {
		visitEnd();
	}
	
	public void visitTypeCast(TypeCast o) {
		print(o.getFromType()+"->"+o.getToType());
	}
	
	/**
	 * Last resort
	 * 
	 * @param o
	 */
	public void visitObject(Object o) {
		//print("Unimplemented visit called for Object "+o.toString());
	}

	/**
	 * Last resort
	 * 
	 * @param o
	 */
	public void endVisitObject(Object o) {
	}
	
	/* (non-Javadoc)
	 * @see ch.toe.famix.Visitor#visit(java.lang.Object)
	 */
	public void visit(Object o) {
		// invoke visit
		Method method = getMethod("visit", o.getClass());
		try {
			method.invoke(this, new Object[] {o});
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		} catch (InvocationTargetException e) {
			e.getTargetException().printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see ch.toe.famix.Visitor#endVisit(java.lang.Object)
	 */
	public void endVisit(Object o) {
		// invoke endVisit
		Method method = getMethod("endVisit", o.getClass());
		try {
			method.invoke(this, new Object[] {o});
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		} catch (InvocationTargetException e) {
			e.getTargetException().printStackTrace();
		}		
	}

	/**
	 * This looks up a matching prefixCLASS(CLASS) method for the given class.
	 * 
	 * Code from "Java Tip 98: Reflect on the Visitor design pattern"
	 * (see http://www.javaworld.com/javaworld/javatips/jw-javatip98.html)
	 *
	 * @param prefix the String to prepend the method name
	 * @param c
	 * @return the method to call
	 */
	protected Method getMethod(String prefix, Class c) {
		Class newc = c;
		Method m = null;
		// Try the superclasses
		while (m == null && newc != Object.class) {
			String method = newc.getName();
			method = prefix + method.substring(method.lastIndexOf('.') + 1);
			try {
				m = getClass().getMethod(method, new Class[] {newc});
			} catch (NoSuchMethodException e) {
				newc = newc.getSuperclass();
			}
		}
		// Try the interfaces.  If necessary, you
		// can sort them first to define 'visitable' interface wins
		// in case an object implements more than one.
		if (newc == Object.class) {
			Class[] interfaces = c.getInterfaces();
			for (int i = 0; i < interfaces.length; i++) {
				String method = interfaces[i].getName();
				method = prefix + method.substring(method.lastIndexOf('.') + 1);
				try {
					m = getClass().getMethod(method, new Class[] {interfaces[i]});
				} catch (NoSuchMethodException e) {}
			}
		}
		if (m == null) {
			try {
				m = getClass().getMethod(prefix+"Object", new Class[] {Object.class});
			} catch (Exception e) {
				// Can't happen
			}
		}
		return m;		
	}

	/**
	 * Nicely prints a string to the printwriter using indentationLevel and all
	 * 
	 * @param str
	 */
	private void print(String str) {
		for (int i=0; i<indentationLevel; i++)
			pw.print(PrintVisitor.INDENTATION);
		pw.println(str);
	}

}
