/*
 * $Id$
 * 
 * Created on 22.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.Visitable;
import ch.toe.famix.Visitor;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public abstract class BehaviouralEntity extends Entity implements Visitable {

	/**
	 * Optional FAMIX attribute
	 * valid contents: ["", "public", "protected", "private"]
	 * "" equals default visibility which denotes package visibility
	 */
	private String accessControlQualifier = "";

	/**
	 * Mandatory FAMIX attribute
	 */
	private String signature;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isPureAccessor;

	/**
	 * Optional FAMIX attribute
	 */
	private String declaredReturnType;
	
	/**
	 * Optional FAMIX attribute
	 */
	private Class declaredReturnClass;	

	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type FormalParameter
	 * 
	 * @see FormalParameter
	 */
	private Set<FormalParameter> formalParameters = new LinkedHashSet<FormalParameter>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type LocalVariable
	 * 
	 * @see LocalVariable
	 */
	private Set<LocalVariable> localVariables = new LinkedHashSet<LocalVariable>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type ImplicitVariable
	 * 
	 * @see ImplicitVariable
	 */
	private Set<ImplicitVariable> implicitVariables = new LinkedHashSet<ImplicitVariable>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Access
	 * 
	 * @see Access
	 */
	private Set<Access> accesses = new LinkedHashSet<Access>();	
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type TypeCast
	 * 
	 * @see TypeCast
	 */
	private Set<TypeCast> typeCasts = new LinkedHashSet<TypeCast>();	

	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Invocation
	 * 
	 * @see Invocation
	 */
	private Set<Invocation> invocations = new LinkedHashSet<Invocation>();	
	
	/**
	 * 
	 * @param name
	 * @param signature
	 */
	public BehaviouralEntity(String name, String signature) {
		super(name);
		this.signature = signature;
	}

	public void accept(Visitor v) {
		v.visit(this);
		Iterator<FormalParameter> iterFormalParameter = this.getFormalParameters().iterator();
		while (iterFormalParameter.hasNext())
			(iterFormalParameter.next()).accept(v);
		Iterator<LocalVariable> iterLocalVariable = this.getLocalVariables().iterator();
		while (iterLocalVariable.hasNext())
			(iterLocalVariable.next()).accept(v);
		Iterator<ImplicitVariable> iterImplicitVariable = this.getImplicitVariables().iterator();
		while (iterImplicitVariable.hasNext())
			(iterImplicitVariable.next()).accept(v);
		Iterator<Access> iterAccess = this.getAccesses().iterator();
		while (iterAccess.hasNext())
			(iterAccess.next()).accept(v);
		Iterator<Invocation> iterInvocation = this.getInvocations().iterator();
		while (iterInvocation.hasNext())
			(iterInvocation.next()).accept(v);
		Iterator<TypeCast> iterTypeCast = this.getTypeCasts().iterator();
		while (iterTypeCast.hasNext())
			(iterTypeCast.next()).accept(v);
		v.endVisit(this);
	}

	/**
	 * @return Returns the signature.
	 */
	public String getSignature() {
		return signature;
	}

	/**
	 * @return Returns the uniqueName. 
	 */
	public String getUniqueName() {
		return super.getName();
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return getUniqueName();
	}

	/**
	 * @return true if instance is valid 
	 */
	public boolean validate() {
		return super.validate() && (signature != null);
	}
	/**
	 * @return Returns the accessControlQualifier.
	 */
	public String getAccessControlQualifier() {
		return accessControlQualifier;
	}
	/**
	 * @param accessControlQualifier The accessControlQualifier to set.
	 */
	public void setAccessControlQualifier(String accessControlQualifier) {
		this.accessControlQualifier = accessControlQualifier;
	}
	/**
	 * @return Returns the declaredReturnClass.
	 */
	public Class getDeclaredReturnClass() {
		return declaredReturnClass;
	}
	/**
	 * @param declaredReturnClass The declaredReturnClass to set.
	 */
	public void setDeclaredReturnClass(Class declaredReturnClass) {
		this.declaredReturnClass = declaredReturnClass;
	}
	/**
	 * @return Returns the declaredReturnType.
	 */
	public String getDeclaredReturnType() {
		return declaredReturnType;
	}
	/**
	 * @param declaredReturnType The declaredReturnType to set.
	 */
	public void setDeclaredReturnType(String declaredReturnType) {
		this.declaredReturnType = declaredReturnType;
	}
	/**
	 * @return Returns the isPureAccessor.
	 */
	public boolean isPureAccessor() {
		return isPureAccessor;
	}
	/**
	 * @param isPureAccessor The isPureAccessor to set.
	 */
	public void setPureAccessor(boolean isPureAccessor) {
		this.isPureAccessor = isPureAccessor;
	}

	/**
	 * @return Returns the accesses.
	 */
	public Set<Access> getAccesses() {
		return accesses;
	}

	/**
	 * @return Returns the formalParameters.
	 */
	public Set<FormalParameter> getFormalParameters() {
		return formalParameters;
	}

	/**
	 * @return Returns the implicitVariables.
	 */
	public Set<ImplicitVariable> getImplicitVariables() {
		return implicitVariables;
	}

	/**
	 * @return Returns the invocations.
	 */
	public Set<Invocation> getInvocations() {
		return invocations;
	}

	/**
	 * @return Returns the localVariables.
	 */
	public Set<LocalVariable> getLocalVariables() {
		return localVariables;
	}

	/**
	 * @return Returns the typeCasts.
	 */
	public Set<TypeCast> getTypeCasts() {
		return typeCasts;
	}
}
