/*
 * $Id$
 * 
 * Created on 22.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.Visitable;
import ch.toe.famix.Visitor;


/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Class extends Context implements Visitable {

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isAbstract = false;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isInterface = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isPublic = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isFinal = false;

	/**
	 * Additional FAMIX attribute, mandadory with default value
	 */
	private InheritanceDefinition inheritance = null;
	
	/**
	 * Additional FAMIX attribute, optional
	 */
	private Set<InheritanceDefinition> interfacing = new LinkedHashSet<InheritanceDefinition>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Method
	 * 
	 * @see Method
	 */
	private Set<Method> methods = new LinkedHashSet<Method>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Attribute
	 * 
	 * @see Attribute
	 */
	private Set<Attribute> attributes = new LinkedHashSet<Attribute>();

	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type ImplicitVariable
	 * 
	 * @see ImplicitVariable
	 */
	private Set<ImplicitVariable> implicitVariables = new LinkedHashSet<ImplicitVariable>();
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Class
	 * 
	 * @see Class
	 */
	private Set<Class> innerClasses = new LinkedHashSet<Class>(); 
	
	/**
	 * Additional FAMIX attribute, optional
	 * Containing objects of type Class
	 * 
	 * @see Class
	 */
	private Set<Class> anonymousClasses = new LinkedHashSet<Class>(); 

	/**
	 * @param name
	 */
	public Class(String name) {
		super(name);
		// Inheritance with assumed java.lang.Object as parent
		this.inheritance = new InheritanceDefinition(this);
	}

	/**
	 * @param name
	 * @param superclass
	 */
	public Class(String name, Class superclass) {
		super(name);
		this.inheritance = new InheritanceDefinition(this, superclass, true); 
	}
	
	/**
	 * @param name
	 * @param superclass
	 */
	public Class(String name, String superclass) {
		super(name);
		this.inheritance = new InheritanceDefinition(this, superclass, true);
	}

	/**
	 * @return true if instance has no resolved superclass, false otherwise
	 */
	public boolean hasUnresolvedSuperclass() {
		// when has unresolved superclass a Class has null in .superclass and not null in .superclassString
		if ((getInheritance().getSuperclass() == null) && (getInheritance().getSuperclassString() != null)) {
			return true;
		}
		return false;
	}
	
	/**
	 * @return true if instance has one or more unresolved interface references, false otherwise
	 */
	public boolean hasUnresolvedInterface() {
		// process all interfacing
		Iterator iter = this.getInterfacing().iterator();
		while (iter.hasNext()) {
			// when has unresolved superclass a Class has null in .superclass and not null in .superclassString
			if ((getInheritance().getSuperclass() == null) && (getInheritance().getSuperclassString() != null)) {
				return true;
			}			
		}
		return false;
	}
	
	public void accept(Visitor v) {
		v.visit(this);
		if (inheritance != null)
			inheritance.accept(v);
		Iterator<InheritanceDefinition> iterInterfacing = this.getInterfacing().iterator();
		while (iterInterfacing.hasNext())
			(iterInterfacing.next()).accept(v);
		Iterator<Class> iterClass = this.getInnerClasses().iterator();
		while (iterClass.hasNext())
			(iterClass.next()).accept(v);			
		Iterator<Class> iterAnonClass = this.getAnonymousClasses().iterator();
		while (iterAnonClass.hasNext())
			(iterAnonClass.next()).accept(v);			
		Iterator<Attribute> iterAttribute = this.getAttributes().iterator();
		while (iterAttribute.hasNext())
			(iterAttribute.next()).accept(v);
		Iterator<ImplicitVariable> iterImplicitVar = this.getImplicitVariables().iterator();
		while (iterImplicitVar.hasNext())
			(iterImplicitVar.next()).accept(v);
		Iterator<Method> iterMethod = this.getMethods().iterator();
		while (iterMethod.hasNext())
			(iterMethod.next()).accept(v);
		v.endVisit(this);
	}

	/**
	 * @return Returns the isAbstract.
	 */
	public boolean isAbstract() {
		return isAbstract;
	}
	/**
	 * @param isAbstract The isAbstract to set.
	 */
	public void setAbstract(boolean isAbstract) {
		this.isAbstract = isAbstract;
	}
	
	/**
	 * @return Returns the isFinal.
	 */
	public boolean isFinal() {
		return isFinal;
	}
	/**
	 * @param isFinal The isFinal to set.
	 */
	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}
	/**
	 * @return Returns the isInterface.
	 */
	public boolean isInterface() {
		return isInterface;
	}
	/**
	 * @param isInterface The isInterface to set.
	 */
	public void setInterface(boolean isInterface) {
		this.isInterface = isInterface;
	}
	/**
	 * @return Returns the isPublic.
	 */
	public boolean isPublic() {
		return isPublic;
	}
	/**
	 * @param isPublic The isPublic to set.
	 */
	public void setPublic(boolean isPublic) {
		this.isPublic = isPublic;
	}
	/**
	 * @return Returns the inheritance.
	 */
	public InheritanceDefinition getInheritance() {
		return inheritance;
	}
	/**
	 * @param inheritance The inheritance to set.
	 */
	public void setInheritance(InheritanceDefinition inheritance) {
		this.inheritance = inheritance;
	}

	/**
	 * @return Returns the attributes.
	 */
	public Set<Attribute> getAttributes() {
		return attributes;
	}

	/**
	 * @return Returns the implicitVariables.
	 */
	public Set<ImplicitVariable> getImplicitVariables() {
		return implicitVariables;
	}

	/**
	 * @return Returns the innerClasses.
	 */
	public Set<Class> getInnerClasses() {
		return innerClasses;
	}

	/**
	 * @return Returns the methods.
	 */
	public Set<Method> getMethods() {
		return methods;
	}

	/**
	 * @return Returns the interfacing.
	 */
	public Set<InheritanceDefinition> getInterfacing() {
		return interfacing;
	}

	/**
	 * @return Returns the anonymousClasses.
	 */
	public Set<Class> getAnonymousClasses() {
		return anonymousClasses;
	}
}
