/*
 * $Id$
 * 
 * Created on 27.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class InheritanceDefinition extends Association {

	/**
	 * Mandatory FAMIX attribute
	 */
	private Class subclass;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private Class superclass;

	/**
	 * Optional FAMIX attribute, fixed in Java model
	 */
	private static final String accessControlQualifier = "public";

	/**
	 * Optional FAMIX attribute, fixed in Java model
	 */	
	private static final Integer index = null;

	/**
	 * This is used when superclass == null; Contains the name of the superclass
	 */
	private String superclassString;
	
	/**
	 * Defines if the inheritance is of type extends or implements.
	 */
	private boolean extending;
	
	/**
	 * Final top class
	 */
	public static final Class OBJECT_CLASS = new Class("java.lang.Object", ""); 
	
	/**
	 * Defines the relation between a parent and child Class
	 * 
	 * @param subclass
	 * @param superclass
	 * @param extending true if inheritance is extending, false if inheritance is implementing
	 */
	public InheritanceDefinition(Class subclass, Class superclass, boolean extending) {
		super();
		this.subclass = subclass;
		this.superclass = superclass;
		this.extending = extending;
	}

	/**
	 * Caller does not know a superclass and only passes the superclass name.
	 * Special case is passing an empty string as superclass which then leads to
	 * a relation without parent (not even java.lang.Object!).
	 * 
	 * @param subclass
	 * @param superclass
	 * @param extending true if inheritance is extending, false if inheritance is implementing
	 */
	public InheritanceDefinition(Class subclass, String superclass, boolean extending) {
		super();
		this.subclass = subclass;
		this.superclass = null;
		// The class has no superclass
		if (superclass != null && superclass.equals(""))
			this.superclassString = null;
		else
			this.superclassString = superclass;
		this.extending = extending;
	}

	/**
	 * As caller is not passing a superclass, java.lang.Object is assumed as parent
	 *  
	 * @param subclass
	 */
	public InheritanceDefinition(Class subclass) {
		super();
		this.subclass = subclass;
		this.superclass = OBJECT_CLASS;
		this.extending = true;
	}
	
	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return (isExtending() ? "extends" : "implements")+" "+(hasUnresolvedSuperclass() ? superclassString : superclass.toString());
	}

	/**
	 * @return true if has unresolved access
	 */
	public boolean hasUnresolvedSuperclass() {
		if (superclass == null && superclassString != null)
			return true;
		return false;
	}

	/**
	 * @return Returns the accessControlQualifier.
	 */
	public String getAccessControlQualifier() {
		return accessControlQualifier;
	}
	/**
	 * @return Returns the index.
	 */
	public Integer getIndex() {
		return index;
	}
	/**
	 * @return Returns the subclass.
	 */
	public Class getSubclass() {
		return subclass;
	}
	/**
	 * @return Returns the superclass.
	 */
	public Class getSuperclass() {
		return superclass;
	}

	/**
	 * @param superclass The superclass to set.
	 */
	public void setSuperclass(Class superclass) {
		this.superclass = superclass;
	}

	/**
	 * @return Returns the superclassString.
	 */
	public String getSuperclassString() {
		return superclassString;
	}

	/**
	 * @return Returns true if inheritance is extending.
	 */
	public boolean isExtending() {
		return extending;
	}
	
	/**
	 * @return Returns true if inheritance is implementing.
	 */
	public boolean isImplementing() {
		return !isExtending();
	}
}
