/*
 * $Id$
 * 
 * Created on 22.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.Visitable;
import ch.toe.famix.Visitor;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Package extends Context implements Visitable {
	
	/**
	 * A Package contains zero or more classes
	 */
	private Set<Class> classes = new LinkedHashSet<Class>();
	
	/**
	 * A Package contains zero or more sub packages
	 */
	private Set<Package> packages = new LinkedHashSet<Package>();
	
	/**
	 * Package does not belong to another package. So called Top-Package.
	 * The name may contain dots and therefore represent multiple packages.
	 * 
	 * @param name the name of the package 
	 */
	public Package(String name) {
		super(name);
	}
	
	/**
	 * Package belongs to another package. Constructor adds this Package to
	 * the parent.<br/>
	 * The name may contain dots and therefore represent multiple packages.
	 * 
	 * @param name the name of the package 
	 * @param parent the parent package
	 */
	public Package(String name, Package parent) {
		super(name);
		if (parent != null) {
			parent.getPackages().add(this);			
			this.setBelongsTo(parent);
		}
	}

	/**
	 * Splits a given name in multiple packages by tearing it at the dots.
	 * 
	 * @param name the package name to be split
	 * @return a set of package names, returns name if a single package
	 */
	public static String[] splitName(String name) {
		return name.split("\\.");
	}
	
	/**
	 * Check if a given name is a full name (containing dots separating parent/childs) or
	 * a simple name of a package.
	 * 
	 * @param name
	 * @return true if given string is a simple name, false if not
	 */
	public static boolean isSimpleName(String name) {
		return !(name.lastIndexOf('.') > -1);		
	}
	
	public void accept(Visitor v) {
		v.visit(this);
		for (Iterator iter = this.getClasses().iterator(); iter.hasNext(); )
			((Class)iter.next()).accept(v);
		for (Iterator iter = this.getPackages().iterator() ; iter.hasNext(); )
			((Package)iter.next()).accept(v);
		v.endVisit(this);
	}

	/**
	 * @return Returns the classes.
	 */
	public Set<Class> getClasses() {
		return classes;
	}

	/**
	 * @return Returns the packages.
	 */
	public Set<Package> getPackages() {
		return packages;
	}
}
