/*
 * $Id$
 * 
 * Created on 17.08.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model.container;

import org.apache.log4j.Logger;

import ch.toe.famix.model.Access;
import ch.toe.famix.model.BehaviouralEntity;
import ch.toe.famix.model.InheritanceDefinition;
import ch.toe.famix.model.Invocation;
import ch.toe.famix.model.Object;
import ch.toe.famix.model.Class;
import ch.toe.famix.model.StructuralEntity;

/**
 * A Claim has a <i>claimer</i> (the object who is looking for a object) and a <i>claimed</i> resource (the object the
 * claimer is looking for).
 * 
 * <p>This is used to defer lookups for not-yet-in-the-model-objects. As soon as a new object is added to the FAMIXInstance,
 * the claim collection is checked for existing claims for that object and references are updated to it. 
 * 
 * @author tsa
 */
public class Claim {

	// logging device
	private static Logger logger = Logger.getLogger(Claim.class);

	/**
	 * Contains the name of the claimed object 
	 */
	private String claimed;
	
	/**
	 * Contains the claiming Object
	 */
	private Object claimer;
	
	/**
	 * @param claimer the object that claims a ressource
	 * @param claimed name of the object the claimer is looking for
	 * @throws InvalidClaimException 
	 */
	public Claim(Object claimer, String claimed) throws InvalidClaimException {
		if (claimer != null && claimedIsInValidPackage(claimed)) {
			logger.debug("Add claim for "+claimed);
			this.claimer = claimer;				
			this.claimed = claimed;			
		}
		else {
			throw new InvalidClaimException();
		}
	}

	/**
	 * Applies a filter for possible claimed resources.
	 * At the moment this excludes java.* and simple types.
	 * 
	 * @return true if claimed does not match the exclude parse filter
	 */
	private boolean claimedIsInValidPackage(String claimed) {
		// exclude simple types such as int, byte, etc.
		boolean simpleName = !(claimed.lastIndexOf('.') > -1);
		if (simpleName)
			return false;
		// exclude java.*;
		if (claimed.startsWith("java."))
			return false;
		return true;
	}
	
	/**
	 * Fulfils the given claim by adding the object to the claimer.
	 * 
	 * @param claim Claim containing information about claimer/claimed
	 * @param object 
	 * @return true if the operation was successful
	 */
	public static boolean fulfilClaim(Claim claim, Object object) {
		Object claimer = claim.getClaimer();
		// BehaviouralEntity.declaredReturnClass
		if (BehaviouralEntity.class.isInstance(claimer)) {
			if (Class.class.isInstance(object))
				((BehaviouralEntity)claimer).setDeclaredReturnClass((Class)object);		
		}
		// StructuralEntity.declaredClass
		else if (StructuralEntity.class.isInstance(claimer)) {
			if (Class.class.isInstance(object))
				((StructuralEntity)claimer).setDeclaredClass((Class)object);		
		}
		// InheritanceDefinition.superclass
		else if (InheritanceDefinition.class.isInstance(claimer)) {
			if (Class.class.isInstance(object))
				((InheritanceDefinition)claimer).setSuperclass((Class)object);						
		}
		// Access.accesses
		else if (Access.class.isInstance(claimer)) {
			if (StructuralEntity.class.isInstance(object))
				((Access)claimer).setAccesses((StructuralEntity)object);
		}
		// Invocation.invokes
		else if (Invocation.class.isInstance(claimer)) {
			if (BehaviouralEntity.class.isInstance(object))
				((Invocation)claimer).setInvokes((BehaviouralEntity)object);
		}
		// no match
		else {
			return false;			
		}

		// claim was fulfilled successfully
		return true;
	}
	
	/**
	 * @return Returns the claimed.
	 */
	public String getClaimed() {
		return claimed;
	}

	/**
	 * @return Returns the claimer.
	 */
	public Object getClaimer() {
		return claimer;
	}
	
	/**
	 * @return Name of claimer. Null if no name could be looked up.
	 */
	public String getClaimerName() {
		// BehaviouralEntity.declaredReturnClass
		if (BehaviouralEntity.class.isInstance(claimer)) {
			return ((BehaviouralEntity)claimer).getUniqueName();
		}
		// StructuralEntity.declaredClass
		else if (StructuralEntity.class.isInstance(claimer)) {
			return ((StructuralEntity)claimer).getUniqueName();
		}
		// InheritanceDefinition.superclass
		else if (InheritanceDefinition.class.isInstance(claimer)) {
			return ((InheritanceDefinition)claimer).getSubclass().getUniqueName();
		}
		// Access.accesses
		else if (Access.class.isInstance(claimer)) {
			return ((Access)claimer).getAccessedIn().getUniqueName();
		}
		// Invocation.invokes
		else if (Invocation.class.isInstance(claimer)) {
			return ((Invocation)claimer).getInvokedBy().getUniqueName();
		}
		// no match
		return null;
	}
}
