/*
 * $Id$
 * 
 * Created on 15.08.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model.container;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import org.apache.log4j.Logger;

import ch.toe.famix.model.Class;
import ch.toe.famix.model.InheritanceDefinition;

/**
 * This defines a container for Classes.
 * 
 * @author tsa
 */
public class ClassContainer {

	// logging device
	private static Logger logger = Logger.getLogger(ClassContainer.class);

	/**
	 * Container; child elements are of type Class
	 */
	private Set<Class> container = new LinkedHashSet<Class>();
	
	/**
	 * Contains claims for not yet existing classes
	 */
	private Set<Claim> claims = new LinkedHashSet<Claim>();
	
	/**
	 * Finds the Class with the specified name in the container. Checks uniqueName and simple name.	
	 * 
	 * @param name
	 * @return the Class looking for or null if no such Class was found
	 */
	public Class findByName(String name) {
		Class ret = null;
		// if name contains dots, it's a uniqueName
		boolean simpleName = !(name.lastIndexOf('.') > -1);
		Iterator iter = this.getContainer().iterator();
		while (iter.hasNext()) {
			Class myclass = (Class)iter.next();
			if (!simpleName) {
				if (myclass.getUniqueName().equals(name)) {
					ret = myclass;
					break;
				}				
			}
			else {
				if (myclass.getName().equals(name)) {
					ret = myclass;
					break;
				}				
			}
		}						
		return ret;
	}

	/**
	 * Finds all claimers that claim the given Class resource. It compares name and uniqueName.
	 * 
	 * @param myclass
	 * @return an iterator containing Claim objects that matched the Class name
	 */
	private Iterator findClaimers(Class myclass) {
		Set<Claim> claims = new LinkedHashSet<Claim>();
		Iterator iter = this.getClaims().iterator();
		while (iter.hasNext()) {
			Claim claim = (Claim)iter.next();
			// check uniqueName and name
			if (claim.getClaimed().equals(myclass.getName()) 
					|| claim.getClaimed().equals(myclass.getUniqueName()))
				claims.add(claim);				
		}
		return claims.iterator();
	}
	
	/**
	 * Adds a class to the collection. Checks for possible claims and adds the Class as
	 * reference to those.
	 * 
	 * @param myclass
	 */
	public boolean add(Class myclass) {
		if (!container.add(myclass)) {
			return false;
		}
		// find possible claims of myclass itself
		if (myclass.hasUnresolvedSuperclass()) {
			String claimed = myclass.getInheritance().getSuperclassString();
			addClaim(myclass, claimed);
		}
		if (myclass.hasUnresolvedInterface()) {
			Iterator iter = myclass.getInterfacing().iterator();
			while (iter.hasNext()) {
				String claimed = ((InheritanceDefinition)iter.next()).getSuperclassString();
				addClaim(myclass, claimed);					
			}
		}
		// process all existing claimers and add myclass as reference if requested
		Iterator iter = findClaimers(myclass);
		while (iter.hasNext()) {
			logger.debug("Looking for claimers of "+myclass.getUniqueName());
			Claim claim = (Claim)iter.next();
			// remove the claim from my claim container if I can add myclass to the claimer
			if (Claim.fulfilClaim(claim, myclass))
				this.getClaims().remove(claim);
		}
		return true;
	}

	/**
	 * Adds a new Claim on myclass to the claimed resource
	 * 
	 * @param myclass claiming class
	 * @param claimed claimed resource name
	 */
	private void addClaim(Class myclass, String claimed) {
		if (claimed != null && !claimed.equals("")) {
			logger.debug("Adding new Claim for class "+claimed+" (claimed by "+myclass.getUniqueName()+")");
			try {
				Claim claim = new Claim(myclass, claimed);
				this.getClaims().add(claim);					
			}
			catch (InvalidClaimException ice) {
				// ignore claim, then.
			}
		}
	}
	
	/**
	 * @return Returns the container.
	 */
	public Set<Class> getContainer() {
		return container;
	}

	/**
	 * @return Returns the claims.
	 */
	public Set<Claim> getClaims() {
		return claims;
	}
}
