// The master class that holds the set of tuples in a
// decomposition

package ch.unibern.graphmatching2;

import java.util.*;

public class Decomposition {
	Vector m_tupleList;

	public Decomposition() {
		m_tupleList = new Vector(10, 10);
	}

	// Takes input graph and decomposes it using Messmer's algorithm
	// Adds new tuples to m_tupleList
	public Graph decompose(Graph graph) {
		Graph Smax = matchTarget(graph);
		if (Smax == null) {
			addTupleSmaxNull(graph);
		} else if (Smax.getNumberOfVertices() == graph.getNumberOfVertices()) {

			// Smax is isomorphic to graph - don't add anything to decomp
			// but we need to replace graph by Smax in the new tuple, so
			// return Smax so it can be replaced in the calling method

			return Smax;
		} else {
			addTupleSmaxValid(graph, Smax);
		}
		// returning null means that 'graph' isn't a parent in an existing
		// tuple, so it's OK to add it
		return null;
	}

	private void addTupleSmaxNull(Graph graph) {
		int graphNumVertices = graph.getNumberOfVertices();
		if (graphNumVertices == 1) {
			Tuple newTuple = new Tuple(graph, null, null, null, null, null,
					null);
			m_tupleList.add(newTuple);
			return;
		}

		// Split graph into 2 equal subgraphs by taking the first
		// numVertices / 2 vertices and putting them into the first subgraph,
		// and the remainder into the second.

		Graph subGraph1 = new Graph();
		Graph subGraph2 = new Graph();
		Enumeration graphVertexEnum = graph.getVertexList().elements();
		for (int vert = 0; vert < graphNumVertices / 2; ++vert) {
			subGraph1.addVertex((Vertex) graphVertexEnum.nextElement());
		}

		for (int vert = graphNumVertices / 2; vert < graphNumVertices; ++vert) {
			subGraph2.addVertex((Vertex) graphVertexEnum.nextElement());
		}

		// Now duplicate the vertices in each subgraph and construct
		// a HashMap mapping the new vertices onto the old ones in the parent
		// graph

		HashMap subHashMap1 = mapVertices(subGraph1);
		HashMap subHashMap2 = mapVertices(subGraph2);

		// Now we need to copy the edges over to the new vertices, and
		// create the edge sets for edges joining the two subgraphs

		// Edges from subGraph1 to subGraph 2
		Vector edgeSet1_2 = new Vector(10, 10);
		buildEdgeSet(subGraph1, subGraph2, edgeSet1_2);

		// Edges from subGraph2 to subGraph 1
		Vector edgeSet2_1 = new Vector(10, 10);
		buildEdgeSet(subGraph2, subGraph1, edgeSet2_1);

		removeParentVertices(subGraph1);
		removeParentVertices(subGraph2);

		Tuple newTuple;
		decompose(subGraph1);
		Graph isomorph = decompose(subGraph2);
		// If subGraph2 is isomorphic to subGraph1, only subGraph1 will
		// be entered as a tuple parent, so we must replace subGraph2 by
		// subGraph1 in the new tuple being created. This requires the
		// hash map from subGraph1 to subGraph2
		if (isomorph != null) {
			newTuple = replaceIsomorph(isomorph, subHashMap1, subHashMap2,
					edgeSet1_2, edgeSet2_1, subGraph1, graph);
		} else {
			newTuple = new Tuple(graph, subGraph1, subGraph2, edgeSet1_2,
					edgeSet2_1, subHashMap1, subHashMap2);
		}
		m_tupleList.add(newTuple);
	}

	// Assumes that isomorph is a graph already present as a parent of
	// a tuple, that it is isomorphic to subGraph2,
	// and that it therefore is to replace subGraph2. This requires
	// modifying subHashMap2 and the two edge sets to replace all
	// references to subGraph2 vertices by the corresponding vertices
	// in isomorph.
	private Tuple replaceIsomorph(Graph isomorph, HashMap subHashMap1,
			HashMap subHashMap2, Vector edgeSet1_2, Vector edgeSet2_1,
			Graph subGraph1, Graph graph) {
		HashMap newMap2 = new HashMap();
		Iterator iter = isomorph.getFirstTargetMap().entrySet().iterator();
		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			Vertex key1 = (Vertex) mapEntry.getKey();
			Vertex value1 = (Vertex) mapEntry.getValue();
			Vertex parentVertex = (Vertex) subHashMap2.get(value1);
			newMap2.put(key1, parentVertex);
		}

		HashMap isoMap = isomorph.getFirstTargetMap();
		Enumeration edgeList = edgeSet1_2.elements();
		while (edgeList.hasMoreElements()) {
			Edge edge = (Edge) edgeList.nextElement();
			Vertex dest = edge.getDestVertex();
			Vertex newDest = findKeyVertex(isoMap, dest);
			edge.setDestVertex(newDest);
		}
		edgeList = edgeSet2_1.elements();
		while (edgeList.hasMoreElements()) {
			Edge edge = (Edge) edgeList.nextElement();
			Vertex source = edge.getSourceVertex();
			Vertex newSource = findKeyVertex(isoMap, source);
			edge.setSourceVertex(newSource);
		}

		return new Tuple(graph, subGraph1, isomorph, edgeSet1_2, edgeSet2_1,
				subHashMap1, newMap2);
	}

	private void addTupleSmaxValid(Graph graph, Graph Smax) {
		// Smax will be one of the subgraphs in the new tuple
		// The other will be (graph - Smax), so we must build this graph
		// Smax may contain more than one map to graph, so we just take
		// the first one:
		HashMap SmaxTargetMap = Smax.getFirstTargetMap();

		// Now we iterate through the vertices in graph - those vertices
		// NOT appearing as values in SmaxTargetMap belong in
		// (graph - Smax)
		Graph targetMinusSmax = new Graph();
		Enumeration targetVertices = graph.getVertexList().elements();
		while (targetVertices.hasMoreElements()) {
			Vertex targetVertex = (Vertex) targetVertices.nextElement();
			if (!SmaxTargetMap.containsValue(targetVertex)) {
				targetMinusSmax.addVertex(targetVertex);
			}
		}

		// Duplicate vertices and construct hash map for (graph - Smax):
		HashMap targetMinusSmaxMap = mapVertices(targetMinusSmax);

		// Build edge lists - can't use same technique as when Smax is null,
		// since Smax already a complete graph, while other graph is being built
		Vector edgeSet1_2 = new Vector(10, 10);
		Vector edgeSet2_1 = new Vector(10, 10);
		buildSmaxEdgeSets(SmaxTargetMap, targetMinusSmax, edgeSet1_2,
				edgeSet2_1);

		removeParentVertices(targetMinusSmax);

		// Should check targetMinusSmax here to see if it too is already
		// present as a parent in a tuple. Since Smax has the largest number
		// of vertices of any subgraph present in target, if targetMinusSmax
		// is in the decomp, it must have <= that number of vertices
		Tuple newTuple;
		if (targetMinusSmax.getNumberOfVertices() <= Smax.getNumberOfVertices()) {
			Graph minusMax = matchTarget(targetMinusSmax);

			// If the number of vertices are the same,
			// minusMax is isomorphic to targetMinusSmax
			if (minusMax != null
					&& minusMax.getNumberOfVertices() == targetMinusSmax
							.getNumberOfVertices()) {
				newTuple = replaceIsomorph(minusMax, SmaxTargetMap,
						targetMinusSmaxMap, edgeSet1_2, edgeSet2_1, Smax, graph);
				m_tupleList.add(newTuple);
				return;
			}
		}

		newTuple = new Tuple(graph, Smax, targetMinusSmax, edgeSet1_2,
				edgeSet2_1, SmaxTargetMap, targetMinusSmaxMap);
		decompose(targetMinusSmax);
		m_tupleList.add(newTuple);
	}

	private HashMap mapVertices(Graph subGraph) {
		HashMap subHashMap = new HashMap();
		Vector subVertexList = subGraph.getVertexList();
		int subNumVertices = subGraph.getNumberOfVertices();
		for (int vert = 0; vert < subNumVertices; ++vert) {
			Vertex oldVertex = (Vertex) subVertexList.get(vert);
			Vertex newVertex = new Vertex(oldVertex.getLabel());
			subGraph.addVertex(newVertex);
			subHashMap.put(newVertex, oldVertex);
		}
		return subHashMap;
	}

	private void buildSmaxEdgeSets(HashMap SmaxMap, Graph targetMinusSmax,
			Vector edgeSet1_2, Vector edgeSet2_1) {
		Iterator iter = SmaxMap.entrySet().iterator();
		Vector otherVertexList = targetMinusSmax.getVertexList();
		int otherNumVertex = targetMinusSmax.getNumberOfVertices() / 2;

		// Edges from Smax to targetMinusSmax:
		//
		// For each vertex in Smax, find the vertex in the target that it
		// maps to, and get the edge list for that vertex
		// Then any edges that do not lead to vertices within Smax must lead
		// to vertices in (target - Smax)
		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			Vertex SmaxVertex = (Vertex) mapEntry.getKey();
			Vertex targetVertex = (Vertex) mapEntry.getValue();
			Enumeration edgeEnum = targetVertex.getEdgeList().elements();
			while (edgeEnum.hasMoreElements()) {
				Edge currEdge = (Edge) edgeEnum.nextElement();
				Vertex destVertex = currEdge.getDestVertex();
				if (!SmaxMap.containsValue(destVertex)) {
					Vertex newDestVertex = (Vertex) otherVertexList
							.get(otherVertexList.indexOf(destVertex)
									+ otherNumVertex);
					Edge newEdge = new Edge(SmaxVertex, newDestVertex);
					edgeSet1_2.add(newEdge);
				}
			}
		}

		// Edges within targetMinusSmax and to Smax
		//
		// Similar to above, except we must also build the edges within
		// targetMinusSmax
		for (int vert = 0; vert < otherNumVertex; vert++) {
			Vertex currVertex = (Vertex) otherVertexList.get(vert);
			Enumeration edgeEnum = currVertex.getEdgeList().elements();
			while (edgeEnum.hasMoreElements()) {
				Edge currEdge = (Edge) edgeEnum.nextElement();
				Vertex destVertex = currEdge.getDestVertex();
				Vertex newCurrVertex = (Vertex) otherVertexList.get(vert
						+ otherNumVertex);
				Vertex newDestVertex;

				// Destination vertex within targetMinusSmax
				if (otherVertexList.contains(destVertex)) {
					newDestVertex = (Vertex) otherVertexList
							.get(otherVertexList.indexOf(destVertex)
									+ otherNumVertex);
					newCurrVertex.addEdge(newDestVertex);
				} else { // Destination within Smax
					newDestVertex = findKeyVertex(SmaxMap, destVertex);
					Edge newEdge = new Edge(newCurrVertex, newDestVertex);
					edgeSet2_1.add(newEdge);
				}
			}
		}
	}

	private Vertex findKeyVertex(HashMap map, Vertex value) {
		Iterator iter = map.entrySet().iterator();
		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			if (value == (Vertex) mapEntry.getValue())
				return (Vertex) mapEntry.getKey();
		}
		return null;
	}

	private void buildEdgeSet(Graph graphA, Graph graphB, Vector edgeSet) {
		Vector graphAVertexList = graphA.getVertexList();
		Vector graphBVertexList = graphB.getVertexList();

		// Take half the number of vertices since we duplicated the
		// vertices earlier
		int subNumVertex = graphA.getNumberOfVertices() / 2;
		int otherNumVertex = graphB.getNumberOfVertices() / 2;
		for (int vert = 0; vert < subNumVertex; ++vert) {
			Vertex currVertex = (Vertex) graphAVertexList.get(vert);
			Enumeration edgeEnum = currVertex.getEdgeList().elements();
			while (edgeEnum.hasMoreElements()) {
				Edge currEdge = (Edge) edgeEnum.nextElement();
				Vertex destVertex = currEdge.getDestVertex();
				Vertex newCurrVertex = (Vertex) graphAVertexList.get(vert
						+ subNumVertex);
				Vertex newDestVertex;
				if (graphAVertexList.contains(destVertex)) {
					newDestVertex = (Vertex) graphAVertexList
							.get(graphAVertexList.indexOf(destVertex)
									+ subNumVertex);
					newCurrVertex.addEdge(newDestVertex);
				} else {
					newDestVertex = (Vertex) graphBVertexList
							.get(graphBVertexList.indexOf(destVertex)
									+ otherNumVertex);
					Edge newEdge = new Edge(newCurrVertex, newDestVertex);
					edgeSet.add(newEdge);
				}
			}
		}
	}

	// Removes the vertices copied from the parent graph
	private void removeParentVertices(Graph graph) {
		// Since remove() shifts remaining indices over, just remove
		// index 0 num/2 times

		int numVertices = graph.getNumberOfVertices() / 2;
		for (int vert = 0; vert < numVertices; ++vert) {
			graph.getVertexList().remove(0);
		}
	}

	// matchTarget() will find the largest graph in the decomposition
	// that matches a subgraph of target.
	public Graph matchTarget(Graph target) {
		// Mark all tuple parents as DORMANT, except for single-vertex graphs,
		// which are marked as ACTIVE
		Enumeration tupleList = m_tupleList.elements();
		while (tupleList.hasMoreElements()) {
			Graph tupleParent = (Graph) ((Tuple) tupleList.nextElement())
					.getParent();
			if (tupleParent.getNumberOfVertices() == 1) {
				tupleParent.setState(buildVertexTargetMap(tupleParent, target));
			} else {
				tupleParent.setState(Graph.DORMANT);
			}
		}

		// We have now initialized all single vertices, so now we need to
		// attempt to join these into larger subgraphs. We do this by
		// looping over the set of tuples looking for ACTIVE tuples,
		// which are tuples both of whose subgraphs are ALIVE. If either
		// of a tuple's subgraphs is DEAD, the parent graph for that
		// tuple should be marked as DEAD as well.
		// The looping continues until all tuples have been labelled as either
		// ALIVE or DEAD.

		while (mergeGraphs(target))
			;

		// At this point, all tuples in the decomposition have been
		// processed. We can find the largest subgraph (or one of the
		// largest, if there are several largest subgraphs) by examining
		// all ALIVE tuples and finding the one with the largest parent graph

		Graph Smax = null;
		int SmaxSize = 0;
		tupleList = m_tupleList.elements();
		while (tupleList.hasMoreElements()) {
			Graph parent = ((Tuple) tupleList.nextElement()).getParent();
			if (parent.getState() == Graph.ALIVE
					&& parent.getNumberOfVertices() > SmaxSize) {
				SmaxSize = parent.getNumberOfVertices();
				Smax = parent;
			}
		}
		return Smax;
	}

	// This method assumes that vertexGraph contains 1 vertex.
	// It adds a target map entry from vertexGraph to each vertex in target
	// whose vertex label matches that of the single vertex in vertexGraph
	// It returns Graph.ALIVE if any matches were found, and Graph.DEAD
	// otherwise. This is the initial step in finding the largest graph
	// in a decomposition that matches a subgraph in target
	// It is also used in building the decomposition
	private int buildVertexTargetMap(Graph vertexGraph, Graph target) {
		Enumeration targetVertexList = target.getVertexList().elements();
		Enumeration singleVertexList = vertexGraph.getVertexList().elements();
		Vertex singleVertex = (Vertex) singleVertexList.nextElement();

		// Clear all previous maps for the source graph
		vertexGraph.getTargetMap().clear();

		// Count the number of maps created
		int numMaps = 0;

		while (targetVertexList.hasMoreElements()) {
			Vertex targetVertex = (Vertex) targetVertexList.nextElement();
			if (targetVertex.getLabel().toString().equals(
					singleVertex.getLabel().toString())) {
				HashMap singleMap = new HashMap();
				singleMap.put(singleVertex, targetVertex);
				vertexGraph.addTargetMap(singleMap);
				numMaps++;
			}
		}
		if (numMaps > 0)
			return Graph.ALIVE;
		return Graph.DEAD;
	}

	public String printTargetMaps() {
		String result = "";
		Enumeration tupleList = m_tupleList.elements();
		while (tupleList.hasMoreElements()) {
			Graph tupleParent = (Graph) ((Tuple) tupleList.nextElement())
					.getParent();
			result += tupleParent.printTargetMap();
		}
		return result;
	}

	// mergeGraphs() assumes that all single-vertex tuples have
	// been processed and are labelled as either ALIVE or DEAD.
	// It iterates through the list of tuples searching for tuples
	// with a DORMANT parent. If both subgraphs in this tuple
	// are ALIVE, then the parent is marked as ACTIVE, and an attempt
	// is made to match it to the target. If this attempt fails,
	// the parent is marked as DEAD, otherwise it is marked as ALIVE.
	// If either subgraph is DEAD, the parent is marked as DEAD as well.
	// Otherwise, the tuple is left unchanged.
	// 
	// The method returns true if any parent remains DORMANT at the
	// end of the method, indicating that further processing of the
	// decomposition set is needed. If all tuples have been considered,
	// all parents will be marked either ALIVE or DEAD, and the
	// algorithm is finished.

	private boolean mergeGraphs(Graph target) {

		int dormantCount = 0;
		Enumeration tupleList = m_tupleList.elements();
		while (tupleList.hasMoreElements()) {
			Tuple tuple = (Tuple) tupleList.nextElement();
			Graph tupleGraph1 = tuple.getSubgraph(1);
			Graph tupleGraph2 = tuple.getSubgraph(2);
			Graph tupleParent = tuple.getParent();
			if (tupleGraph1 == null || tupleGraph2 == null) {
				continue;
			}
			if (tupleGraph1.getState() == Graph.DEAD
					|| tupleGraph2.getState() == Graph.DEAD) {
				tupleParent.setState(Graph.DEAD);
			} else if (tupleGraph1.getState() == Graph.ALIVE
					&& tupleGraph2.getState() == Graph.ALIVE) {
				// Attempt to match subgraphs to target
				matchTuple(tuple, target);
			} else {
				dormantCount++;
			}
		}
		if (dormantCount > 0)
			return true;
		return false;
	}

	// matchTuple() assumes that both subgraphs in tuple are ALIVE
	// and attempts to find a match from the tuple's parent to a subgraph
	// of target. For each match found, a target map is added to the
	// parent graph. If no matches are found, method returns false,
	// otherwise true.
	//
	// The matching process must do:
	// (1) Check that the two subgraphs are disjoint in the target. If not,
	// quit, otherwise proceed with...
	// (2) For all possible combinations of elements in the target maps
	// of the two subgraphs, see if the proper edge sets are present
	// in the target.
	// (3) For each pairing of maps that produces correct edge sets, add a
	// target map to the parent graph.

	private boolean matchTuple(Tuple tuple, Graph target) {
		Graph tupleGraph1 = tuple.getSubgraph(1);
		Graph tupleGraph2 = tuple.getSubgraph(2);
		Vector targetMapList1 = tupleGraph1.getTargetMap();
		Vector targetMapList2 = tupleGraph2.getTargetMap();
		tuple.getParent().getTargetMap().clear();

		Enumeration targetMapEnum1 = targetMapList1.elements();
		while (targetMapEnum1.hasMoreElements()) {
			HashMap map1 = (HashMap) targetMapEnum1.nextElement();
			Enumeration targetMapEnum2 = targetMapList2.elements();
			while (targetMapEnum2.hasMoreElements()) {
				HashMap map2 = (HashMap) targetMapEnum2.nextElement();
				if (!disjointTarget(map1, map2)) {
					continue;
				} else {
					// Now do edge set checks.
					// Build target edge sets
					Vector targetEdges1_2 = new Vector(5, 5);
					Vector targetEdges2_1 = new Vector(5, 5);
					buildTargetEdgeSets(map1, map2, target, targetEdges1_2);
					buildTargetEdgeSets(map2, map1, target, targetEdges2_1);

					// Compare target edge sets with subgraph edge sets
					Vector modelEdges1_2 = tuple.getEdgeSet(1);
					if (!compareEdgeSets(map1, map2, targetEdges1_2,
							modelEdges1_2))
						continue;
					Vector modelEdges2_1 = tuple.getEdgeSet(2);
					if (!compareEdgeSets(map2, map1, targetEdges2_1,
							modelEdges2_1))
						continue;

					// If match found, add target map to parent graph
					HashMap parentMap = new HashMap();
					buildParentToTargetMap(tuple, map1, 1, parentMap);
					buildParentToTargetMap(tuple, map2, 2, parentMap);
					tuple.getParent().addTargetMap(parentMap);
					tuple.getParent().setState(Graph.ALIVE);
				}
			}
		}
		if (tuple.getParent().getState() == Graph.ALIVE) {
			return true;
		}
		tuple.getParent().setState(Graph.DEAD);
		return false;
	}

	private void buildParentToTargetMap(Tuple tuple, HashMap map,
			int subGraphNum, HashMap parentMap) {
		HashMap modelMap = tuple.getHashMap(subGraphNum);
		Iterator iter = map.entrySet().iterator();

		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			Vertex subVertex = (Vertex) mapEntry.getKey();
			Vertex parentVertex = (Vertex) modelMap.get(subVertex);
			Vertex targetVertex = (Vertex) mapEntry.getValue();
			parentMap.put(parentVertex, targetVertex);
		}
	}

	// Compares edges in the two edge sets to see if they match.
	// First, sizes of both sets must be equal
	// If so, make sure each edge in modelEdges has a matching edge
	// in targetEdges
	private boolean compareEdgeSets(HashMap map1, HashMap map2,
			Vector targetEdges, Vector modelEdges) {
		if (targetEdges.size() != modelEdges.size()) {
			return false;
		}
		Enumeration modelList = modelEdges.elements();
		while (modelList.hasMoreElements()) {
			Edge modelEdge = (Edge) modelList.nextElement();
			Vertex modelSource = modelEdge.getSourceVertex();
			Vertex modelDest = modelEdge.getDestVertex();
			Vertex targetSource = (Vertex) map1.get(modelSource);
			Vertex targetDest = (Vertex) map2.get(modelDest);
			Enumeration targetList = targetEdges.elements();
			boolean foundEdge = false;
			while (targetList.hasMoreElements()) {
				Edge targetEdge = (Edge) targetList.nextElement();
				if (targetSource == targetEdge.getSourceVertex()
						&& targetDest == targetEdge.getDestVertex()) {
					// May also want to compare edge label (e.g. cost)

					foundEdge = true;
					break;
				}
			}
			if (!foundEdge) {
				return false;
			}
		}
		return true;
	}

	// Build target edge sets. Algorithm:
	// For each key vertex in mapA, find value vertex - this is a vertex in
	// the target graph. Iterate through edge list for this target vertex.
	// For each edge, find out where its destination vertex lies.
	// If destination vertex is:
	// (1) A value in mapA, ignore it, since the edge doesn't go outside
	// the mapping of subgraphA to the target
	// (2) A value in mapB, add the edge to targetEdges, since this is an
	// edge that joins the two subgraphs
	// (3) Neither in mapA or mapB, ignore it
	private void buildTargetEdgeSets(HashMap mapA, HashMap mapB, Graph target,
			Vector targetEdges) {
		Iterator iterMapA = mapA.entrySet().iterator();
		while (iterMapA.hasNext()) {
			Vertex targetDest = (Vertex) ((Map.Entry) iterMapA.next())
					.getValue();
			Enumeration targetEdgeList = targetDest.getEdgeList().elements();
			while (targetEdgeList.hasMoreElements()) {
				Edge targetEdge = (Edge) targetEdgeList.nextElement();
				Vertex destVertex = targetEdge.getDestVertex();
				if (mapB.containsValue(destVertex)) {
					targetEdges.add(targetEdge);
				}
			}
		}
	}

	// Checks if the vertex sets in the target that are mapped to
	// by set1 and set2 respectively are in disjoint sets
	private boolean disjointTarget(HashMap set1, HashMap set2) {
		Set entrySet1 = set1.entrySet();
		Iterator iterSet1 = entrySet1.iterator();
		while (iterSet1.hasNext()) {
			Map.Entry mapEntry1 = (Map.Entry) iterSet1.next();
			if (set2.containsValue(mapEntry1.getValue()))
				return false;
		}
		return true;
	}

	public String printEdgeSet(Vector set) {
		String result = "";
		Enumeration setList = set.elements();
		while (setList.hasMoreElements()) {
			result += ((Edge) setList.nextElement()).printEdge() + "\n";
		}
		return result;
	}

	public String printHashMap(HashMap map) {
		String result = "";
		Iterator iter = map.entrySet().iterator();
		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			result += ((Vertex) mapEntry.getKey()).getLabel().toString();
			result += " ==> "
					+ ((Vertex) mapEntry.getValue()).getLabel().toString()
					+ "(" + mapEntry.getKey() + " => " + mapEntry.getValue()
					+ ")\n";
		}
		return result;
	}

	public String printTuples() {
		String result = "+++++ Tuples currently in decomposition: +++++\n";
		Enumeration tupleList = m_tupleList.elements();
		result += "Number of tuples: " + m_tupleList.size() + "\n";
		int tupleNum = 1;
		while (tupleList.hasMoreElements()) {
			Tuple tuple = (Tuple) tupleList.nextElement();
			result += "Tuple #" + tupleNum++ + "\nParent:\n";
			result += tuple.getParent().printEdgeTable() + "\n";
			if (tuple.getParent().getNumberOfVertices() > 1) {
				result += "Subgraph 1:\n";
				result += tuple.getSubgraph(1).printEdgeTable()
						+ "\nSubgraph 2:\n";
				result += tuple.getSubgraph(2).printEdgeTable();
			}
			result += "\n=========\n";
		}
		result += "----------------------------------\n";
		return result;
	}
}
