package com.wcohen.ss;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.wcohen.ss.api.StringWrapper;
import com.wcohen.ss.api.StringWrapperIterator;
import com.wcohen.ss.api.Token;
import com.wcohen.ss.api.Tokenizer;
import com.wcohen.ss.tokens.NGramTokenizer;
import com.wcohen.ss.tokens.SimpleTokenizer;

/**
 * TFIDF-based distance metric.
 */

public class TFIDF extends AbstractStatisticalTokenDistance {
	public TFIDF(Tokenizer tokenizer) {
		super(tokenizer);
	}

	public TFIDF() {
		super();
	}

	public double score(StringWrapper s, StringWrapper t) {
		checkTrainingHasHappened(s, t);
		UnitVector sBag = asUnitVector(s);
		UnitVector tBag = asUnitVector(t);
		double sim = 0.0;
		for (Iterator i = sBag.tokenIterator(); i.hasNext();) {
			Token tok = (Token) i.next();
			if (tBag.contains(tok)) {
				sim += sBag.getWeight(tok) * tBag.getWeight(tok);
				// System.out.println("Common tokens: " + tok.getValue()
				// + ":\t\t " + sBag.getWeight(tok) + " "
				// + tBag.getWeight(tok));
			}
		}
		// System.out.println("common="+numCommon+" |s| = "+sBag.size()+" |t| =
		// "+tBag.size());
		return sim;
	}

	protected UnitVector asUnitVector(StringWrapper w) {
		if (w instanceof UnitVector) {
			return (UnitVector) w;
		} else if (w instanceof BagOfTokens) {
			return new UnitVector((BagOfTokens) w);
		} else {
			return new UnitVector(w.unwrap(), tokenizer.tokenize(w.unwrap()));
		}
	}

	/** Preprocess a string by finding tokens and giving them TFIDF weights */
	public StringWrapper prepare(String s) {
		return new UnitVector(s, tokenizer.tokenize(s));
	}

	/** Marker class extending BagOfTokens */
	protected class UnitVector extends BagOfTokens {
		public UnitVector(String s, Token[] tokens) {
			super(s, tokens);
			termFreq2TFIDF();
		}

		public UnitVector(BagOfTokens bag) {
			this(bag.unwrap(), bag.getTokens());
			termFreq2TFIDF();
		}

		/** convert term frequency weights to unit-length TFIDF weights */
		private void termFreq2TFIDF() {
			double normalizer = 0.0;
			for (Iterator i = tokenIterator(); i.hasNext();) {
				Token tok = (Token) i.next();
				if (collectionSize > 0) {
					Integer dfInteger = (Integer) documentFrequency.get(tok);
					// set previously unknown words to df==1, which gives them a
					// high value
					double df = dfInteger == null ? 1.0 : dfInteger.intValue();
					double localWeight = Math.log(getWeight(tok) + 1);
					double globalWeight = Math.log(collectionSize / df);
					double w = localWeight * globalWeight;
					setWeight(tok, w);
					// System.out.println(tok.getValue() + ":\t\t " +
					// localWeight
					// + " " + globalWeight + " " + w);
					normalizer += w * w;
				} else {
					setWeight(tok, 1.0);
					normalizer += 1.0;
				}
			}
			normalizer = Math.sqrt(normalizer);
			// System.out.println("normalizer: " + normalizer);
			for (Iterator i = tokenIterator(); i.hasNext();) {
				Token tok = (Token) i.next();
				setWeight(tok, getWeight(tok) / normalizer);
			}
		}
	}

	/**
	 * Explain how the distance was computed. In the output, the tokens in S and
	 * T are listed, and the common tokens are marked with an asterisk.
	 */
	public String explainScore(StringWrapper s, StringWrapper t) {
		BagOfTokens sBag = (BagOfTokens) s;
		BagOfTokens tBag = (BagOfTokens) t;
		StringBuffer buf = new StringBuffer("");
		PrintfFormat fmt = new PrintfFormat("%.3f");
		buf.append("Common tokens: ");
		for (Iterator i = sBag.tokenIterator(); i.hasNext();) {
			Token tok = (Token) i.next();
			if (tBag.contains(tok)) {
				buf.append(" " + tok.getValue() + ": ");
				buf.append(fmt.sprintf(sBag.getWeight(tok)));
				buf.append("*");
				buf.append(fmt.sprintf(tBag.getWeight(tok)));
			}
		}
		buf.append("\nscore = " + score(s, t));
		return buf.toString();
	}

	public String toString() {
		return "[TFIDF]";
	}

	public static void main(String[] argv) {
		TFIDF tfidf = new TFIDF(SimpleTokenizer.DEFAULT_TOKENIZER);

		String str1 = "service hotel city locator";
		String str2 = "service country capital";
		String str3 = "country hospital finder";

		List<BasicStringWrapper> corpus = new ArrayList<BasicStringWrapper>();
		String[] words = { str1, str2, str3 };
		for (int i = 0; i < words.length; i++) {
			corpus.add(new BasicStringWrapper(words[i]));
		}

		StringWrapperIterator iterator = new BasicStringWrapperIterator(corpus
				.iterator());
		tfidf.train(iterator);

		System.out.println("### Preparing string ###");
		StringWrapper w1 = tfidf.prepare(str1);

		System.out.println("### Preparing string ###");
		StringWrapper w2 = tfidf.prepare(str2);

		tfidf.score(w1, w2);
		System.out.println(tfidf.explainScore(w1, w2));

		str1 = "Programming";
		str2 = "Semantics";

		NGramTokenizer tokenizer = new NGramTokenizer(1, 1, false,
				SimpleTokenizer.DEFAULT_TOKENIZER);
		tfidf = new TFIDF(tokenizer);
		corpus = new ArrayList<BasicStringWrapper>();
		String[] words2 = { str1, str2 };
		for (int i = 0; i < words2.length; i++) {
			corpus.add(new BasicStringWrapper(words2[i]));
		}
		iterator = new BasicStringWrapperIterator(corpus.iterator());
		tfidf.train(iterator);

		System.out.println("### Preparing string ###");
		w1 = tfidf.prepare(str1);

		System.out.println("### Preparing string ###");
		w2 = tfidf.prepare(str2);

		tfidf.score(w1, w2);
		System.out.println(tfidf.explainScore(w1, w2));

	}
}
