package ch.unizh.ifi.isparql.evaluation.engines;

import ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.io.*;
import java.util.List;

/**
 * The framework supports the execution of test queries for multiple
 * engines. The execution time of a query is calculated as an average 
 * execution time of i loops of the same query.
 * Each test can be executed j times. 
 * The result values of each test are saved to a log file.
 * Execute this class with --help command line param to get a list
 * of options. For an accurate execution time evaluation, this framework
 * uses Roubtsov timer (vladium), which requires a DDL or .so library
 * installed on the system.
 * 
 * @see com.vladium.utils.timing.ITimer;
 *
 * @author		Markus Stocker
 * @version 	$Id: Framework.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class Framework 
{
	/** Number of loops for each test */
	private static int nrOfLoops = 1;
	/** Number of tests for an evaluation */
	private static int nrOfTests = 1;
	/** Verbose flag, to display log messages */
	private static boolean verbose = true;
	private static String inGraphFileName, inQueryFileName, logFileName, evalEngine, baseURI, costModelFileName;
	
	/**
	 * Main entry point to the application
	 * 
	 * @param args
	 */
	public static void main(String[] args)
	{
		// Process all command line params
		evalCmdLineArgs(args);
		
		// Create the required engine to evaluate
		AbstractEngine engine = AbstractEngine.create(evalEngine);
		
		// Read the query from file
		String inQueryString = inQueryString(inQueryFileName);
		
		// Print out settings
		if (verbose)
		{
			System.out.println("=================================");
			System.out.println("Start evaluation");
			System.out.println("Evaluating engine: " + evalEngine);
			System.out.println("Graph file: " + inGraphFileName);
			System.out.println("Base URI: " + baseURI);
			System.out.println("Log file: " + logFileName);
			System.out.println("Number of tests: " + nrOfTests);
			System.out.println("Number of loops: " + nrOfLoops);
			System.out.println("Execute query");
			System.out.println(inQueryString);
			System.out.println("==================================\n\n");
		}
		
		// Set configuration to the selected engine
		engine.setNrOfLoops(nrOfLoops);
		engine.setNrOfTests(nrOfTests);
		engine.setInGraphFileName(inGraphFileName);
		engine.setBaseURI(baseURI);
		engine.setQueryString(inQueryString);
		engine.setVerbose(verbose);
		engine.setCostModelFileName(costModelFileName);
		
		// Start a two timer test (Java vs. Roubtsov timer) for validation
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup:
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		// Start both timers
		long start = System.currentTimeMillis();
		timer.start();
		
		// ***********************************
		// Run the test on the selected engine
		List<Double> durations = engine.run();
		// ***********************************
		
		// Stop both timers
		long end = System.currentTimeMillis();
		timer.stop();
		
		// Send log message
		if (verbose)
			System.out.println("End evaluation");
		
		// Get time difference for both timers
		long javaTime = end - start;
		Double roubtsovTime = timer.getDuration();
		
		// Reset the Roubtsov Timer
		timer.reset();
		
		// Write evaluation values to log file
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(logFileName));
			
			for (Double d : durations)
				out.append(d.toString() + "\r\n");
			
			out.close();
		} catch (IOException e) {
			System.out.println("[ERROR] Could not write to: " + logFileName);
		}
	
		// Print some results to system out
		if (verbose)
		{
			System.out.println("Total time of test (msec)");
			System.out.println("Java Timer: " + javaTime);
			System.out.println("Roubtsov Timer: " + roubtsovTime);
			System.out.println("Validation: timers should not differ too much!");
		}
		
		System.out.println("Test done, check " + logFileName);
	}
	
	/* Processing unit for command line arguments 
	 * @param args */
	private static void evalCmdLineArgs(String[] args)
	{
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i = i + 2)
			{				
				if (args[i].equals("--engine"))
					evalEngine = args[i + 1];
				else if (args[i].equals("--query"))
					inQueryFileName = args[i + 1];
				else if (args[i].equals("--graph"))
					inGraphFileName = args[i + 1];
				else if (args[i].equals("--cm"))
					costModelFileName = args[i + 1];
				else if (args[i].equals("--logs"))
					logFileName = args[i + 1];
				else if (args[i].equals("--baseURI"))
					baseURI = args[i + 1];
				else if (args[i].equals("--loops"))
					nrOfLoops = new Integer(args[i + 1]);
				else if (args[i].equals("--tests"))
					nrOfTests = new Integer(args[i + 1]);
				else if (args[i].equals("--verbose"))
					verbose = true;
				else if (args[i].equals("--help"))
					usage();
			}
		} else
			usage();
	}
	
	/* Read the query from a file 
	 * @param inQueryFileName
	 * @return String */
	private static String inQueryString(String inQueryFileName)
	{
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";
		
		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(inQueryFileName)));
		} catch (FileNotFoundException e) { System.out.println(e); }
		
		try
		{
			while((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {}
		
		return inQueryString;
	}
	
	/* Framework usage, print the options */
	private static void usage()
	{
		System.out.println("Usage: ch.unizh.ifi.isparql.evaluation.engines.Framework [options]");
		System.out.println("Options:");
		System.out.println("--engine [ARQ | KAON2 | OptARQ | SesameSeRQL | SesameSPARQL]");
		System.out.println("--query [filename]");
		System.out.println("--cm [filename]");
		System.out.println("--graph [filename]");
		System.out.println("--logs [filename]");
		System.out.println("--baseURI [URI, required for Sesame only]");
		System.out.println("--loops [1..n]");
		System.out.println("--tests [1..n]");
		System.out.println("--verbose");
		System.out.println("--help");
		
		System.exit(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
