package ch.unizh.ifi.isparql.evaluation.engines.api.impl;

import ch.unizh.ifi.isparql.evaluation.engines.api.IEngine;
import ch.unizh.ifi.isparql.evaluation.engines.engine.ARQ;
import ch.unizh.ifi.isparql.evaluation.engines.engine.KAON2;
import ch.unizh.ifi.isparql.evaluation.engines.engine.OptARQ;
import ch.unizh.ifi.isparql.evaluation.engines.engine.SesameSPARQL;
import ch.unizh.ifi.isparql.evaluation.engines.engine.SesameSeRQL;
import java.util.*;

/**
 * Framework abstract engine. Implements a factory for evaluation
 * engines and methods for the configuration of engines.
 * 
 * @author Markus Stocker
 * @version $Id: AbstractEngine.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public abstract class AbstractEngine implements IEngine
{
	/** Stores the evaluated query string */
	protected String qstr;
	/** Number of loops, the query is executed for each test */
	protected int nrOfLoops = 0;
	/** Number of tests, executed for a single evaluation */
	protected int nrOfTests = 0;
	/** File name to the underlying ontology */
	protected String inGraphFileName;
	/** File name to the statistical model for optimization */
	protected String costModelFileName;
	/** Base URI, is required for Sesame */
	protected String baseURI;
	/** Flag for log level of messages, if false, no messages are displayed */
	protected boolean verbose = false;
	
	/**
	 * Evaluation engine factory. Given the engine name
	 * as param (engine), it returns an instance of the 
	 * corresponding engine. Available engines are, ARQ,
	 * OptARQ, SesameSPARQL, SesameSeRQL and KAON2. Use this
	 * keywords to instantiate the corresponding engine.
	 * If the engine keyword is not specified, an error is 
	 * returned, and the framework is closed.
	 * 
	 * @param engine
	 * @return AbstractEngine
	 */
	public static AbstractEngine create(String engine)
	{
		if (engine.equals("ARQ"))
			return new ARQ();
		if (engine.equals("OptARQ"))
			return new OptARQ();
		if (engine.equals("SesameSPARQL"))
			return new SesameSPARQL();
		if (engine.equals("SesameSeRQL"))
			return new SesameSeRQL();
		if (engine.equals("KAON2"))
			return new KAON2();
		
		System.out.println("[AbstractEngine ERROR] Engine found found: " + engine);
		System.exit(0);
		
		return null;
	}
	
	/**
	 * General method used by each engine implementation to execute
	 * the loop of tests specified. The Java VM garbage collector is
	 * executed before each test evaluation
	 *  
	 * @return List<Double>
	 */
	protected List<Double> executeTests()
	{
		List<Double> durations = new ArrayList<Double>();
		
		for (int i = 0; i < nrOfTests; ++i)
		{
			/* Run the garbage collector, in hope it is not executed during evaluation */
			Runtime.getRuntime().gc();
			
			Double duration = evalQuery(i);
			
			/* The first execution is not calculated, as we loose time for loading 
			 * all the Java stuff in memory */
			if (i > 0)
			{
				if (verbose)
					System.out.println("Test # " + i + " done in " + duration + " msec");
				
				durations.add(duration);
			}
		}
		
		return durations;
	}
	
	/** 
	 * To implement by evaluation engines
	 * 
	 * @param nr
	 * @return Double
	 */
	protected abstract Double evalQuery(int nr);
	
	/**
	 * To implement by evaluation engines
	 * 
	 * @return List<Double>
	 */
	public abstract List<Double> run();
	
	/**
	 * Set the number of loops for each test
	 * 
	 * @param value
	 */
	public void setNrOfLoops(int value)
	{
		nrOfLoops = value;
	}
	
	/**
	 * Set the number of tests for the evaluation
	 * 
	 * @param value
	 */
	public void setNrOfTests(int value)
	{
		/* We don't calculate the first test, because of the java class loading overhead 
		 * Thus increment the # of tests by one, to have the required # of tests logged */
		nrOfTests = value + 1;
	}
	
	/**
	 * Set the base URI for the evaluation, required only for Sesame
	 * 
	 * @param value
	 */
	public void setBaseURI(String value)
	{
		baseURI = value;
	}
	
	/**
	 * Set the query string used for evaluation.
	 * This is not the input query file name.
	 * 
	 * @param value
	 */
	public void setQueryString(String value)
	{
		qstr = value;
	}
	
	/**
	 * Set the verbose flag
	 * 
	 * @param value
	 */
	public void setVerbose(boolean value)
	{
		verbose = value;
	}
	
	/**
	 * Set the file name for the underlying ontology graph
	 * 
	 * @param value
	 */
	public void setInGraphFileName(String value)
	{
		inGraphFileName = value;
	}
	
	/**
	 * Set the file name for the statistical model used for optimization
	 * @param value
	 */
	public void setCostModelFileName(String value)
	{
		costModelFileName = value;
	}
	
	/**
	 * Calculate the average for a list of query execution durations
	 * 
	 * @param values
	 * @return Double
	 */
	protected Double getAverage(List<Double> values)
	{
		int count = values.size();
		Double sum = new Double(0);
		
		for (Double d : values)	
			sum += d;
		
		if (count > 0)
			return sum / count;
		else
			return sum;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
