package ch.unizh.ifi.isparql.evaluation.engines.engine;

import ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.io.*;
import java.util.*;

/**
 * ARQ Standard Engine: Evaluation implementation for ARQ
 *
 * @author		Markus Stocker
 * @version 	$Id: ARQ.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ARQ extends AbstractEngine
{
	/** Jena model */
	private Model model = ModelFactory.createDefaultModel();
	
	/**
	 * Interface implementation. 
	 * This method executes the evaluation for ARQ, which means
	 * to load the model and execute the tests
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.IEngine
	 * @return List<Double>
	 */
	public List<Double> run()
	{	
		if (verbose)
			System.out.println("Start loading graph model");
		
		// Load RDF data
		InputStream in = FileManager.get().open( inGraphFileName );
		
		if (in == null)
			throw new IllegalArgumentException("File: " + inGraphFileName + " not found");
	
		long startTime = System.currentTimeMillis();
		long startMemory = Runtime.getRuntime().totalMemory();
		
		model.read(in, "");
		
		long endTime = System.currentTimeMillis();
		long endMemory = Runtime.getRuntime().totalMemory();
		
		long diffTime = endTime - startTime;
		long diffMemory = endMemory - startMemory;
		
		if (verbose)
		{
			System.out.println("End loding graph model");
			System.out.println("Time to load model: " + diffTime + " (msec)");
			System.out.println("Model memory consumption: " + diffMemory + "");
		}
		
		return executeTests();
	}
	
	/**
	 * Implements the execution logic of an evalution test for ARQ. The method
	 * returns the average duration for the test, which executes the query
	 * as configured by the number of loops. The method uses the special 
	 * timer to calculate an accurate execution duration.
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine
	 * @param nr
	 * @return Double
	 */
	protected Double evalQuery(int nr)
	{
		Double duration;
		List<Double> durations = new ArrayList<Double>();
		
		// Create an ITimer using the Factory class:
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup:
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		for (int i = 0; i < nrOfLoops; i++)
		{
			/* Run the garbage collector, in hope it is not executed during query execution */
			Runtime.getRuntime().gc();
			
			timer.start();
			
			// Create
			Query query = QueryFactory.create(qstr);
			
			// Prepare
			QueryExecution qe = QueryExecutionFactory.create(query, model);

			// Execute
			ResultSet results = qe.execSelect();
	        
			// Consume
			if (verbose && nr == 0 && i == 0)
			{
				/* In order to check the results, in case of verbose we output the 
				 * results for the test nr 0 and first loop. Since the test nr 0 is 
				 * dropped, we don't have to worry about overheads */
				
				int count = 0;
				
				System.out.println("-------------");
        		System.out.println("Query results");
        		System.out.println("-------------");
        		
        		for ( ; results.hasNext() ; )
        	    {
        			QuerySolution qs = results.nextSolution() ;
        			System.out.println(qs.toString());
        			count++;
        	    }
        		
        		System.out.println("Number of records: " + count);
			}
			else
				// Consume, without showing results
				ResultSetFormatter.consume(results);
			
			timer.stop();
			
			// Close
			qe.close();
			
			duration = timer.getDuration();
			
			timer.reset();
			
			// Add the loop duration to the list of test durations
			durations.add(duration);
		}
		
		// Return the average duration for the test
		return getAverage(durations);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
