package ch.unizh.ifi.isparql.evaluation.engines.engine;

import ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine;
import org.semanticweb.kaon2.api.*;
import org.semanticweb.kaon2.api.reasoner.*;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.util.*;

/**
 * KAON2 Standard Engine: Evaluation implementation for KAON2. 
 * This class based upon the KAON2 examples, which are included
 * in the KAON2 package (http://kaon2.semanticweb.org)
 *
 * @author		Markus Stocker
 * @version 	$Id: KAON2.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class KAON2 extends AbstractEngine
{
	private Reasoner reasoner = null;
	
	/**
	 * Interface implementation. 
	 * This method executes the evaluation for KAON2, which means
	 * to load the model and execute the tests
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.IEngine
	 * @return List<Double>
	 */
	public List<Double> run()
	{
		List<Double> durations = new ArrayList<Double>();
		
		DefaultOntologyResolver resolver = new DefaultOntologyResolver();
		KAON2Connection connection = KAON2Manager.newConnection();
        connection.setOntologyResolver(resolver);
        
		try
		{
			if (verbose)
				System.out.println("Start loading graph model");
			  
			long startTime = System.currentTimeMillis();
			long startMemory = Runtime.getRuntime().totalMemory();
			
			String ontologyURI = resolver.registerOntology(inGraphFileName);
        	Ontology ontology = connection.openOntology(ontologyURI, new HashMap<String,Object>());
        	reasoner = ontology.createReasoner();
        	
        	long endTime = System.currentTimeMillis();
    		long endMemory = Runtime.getRuntime().totalMemory();
    		
    		long diffTime = endTime - startTime;
    		long diffMemory = endMemory - startMemory;
        	
        	if (verbose)
        	{
    			System.out.println("End loding graph model");
    			System.out.println("Time to load model: " + diffTime + " (msec)");
    			System.out.println("Model memory consumption: " + diffMemory + "");
    		}
        	
        } catch (KAON2Exception e) { System.out.println(e); }
          catch (InterruptedException e) { System.out.println(e); }
			
        if (reasoner != null)
        	// Execute the tests, method implemented by AbstractEngine
        	durations = executeTests();
        else
        	System.exit(0);
		
		reasoner.dispose();

	    // Clean up
		try
		{
			connection.close();
		} catch (KAON2Exception e) { System.out.println(e); }
	    
	    return durations;
	}
	
	/**
	 * Implements the execution logic of an evalution test for KAON2. The method
	 * returns the average duration for the test, which executes the query
	 * as configured by the number of loops. The method uses the special 
	 * timer to calculate an accurate execution duration.
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine
	 * @param nr
	 * @return Double
	 */
	protected Double evalQuery(int nr)
	{
		int count = 0;
		Double duration;
		List<Double> durations = new ArrayList<Double>();
		
		// Create an ITimer using the Factory class:
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		for (int i = 0; i < nrOfLoops; i++)
		{   
			// Execute the garbage collector
			Runtime.getRuntime().gc();
			
			timer.start();
			
			try
			{
				// Create the query object
				Query query = reasoner.createQuery(new Namespaces(Namespaces.INSTANCE), qstr);
	        
				// Execute the query
				query.open();
				
				// Create a buffer for the solution tuples
				Object[] tupleBuffer = query.tupleBuffer();
				
				if (verbose && nr == 0 && i == 0)
				{
					System.out.println("-------------");
            		System.out.println("Query results");
            		System.out.println("-------------");
				}
				
				// Iter over the solutions
				while (!query.afterLast())
				{
					// Print the solution only for the first test and loop
					if (verbose && nr == 0 && i == 0)
					{	
						System.out.print("[ ");
						for (int j = 0; j < tupleBuffer.length; j++) {
							if (j != 0)
								System.out.print(", ");
							System.out.print(tupleBuffer[j]);
						}
						System.out.println(" ]");
						
						count++;
					}
					
					// Find the next solution
		            query.next();
		        }
				
		        timer.stop();
				
				query.dispose();
				
				if (verbose && nr == 0 && i == 0)
					System.out.println("Number of records: " + count);
				
			} catch (KAON2Exception e) { System.out.println(e); }
			  catch (InterruptedException e) { System.out.println(e); }
	        
	        duration = timer.getDuration();
			
	        timer.reset();
			
	        // Add the loop duration to the list of test durations
	        durations.add(duration);
		}
		
		// Return the average duration for the test
		return getAverage(durations);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
