package ch.unizh.ifi.isparql.evaluation.engines.engine;

import ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.ParseException;
import org.openrdf.model.Value;
import org.openrdf.sesame.query.QueryLanguage;
import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.TupleQuery;
import org.openrdf.sesame.repository.Repository;
import org.openrdf.sesame.sailimpl.memory.MemoryStore;
import org.openrdf.sesame.sail.SailInitializationException;
import org.openrdf.sesame.sail.SailUpdateException;
import org.openrdf.sesame.query.QueryHandler;
import org.openrdf.sesame.query.Query;
import org.openrdf.util.iterator.CloseableIterator;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.io.*;
import java.util.*;

/**
 * SesameSPARQL Standard Engine: Evaluation implementation for SesameSPARQL. 
 * Sesame includes a SPARQL engine which is developed third party.
 *
 * @author		Markus Stocker
 * @version 	$Id: SesameSPARQL.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class SesameSPARQL extends AbstractEngine
{
	private Repository repository;
	
	/**
	 * Interface implementation. 
	 * This method executes the evaluation for SesameSPARQL, which means
	 * to load the model and execute the tests
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.IEngine
	 * @return List<Double>
	 */
	public List<Double> run()
	{	
		if (verbose)
			System.out.println("Start loading graph model");
		
		long startTime = System.currentTimeMillis();
		long startMemory = Runtime.getRuntime().totalMemory();
		
		try
		{
			repository = new Repository(new MemoryStore());
			repository.initialize();
		
			File file = new File(inGraphFileName);
			
			// Add the underlying ontology model to the repository
			repository.add(file, baseURI, RDFFormat.RDFXML);
		} catch (IOException e) {}
		  catch (ParseException e) {}
		  catch (SailUpdateException e) {}
		  catch (SailInitializationException e) {}
			
		long endTime = System.currentTimeMillis();
		long endMemory = Runtime.getRuntime().totalMemory();
			
		long diffTime = endTime - startTime;
		long diffMemory = endMemory - startMemory;
			
		if (verbose)
		{
			System.out.println("End loding graph model");
			System.out.println("Time to load model: " + diffTime + " (msec)");
			System.out.println("Model memory consumption: " + diffMemory + "");
		}
		
		// Execute the tests
		return executeTests();
	}
	
	/**
	 * Implements the execution logic of an evalution test for SesameSPARQL. The method
	 * returns the average duration for the test, which executes the query
	 * as configured by the number of loops. The method uses the special 
	 * timer to calculate an accurate execution duration.
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.engines.api.impl.AbstractEngine
	 * @param nr
	 * @return Double
	 */
	protected Double evalQuery(int nr)
	{
		int count = 0;
		Query query = null;
		Double duration;
		CloseableIterator<List<Value>> result = null;
		List<Double> durations = new ArrayList<Double>();
    	
		// Create an ITimer using the Factory class:
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		for (int i = 0; i < nrOfLoops; i++)
		{   
			// Run the garbage collector
			Runtime.getRuntime().gc();
			
	        timer.start();
	        
	        try
	        {
	        	query = QueryHandler.parseQuery(QueryLanguage.SPARQL, qstr);
	        	result = ((TupleQuery)query).evaluate(repository.getSail());
	        } catch (MalformedQueryException e) {}
	        
	        try 
	        {	
	        	if (verbose && nr == 0 && i == 0)
	        	{
	        		System.out.println("---------------");
	        		System.out.println("Optimized query");
	        		System.out.println("---------------");
	        	
	        		System.out.println(query.toString());
	        		
	        		System.out.println("\n");
	        		
	        		System.out.println("-------------");
            		System.out.println("Query results");
            		System.out.println("-------------");
	        	}
	        	
	            while (result.hasNext()) 
	            {
	            	if (verbose && nr == 0 && i == 0)
	            	{	
	            		System.out.println(result.next().toString());
	            		
	            		count++;
	            	} 
	            	else 
	            		result.next();
	            }
	            
	            timer.stop();
	            
	            if (verbose && nr == 0 && i == 0)
	            	System.out.println("Number of records: " + count);
	        }
	        finally {
	            result.close();
	        }
	        
	        duration = timer.getDuration();
			
	        timer.reset();
			
	        // Add the loop duration to the list of test durations
			durations.add(duration);
		}
		
		// Return the average duration for the test
		return getAverage(durations);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
