package ch.unizh.ifi.isparql.evaluation.si;

import ch.unizh.ifi.isparql.optimizer.CostModel;
import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyFactory;
import ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.optimizer.SimilarityIndexMem;
import ch.unizh.ifi.isparql.optimizer.ModOptimizer;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.query.util.Symbol;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.vocabulary.*;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.util.*;
import java.io.*;

/**
 * Evaluation framework for iSPARQL similarity index. The application 
 * is used to evaluate the performance of similarity index optimization.
 * The similarity index class is required, as an implementation for the index.
 * The similarity index is a in-memory hash, or a persistent relational 
 * database model. The framework initializes an in-memory similarity index for
 * the test ontology DblpAifb which is used as evaluation for the thesis.
 * Please do not modify this class, it is referenced in the documentation of 
 * ch.unizh.ifi.isparql.query.Constants (or if you modify this class, 
 * check the consistency with the Costants class).
 * 
 * @see ch.unizh.ifi.isparql.query.Constants
 * @see ch.unizh.ifi.isparql.query.optimizer.SimilarityIndexMem
 * 
 * @author		Markus Stocker
 * @version 	$Id: Framework.java,v 1.2 2006-11-21 12:41:52 ms Exp $
 */

public class Framework 
{
	/** Similarity index object, @see ch.unizh.ifi.isparql.query.optimizer.SimilarityIndex */
	private static SimilarityIndexMem si = null;
	/** Query optimizer object, @see ch.unizh.ifi.isparql.optimizer.ModOptimizer */
	private static ModOptimizer modOptimizer = new ModOptimizer();
	/** Jena model */
	private static Model model = ModelFactory.createDefaultModel();
	/** List of test query execution durations */
	private static List<Double> tests = new ArrayList<Double>();
	/** Number of tests which are executed for an evaluation */
	private static Integer nrOfTests = 1;
	/** Number of loops which are executed for a test */
	private static Integer nrOfLoops = 1;
	private static String inFileName, inQueryFileName, logFileName, costModelFileName;

	/**
	 * Main entry point to the application
	 * 
	 * @param args
	 */
	public static void main(String[] args)
	{
		// Process command line arguments
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{
				if (args[i].equals("--graph"))
					inFileName = args[i+1];
				else if (args[i].equals("--query"))
					inQueryFileName = args[i+1];
				else if (args[i].equals("--logs"))
					logFileName = args[i+1];
				else if (args[i].equals("--tests"))
					nrOfTests = new Integer(args[i+1]);
				else if (args[i].equals("--loops"))
					nrOfLoops = new Integer(args[i+1]);
				else if (args[i].equals("--cm"))
					costModelFileName = args[i + 1];
				else if (args[i].equals("--help"))
					usage();
			}
		} else 
			usage();
		
		System.out.println("=================================");
		System.out.println("Start evaluation");
		System.out.println("Graph file: " + inFileName);
		System.out.println("Query file: " + inQueryFileName);
		System.out.println("Cost model: " + costModelFileName);
		System.out.println("Log file: " + logFileName);
		System.out.println("Number of tests: " + nrOfTests);
		System.out.println("Number of loops: " + nrOfLoops);
		System.out.println("Execute query");
		
		// Load the underlying ontology model
		InputStream in = FileManager.get().open( inFileName );
		model.read(in, "");
		
		// Load the cost model
		CostModel cm = new CostModel(costModelFileName);
		
		// Read the query from file
		String inQueryString = inQueryString(inQueryFileName);
		
		System.out.println(inQueryString);
		System.out.println("==================================\n\n");
		
		// If the similarity index is activated, create the similarity index
		if (AbstractOptimizer.optimizer)
		{
			// Create similarity index
			System.out.println("Create similarity index");
			si = createSimilarityIndex();
			System.out.println("Done (similarity index)!");
		}
		
		// Start a two timer test (Java vs. Roubtsov timer) for validation
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup:
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		// Execute the tests
		for (int i = 0; i < nrOfTests; i++)
		{
			Double sum = 0d;
			List<Double> loops = new ArrayList<Double>();
			
			// Execute the loops for a test
			for (int j = 0; j < nrOfLoops + 1; j++)
			{	
				// Run the garbage collector, in hope it is not executed during query execution
				Runtime.getRuntime().gc();
				
				timer.start();
			
				// Create an ARQ query object for the query string (not the query file name!)
				Query query = QueryFactory.create(inQueryString);
				// Optimize, run the iSPARQL optimization framework
				query = modOptimizer.optimize(query, cm);
				// Create an ARQ query execution for the (optimized) query and the underlying ontology model
				QueryExecution qe = QueryExecutionFactory.create(query, model);
				
				// Set similarity index to query execution contex (it is used by iSPARQL,
				// instead to calculate similarity values, they are extracted from the similarity index
				// Please do not modify this class, it is referenced in the documentation of 
				// ch.unizh.ifi.isparql.query.Constants
				if (AbstractOptimizer.optimizer)
					qe.getContext().set(new Symbol(Constants.SIMILARITY_INDEX), si);
				
				// Get the query results
				ResultSet results = qe.execSelect();
			
				if (i == 0 && j == 0)
				{
					int c = 0;
					
					System.out.println("---------------");
	        		System.out.println("Optimized query");
	        		System.out.println("---------------");
	        	
	        		System.out.println(query.toString());
					
					for ( ; results.hasNext() ; )
					{
						QuerySolution qs = results.nextSolution() ;
	        			System.out.println(qs.toString());
						c++;
					}
					
					System.out.println("Number of records: " + c);
				} 
				else
					// Just consume, do not print, to minimize overhead
					ResultSetFormatter.consume(results);
			
				qe.close();
			
				timer.stop();
				
				if (j > 0)
					loops.add(timer.getDuration());
				
				timer.reset();
			}
			
			for (Double d : loops)
				sum += d;
			
			Double avg = sum / loops.size();
			
			System.out.println("Test # " + i + " done in (msec): " + avg);
			
			// Add the loop average duration to the test durations
			tests.add(avg);
		}
		
		model.close();
		
		// Write evaluation values to log file
		try 
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(logFileName));
			
			for (Double d : tests)
				out.append(d.toString() + "\r\n");
			
			out.close();
		} catch (IOException e) {
			System.out.println("[ERROR] Could not write to: " + logFileName);
		}
		
	    try 
	    {
	    	in.close();
	    } catch (IOException e) {}
	}
	
	/** 
	 * Used to load the query string from a query file name
	 * 
	 * @param inQueryFileName
	 * @return String
	 */
	private static String inQueryString(String inQueryFileName)
	{
		String line = "";
		String inQueryString = "";
		BufferedReader br = null;
		
		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(inQueryFileName)));
		} catch (FileNotFoundException e) { System.out.println(e); }
		
		try
		{
			while((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {}
		
		return inQueryString;
	}
	
	/** 
	 * This is a specific method, to create the require similarity index object
	 * for the DblpAifb ontology. This is very specific to the ontology, and was
	 * created for the thesis evaluation of the similarity index performance
	 * for the DblpAifb ontology. The index is created for the strategies
	 * used for the evaluation retrieval task (ie, Dice measure for titles and
	 * Levenshtein of Levenshtein for booktitles).
	 * 
	 * @return SimilarityIndexMem
	 */
	private static SimilarityIndexMem createSimilarityIndex()
	{
		StmtIterator stmtIter = null;
		AbstractStrategy strategy = null;
		SimilarityIndexMem si = new SimilarityIndexMem();
		String OPUSNS = "http://lsdis.cs.uga.edu/projects/semdis/opus#";
		String SWRCNS = "http://swrc.ontoware.org/ontology#";
		
		// Calculate the dice similarities for titles
		String title1 = "Semantic Methods for P2P Query Routing";
		// Get all the resources title
		stmtIter = model.listStatements((Resource)null, RDFS.label, (RDFNode)null);
		// Init the strategy to compare titles
		strategy = StrategyFactory.create(AbstractStrategy.diceStrCmp, model.getGraph());
		
		// Iterate over the Dblp titles
		while (stmtIter.hasNext())
		{
			Statement stmt = stmtIter.nextStatement();
			String title2 = stmt.asTriple().getObject().getLiteralValue().toString();
			strategy.setArgument1(Node.createLiteral(title1));
			strategy.setArgument2(Node.createLiteral(title2));
			// Index the similarity
			si.addIndex(title2, title1, AbstractStrategy.diceStrCmp, strategy.similarity());
		}
		
		// Calculate the LOL similarities for booktitles
		// First get all booktitles of DBLP
		Set<String> dblpBookTitles = new HashSet<String>();
		
		stmtIter = model.listStatements((Resource)null, 
									 	ResourceFactory.createProperty(OPUSNS + "book_title"), 
									 	(RDFNode)null);
		
		// Iterate over the DBLP booktitles
		while (stmtIter.hasNext())
		{
			Statement stmt = stmtIter.nextStatement();
			dblpBookTitles.add(stmt.asTriple().getObject().getLiteralValue().toString());
		}
		
		// Now get all booktitles of AIFB
		Set<String> aifbBookTitles = new HashSet<String>();
		
		stmtIter = model.listStatements((Resource)null,
										ResourceFactory.createProperty(SWRCNS + "booktitle"),
										(RDFNode)null);
		
		// Iterate over the AIFB booktitles
		while (stmtIter.hasNext())
		{
			Statement stmt = stmtIter.nextStatement();
			aifbBookTitles.add(stmt.asTriple().getObject().getLiteralValue().toString());
		}
		
		// Now calculate all booktitle combinations of DBLP - AIFB
		// Init the strategy to compare titles
		strategy = StrategyFactory.create(AbstractStrategy.LOLStrCmp, model.getGraph());
	
		for (Iterator iter1 = dblpBookTitles.iterator(); iter1.hasNext(); )
		{
			String booktitle1 = (String)iter1.next();
			
			for (Iterator iter2 = aifbBookTitles.iterator(); iter2.hasNext(); )
			{
				String booktitle2 = (String)iter2.next();
				strategy.setArgument1(Node.createLiteral(booktitle1));
				strategy.setArgument2(Node.createLiteral(booktitle2));
				si.addIndex(booktitle1, booktitle2, AbstractStrategy.LOLStrCmp, strategy.similarity());
			}
		}
		
		return si;
	}
	
	// Print the usage of the application
	private static void usage()
	{
		System.out.println("Usage: ch.unizh.ifi.isparql.evaluation.si.Framework [options]");
		System.out.println("Options:");
		System.out.println("--graph [filename]");
		System.out.println("--query [filename]");
		System.out.println("--logs [filename]");
		System.out.println("--tests [1..n]");
		System.out.println("--loops [1..n]");
		System.out.println("--cm [filename]");
		System.out.println("--help");
		
		System.exit(0);	
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
