package ch.unizh.ifi.isparql.indexing;

import ch.unizh.ifi.isparql.indexing.IndexerFactory;
import ch.unizh.ifi.isparql.indexing.api.impl.Indexer;

/**
 * Indexing framework for iSPARQL. The application creates indexes
 * for the selectivity estimation cost model and the similarity index,
 * database version. The similarity index is constructed for typical
 * Lin/Resnik measures, where the strategy mainly compares two resources
 * from the underlying ontology.
 * 
 * @author		Markus Stocker
 * @version 	$Id: Framework.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class Framework 
{
	/** Index type, can be selectivity estimation index (SEI) or Similarity Index (SI) */
	private static String type = null;
	/** First resource used in similarity comparison */
	private static String resource1 = null;
	/** Second resource used in similarity comparison */
	private static String resource2 = null;
	/** One of AbstractStrategy, @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy */
	private static String strategyName = null;
	/** MySQL database user name */
	private static String databaseUID = null;
	/** MySQL database password */
	private static String databasePWD = null;
	/** Number of histogram classes, default 100 */
	private static Integer nrOfHistClasses = 100;
	private static String inFileName, outFileName, ontologyURI, ontologyBaseURI, ontologyRootURI, ontologyAltURI;
	
	public static void main(String[] args) 
	{	
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{
				if (args[i].equals("--graph"))
					inFileName = args[i+1];
				else if (args[i].equals("--out"))
					outFileName = args[i+1];
				else if (args[i].equals("--res1"))
					resource1 = args[i+1];
				else if (args[i].equals("--res2"))
					resource2 = args[i+1];
				else if (args[i].equals("--strategy"))
					strategyName = args[i+1];
				else if (args[i].equals("--type"))
					type = args[i+1];
				else if (args[i].equals("--dbuid"))
					databaseUID = args[i+1];
				else if (args[i].equals("--dbpwd"))
					databasePWD = args[i+1];
				else if (args[i].equals("--ontologyURI"))
					ontologyURI = args[i+1];
				else if (args[i].equals("--ontologyBaseURI"))
					ontologyBaseURI = args[i+1];
				else if (args[i].equals("--ontologyRootURI"))
					ontologyRootURI = args[i+1];
				else if (args[i].equals("--ontologyAltURI"))
					ontologyAltURI = args[i+1];
				else if (args[i].equals("--histclasses"))
					nrOfHistClasses = new Integer(args[i+1]);
				else if (args[i].equals("--help"))
					usage();
			}
		} else 
			usage();
		
		System.out.println("=================================");
		System.out.println("Start indexing");
		System.out.println("Graph file: " + inFileName);
		System.out.println("Resource: " + resource1);
		System.out.println("Strategy: " + strategyName);
		System.out.println("Number of histogram classes: " + nrOfHistClasses);
		System.out.println("Ontology URI: " + ontologyURI);
		System.out.println("Ontology Base URI: " + ontologyBaseURI);
		System.out.println("Ontology Root URI: " + ontologyRootURI);
		System.out.println("Ontology Alt URI: " + ontologyAltURI);
		System.out.println("=================================");
		
		Indexer indexer = IndexerFactory.create(inFileName, type);
		
		// Set some required param
		indexer.setOutputFileName(outFileName);
		indexer.setNrOfHistClasses(nrOfHistClasses);
		indexer.setResource1(resource1);
		indexer.setResource2(resource2);
		indexer.setStrategyName(strategyName);
		indexer.setDatabaseUsername(databaseUID);
		indexer.setDatabasePassword(databasePWD);
		indexer.setOntologyURI(ontologyURI);
		indexer.setOntologyBaseURI(ontologyBaseURI);
		indexer.setOntologyRootURI(ontologyRootURI);
		indexer.setOntologyAltURI(ontologyAltURI);
		
		// Do the indexing
		indexer.run();
		
		System.out.println("Indexing done");
	}
	
	private static void usage()
	{
		System.out.println("Usage: ch.unizh.ifi.isparql.indexing.Framework [options]");
		System.out.println("--graph [file to index]");
		System.out.println("--out [index out file]");
		System.out.println("--histclasses [integer]");
		System.out.println("--res1 [resource1]");
		System.out.println("--res2 [resource2]");
		System.out.println("--strategy [strategy name]");
		System.out.println("--type [SEI | SI]");
		System.out.println("--dbuid [database username]");
		System.out.println("--dbpwd [database password]");
		System.out.println("--ontologyURI [URI]");
		System.out.println("--ontologyBaseURI [URI]");
		System.out.println("--ontologyRootURI [URI]");
		System.out.println("--ontologyAltURI [URI]");
		
		System.exit(0);	
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
