package ch.unizh.ifi.isparql.indexing;

import ch.unizh.ifi.isparql.util.Counter;
import java.util.*;

/**
 * Class to model histogram's for the selectivity cost estimation
 * of objects. This class allows creating histograms, where a histogram
 * is simply a tree map where the key is the class lower bound (hashCode)
 * and the value is an instance of a Counter class, which allows
 * histogram class element counting.
 * 
 * @see ch.unizh.ifi.isparql.util.Counter
 * 
 * @author		Markus Stocker
 * @version 	$Id: Histogram.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class Histogram
{
	/** Nr of distinct items in a histogram class */
	private Double histogramClassSize;
	/** Store all elements for this histogram */
	private List<Double> valueList = new ArrayList<Double>();
	/** 
	 * This map stores the histogram.
	 * We store the object hash code as (double) key map 
	 * and a specific counter for the histogram class.
	 */
	private Map<Double, Counter> histogram = new TreeMap<Double, Counter>();
	/** The histogram number of classes, default is 100 */
	private Integer nrOfClasses = 100;
	
	/**
	 * Constructor. It takes the first histogram element as param.
	 * 
	 * @param value
	 */
	public Histogram(Number value)
	{
		valueList.add(value.doubleValue());
	}
	
	/**
	 * Add new elements to the histogram. 
	 * Values are hash codes of triple objects.
	 * 
	 * @param value
	 */
	public void addValue(Number value)
	{
		valueList.add(value.doubleValue());
	}
	
	/**
	 * After adding all the elements to the histogram, 
	 * create it. The histogram elements (hashCodes) are 
	 * sorted to the the histogram boundaries. This method
	 * allows setting the histogram number of classes, which 
	 * is per default set to 100. If a different number of classes
	 * is required, change it in this method. After identifying the
	 * histogram lower and upper bound, the histogram class size is
	 * computed (in respect to the number of classes). Afterwards the
	 * histogram classes are created, where the class lower bound is
	 * used as key and a new Counter class is inizialized as value.
	 * The histogram elements previously added are now processes
	 * one by one. We identify the class lower bound for each element, 
	 * and we increment the corresponding histogram class counter.
	 * 
	 * @return Map<Double, Counter>
	 */
	public Map<Double, Counter> create()
	{	
		int histogramNrOfClasses = getNrOfClasses();
		
		Collections.sort(valueList);
		
		// Get the histogramm value range
		Double histogramLowerBound = getHistogramLowerBound();
		Double histogramUpperBound = getHistogramUpperBound();
		
		setHistogramClassSize(histogramLowerBound, histogramUpperBound, histogramNrOfClasses);
		
		// Init the histogram classes with a counter set to 0
		for (int i = 0; i < histogramNrOfClasses; i++)
		{
			histogram.put(getClassLowerBound(i), new Counter(0));
		}

		// Get through the value list, find the appropriate class and add the value
		for (Iterator iter = valueList.iterator(); iter.hasNext(); )
		{
			int classCount = 1;
			Double value = (Double)iter.next();
			
			// Step throught all histogram classes and get the right one
			for (Iterator it = histogram.keySet().iterator(); it.hasNext(); )
			{
				// Extract the counter
				Double classLowerBound = (Double)it.next();
				Counter c = histogram.get(classLowerBound);
				Double classUpperBound = getClassUpperBound(classLowerBound);
				
				if (value >= classLowerBound && value < classUpperBound) 
					c.increment();
				else if (classCount == histogramNrOfClasses && value.equals(histogramUpperBound))
					c.increment();
				
				classCount++;
			}
		}
	
		return histogram;
	}
	
	/** 
	 * Set the number of histogram classes
	 * 
	 * @param value
	 */
	public void setNrOfClasses(Integer value)
	{
		nrOfClasses = value;
	}
	
	/** 
	 * Get the number of histogram classes
	 * 
	 * @return Integer
	 */
	public Integer getNrOfClasses()
	{
		return nrOfClasses;
	}
	
	/** 
	 * Returns the histogram lower bound for a sorted list of histogram elements (hash codes)
	 * 
	 * @return Double
	 */
	public Double getHistogramLowerBound()
	{
		return valueList.get(0);
	}
	
	/**
	 * Returns the histogram upper bound for a sorted list of histogram elements (hash codes)
	 * 
	 * @return Double
	 */
	public Double getHistogramUpperBound()
	{
		return valueList.get(valueList.size() - 1);
	}
	
	/**
	 * Returns the histogram class size, must be previously computed by setHistogramClassSize()
	 * 
	 * @see setHistogramClassSize()
	 * @return Double
	 */
	public Double getHistogramClassSize()
	{
		return histogramClassSize;
	}
	
	/**
	 * Compute the histogram class size given a histogram lower and upper bound, as well as 
	 * the number of histogram classes.
	 * 
	 * @param lowerBound
	 * @param upperBound
	 * @param nrOfClasses
	 */
	public void setHistogramClassSize(Double lowerBound, Double upperBound, int nrOfClasses)
	{
		histogramClassSize = (upperBound - lowerBound) / nrOfClasses;
	}
	
	/**
	 * Compute the class lower bound (please note, that's different to the histogram
	 * lower bound!) given the class index.
	 * 
	 * @param index
	 * @return Double
	 */
	public Double getClassLowerBound(int index)
	{
		return new Double(getHistogramLowerBound()) + index * getHistogramClassSize();
	}
	
	/**
	 * Compute the class upper bound (please note, that's different to the histogram
	 * upper bound!) given a class lower bound.
	 * 
	 * @param classLowerBound
	 * @return Double
	 */
	public Double getClassUpperBound(Double classLowerBound)
	{
		return classLowerBound + getHistogramClassSize();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
