package ch.unizh.ifi.isparql.indexing.api.impl;

import java.io.InputStream;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;
import ch.unizh.ifi.isparql.indexing.api.IIndexer;

/**
 * Abstract indexer class. This class is the abstract class
 * for specific indexer implementations. The class implements
 * the Indexer interface. This abstract class mainly provides
 * setter and getter methods for indexing configuration, and
 * loads the underlying ontology in-memory.
 * 
 * @author		Markus Stocker
 * @version 	$Id: Indexer.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public abstract class Indexer implements IIndexer 
{
	/** Used for similarity index, first resource used in similarity comparison */
	protected String resource1;
	/** Used for similarity index, second resource used in similarity comparison */
	protected String resource2;
	/** Used for similarity index, the MySQL database user name */
	protected String databaseUID;
	/** Used for similarity index, the MySQL database password */
	protected String databasePWD;
	/** One of AbstractStragey, see @ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy */
	protected String strategyName;
	/** The underlying ontology file name */
	protected String inFileName;
	/** Used for selectivity index, the file name where the model is stored */
	protected String outFileName;
	/** Required for similarity index, for Lin/Resnik measure */
	protected String ontologyURI;
	/** Required for similarity index, for Lin/Resnik measure */
	protected String ontologyBaseURI;
	/** Required for similarity index, for Lin/Resnik measure */
	protected String ontologyRootURI;
	/** Required for similarity index, for Lin/Resnik measure */
	protected String ontologyAltURI;
	/** Used for selectivity index, the number of histogram classes, default is 100 */
	protected Integer nrOfHistClasses;
	/** The Jena model */
	protected Model model = ModelFactory.createDefaultModel();
	
	/** 
	 * Constructor, loads the underlying ontology model from file name into memory
	 * 
	 * @param inFileName
	 */
	public Indexer(String inFileName)
	{
		this.inFileName = inFileName;
		
		InputStream in = FileManager.get().open(this.inFileName);
		
		if (in == null)
			throw new IllegalArgumentException("File: " + this.inFileName + " not found");
	
		model.read(in, "");
	}
	
	/**
	 * Set the input file name of the underlying ontology model 
	 * 
	 * @param value
	 */
	public void setInputFileName(String value)
	{
		inFileName = value;
	}
	
	/**
	 * Set the output file name for the selectivity estimation cost model
	 * 
	 * @param value
	 */
	public void setOutputFileName(String value)
	{
		outFileName = value;
	}
	
	/**
	 * Set the number of histogram classes, default is 100
	 * 
	 * @param value
	 */
	public void setNrOfHistClasses(Integer value)
	{
		nrOfHistClasses = value;
	}
	
	/**
	 * Used in similarity index, the first resource used for similarity comparison
	 * 
	 * @param value
	 */
	public void setResource1(String value) 
	{
		resource1 = value;
	}
	
	/**
	 * Used in similarity index, the second resource used for similarity comparison
	 * 
	 * @param value
	 */
	public void setResource2(String value) 
	{
		resource2 = value;
	}
	
	/** 
	 * The strategy name used for similarity comparison, one of AbstractStrategy
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
	 * @param value
	 */
	public void setStrategyName(String value)
	{
		strategyName = value;
	}
	
	/**
	 * Set the MySQL database user name, required for similarity index
	 * 
	 * @param value
	 */
	public void setDatabaseUsername(String value)
	{
		databaseUID = value;
	}
	
	/**
	 * Set the MySQL database password, required for similarity index
	 * 
	 * @param value
	 */
	public void setDatabasePassword(String value)
	{
		databasePWD = value;
	}
	
	/**
	 * Required for similarity index, for Lin/Resnik measures
	 * 
	 * @param value
	 */
	public void setOntologyURI(String value)
	{
		ontologyURI = value;
	}
	
	/**
	 * Required for similarity index, for Lin/Resnik measures
	 * 
	 * @param value
	 */
	public void setOntologyBaseURI(String value)
	{
		ontologyBaseURI = value;
	}
	
	/**
	 * Required for similarity index, for Lin/Resnik measures
	 * 
	 * @param value
	 */
	public void setOntologyRootURI(String value)
	{
		ontologyRootURI = value;
	}
	
	/**
	 * Required for similarity index, for Lin/Resnik measures
	 * 
	 * @param value
	 */
	public void setOntologyAltURI(String value)
	{
		ontologyAltURI = value;
	}
	
	/** 
	 * To implement by specific indexer (SEI / SI)
	 */
	public abstract void run();
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
