package ch.unizh.ifi.isparql.indexing.indexer;

import ch.unizh.ifi.isparql.util.Counter;
import ch.unizh.ifi.isparql.indexing.Histogram;
import ch.unizh.ifi.isparql.indexing.api.impl.Indexer;
import com.hp.hpl.jena.rdf.model.Seq;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

/**
 * The specific selectivity estimation indexer. The class extends
 * the abstract indexer and implements the indexer interface.
 * The selectivity estimation indexer creates a statistical model
 * of the underlying ontology which is required for triple pattern
 * selectivity estimation.
 * 
 * @see ch.unizh.ifi.isparql.indexing.api.impl.Indexer
 * @see ch.unizh.ifi.isparql.indexing.api.IIndexer
 * @author		Markus Stocker
 * @version 	$Id: SelectivityEstimationIndexer.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class SelectivityEstimationIndexer extends Indexer 
{	
	/** Number of triples contained in the underlying ontology */
	private int nrOfTriples;
	/** Average number of predicates for each resource in the underlying ontology */
	private double avgNrOfPredicates;
	/** Statistics for predicates, contains the frequency of each predicate */
	private Map<String, Counter> predicateStats = new HashMap<String, Counter>();
	/** 
	 * Statistics for the objects, maps predicates to the corresponding histogram,
	 * which describes the object value distributions.
	 */
	private Map<String, Histogram> objectStats = new HashMap<String, Histogram>();
	
	/** 
	 * Constructor. Takes the file name of the underlying ontology. The file
	 * name is read in an in-memory model by the abstract indexer.
	 * 
	 * @param inFileName
	 */
	public SelectivityEstimationIndexer(String inFileName)
	{
		super(inFileName);
	}
	
	/**
	 * Interface implementation. The methods runs the indexing process.
	 * It creates the index for the subjects, predicates and objects.
	 * The model is then serialized to an RDF/XML ontology and displayed
	 * to stdout in a human friendly way in order to review the 
	 * statistical analysis.
	 */
	public void run()
	{
		long start;
		long stop;
		long msec;
		
		// Index the statistics required for subject selectivity estimation
		System.out.println("CREATE SUBJECT INDEX");
		start = System.currentTimeMillis();
		createSubjectIndex();
		stop = System.currentTimeMillis();
		msec = stop - start;
		System.out.println("DONE (" + msec + " msec)\r\n");
		
		// Index the statistics required for predicate selectivity estimation
		System.out.println("CREATE PREDICATE INDEX");
		start = System.currentTimeMillis();
		createPredicateIndex();
		stop = System.currentTimeMillis();
		msec = stop - start;
		System.out.println("DONE (" + msec + " msec)\r\n");
		
		// Index the statistics required for the object selectivity estimation
		System.out.println("CREATE OBJECT INDEX");
		start = System.currentTimeMillis();
		createObjectIndex();
		stop = System.currentTimeMillis();
		msec = stop - start;
		System.out.println("DONE (" + msec + " msec)\r\n");
		
		// Serialize the model to and RDF/XML ontology
		Model mod = serialize();
		
		// Display the model to stdout for review
		display(mod);
	}
	
	// Index the statistics required for subject selectivity estimation
	private void createSubjectIndex()
	{	
		/*
		 * If subject in triple pattern is unknown, the result set size equals 
		 * to the number of triple patterns, 
		 * ie ?s ?p ?o => # of triples
		 * If subject in triple pattern is known, the result set size corresponds to the
		 * number of triple patterns matching the subject, 
		 * ie :s ?p ?o => # of triples matching :s
		 * Since indexing of the exact number of triples matching each subject would
		 * result in too much information, we calculate the average number of 
		 * predicates for a resource. Given a subject, the result set size will 
		 * be approximated by the average number of predicates for a resource.
		 */
		StmtIterator stmtIter = model.listStatements();
		/*
		 * Required if the subject is unknown and also usefull when the predicate or
		 * object is unknown. The # of triples is returned as cost prediction
		 * for unknown S/P/O in a triple pattern.
		 */
		nrOfTriples = stmtIter.toList().size();
		
		// Get the number of resources in the underlying ontology
		ResIterator subIter = model.listSubjects();

		// Get the average number of predicates for a resource, triples / # resources
		avgNrOfPredicates = (double)nrOfTriples / (double)subIter.toList().size();
	}
	
	// Index the statistics required for predicate selectivity estimation
	private void createPredicateIndex()
	{
		/*
		 * Create index information about the predicates of triples
		 * If the predicate in a triple pattern is unknown, it matches each triple
		 * statement, which yields to a result set size equal to the # of triples
		 * If the predicate is known, the result set size corresponds to
		 * the number of triples matching the given predicate.
		 * Thus we index for each predicate the number of triples (statements)
		 * matching the predicate
		 */
		StmtIterator stmtIter = model.listStatements();

		while (stmtIter.hasNext())
		{
			Statement stmt = (Statement)stmtIter.nextStatement();
			Property predicate = stmt.getPredicate();
			
			Counter c = (Counter)predicateStats.get(predicate.asNode().getURI());
				
			if (c == null)
				predicateStats.put(predicate.toString(), new Counter(1));
			else
				c.increment();
		}
	}
	
	// Index the statistics required for object selectivity estimation
	private void createObjectIndex()
	{
		/*
		 * Each predicate has its own histogram representing the distribution
		 * of the predicates object value. Thus we step through each resource (subject)
		 * extract all the predicates and we initialize a new histogram for the 
		 * predicate if it doesn't exist. Else we add the object to the predicate
		 * histogram. This way, the object value distribution of a specific 
		 * predicate is described by a histogram, either of type numeric, alphabetic
		 * or URI.
		 */
		StmtIterator stmtIter = model.listStatements();
		
		while (stmtIter.hasNext())
		{
			Statement stmt = stmtIter.nextStatement();
			Property predicate = stmt.getPredicate();
			RDFNode object = stmt.getObject();
		
			// We do a histogram for each predicate, holding all objects
			Histogram h = (Histogram)objectStats.get(predicate.toString());
				
			if (h == null)
				// Create a new histogram for this predicate
				objectStats.put(predicate.toString(), new Histogram(object.hashCode()));
			else
				// Store the object to the predicate histogram
				h.addValue(object.hashCode());
		}
	}
	
	// Serialize the model to an RDF/XML model
	private Model serialize()
	{
		int histogramId = 1;
		int histogramClassId = 1;
		Model mod = ModelFactory.createDefaultModel();
		String NS = new String("http://localhost/");
		
		// Init required resources and properties
		Resource ontologyR = mod.createResource(NS + "statistics");
		Property nrOfTriplesP = mod.createProperty(NS + "nrOfTriples");
		Property avgNrOfPredicatesP = mod.createProperty(NS + "avgNrOfPredicates");
		Property classItemsP = mod.createProperty(NS + "histogram/class/items");
		Property classLabelP = mod.createProperty(NS + "histogram/class/label");
		Property frequencyP = mod.createProperty(NS + "frequency");
		Property histogramP = mod.createProperty(NS + "histogram");
		
		Seq predicates = mod.createSeq(NS + "predicates");
		
		mod.add(ontologyR, nrOfTriplesP, nrOfTriples);
		mod.add(ontologyR, avgNrOfPredicatesP, avgNrOfPredicates);
		
		// Interate over object statistics, create the histograms
		for (Iterator objIter = objectStats.keySet().iterator(); objIter.hasNext(); )
	    {
			String predicate = (String)objIter.next();
			predicates.add(predicate);
			
			Resource predicateR = mod.createResource(predicate);
			
			Seq histogramR = mod.createSeq(NS + "histogram/#H" + histogramId);
			
			Histogram histogram = (Histogram)objectStats.get(predicate);
			
			// Set the histogram number of classes
			histogram.setNrOfClasses(nrOfHistClasses);
			
			if (histogram != null)
			{
				System.out.println("CREATE HISTOGRAM FOR " + predicate);
				// Create the histogram
				Map<Double, Counter> histogramClasses = histogram.create();
				
				for (Iterator clsIter = histogramClasses.keySet().iterator(); clsIter.hasNext(); )
				{
					// Get the class lower bound value (hash code)
					Double key = (Double)clsIter.next();
					Counter counter = (Counter)histogramClasses.get(key);
					Resource classR = mod.createResource(NS + "histogram/class/#C" + histogramClassId);
					
					histogramR.add(classR);
					
					mod.add(classR, classItemsP, counter.getValue());
					mod.add(classR, classLabelP, key);

					histogramClassId++;
				}
				mod.add(predicateR, frequencyP, predicateStats.get(predicate));
				mod.add(predicateR, histogramP, histogramR);
			}
			
			histogramId++;	
		}
		
		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(outFileName));
			out.append("<?xml version=\"1.0\"?>\r\n\r\n");
			mod.write(out,"RDF/XML");
			out.close();
		} catch (IOException e) {}
		
		return mod;
	}
	
	// Display the model to stdout for review
	private void display(Model mod)
	{
		StmtIterator iter;
		String NS = new String("http://localhost/");
		
		System.out.println("\r\n------------------");
		System.out.println("DISPLAY STATISTICS");
		System.out.println("------------------\r\n");
		
		// Get number of triples
		System.out.println("Nr of triples:");
		
		iter = mod.listStatements(ResourceFactory.createResource(NS + "statistics"),
		     					  ResourceFactory.createProperty(NS + "nrOfTriples"),
						   		  (RDFNode)null);
		
		while (iter.hasNext())
			System.out.println(iter.nextStatement().getString());
		
		// Get average number of predicates
		System.out.println("\r\nAverage number of predicates:");
		
		iter = mod.listStatements(ResourceFactory.createResource(NS + "statistics"),
								  ResourceFactory.createProperty(NS + "avgNrOfPredicates"),
								  (RDFNode)null);
		
		while (iter.hasNext())
			System.out.println(iter.nextStatement().getString());
		
		// Get list of predicates and the corresponding frenquency
		System.out.println("\r\nPredicate frequency:");
		
		Seq seq = mod.getSeq(NS + "predicates");
		
		for (int i = 1; i <= seq.size(); i++)
		{
			String subject = seq.getSeq(i).asNode().toString(false);

			iter = mod.listStatements(ResourceFactory.createResource(subject),
									  (Property)null,
									  (RDFNode)null);
			
			while (iter.hasNext())
			{
				Statement stmt = iter.nextStatement();
				String predicate = stmt.getPredicate().getLocalName();
				String object = stmt.getObject().toString();
				
				if (predicate.equals("frequency"))
					System.out.println(object + "\t" + subject + "\r\n");
				else if (predicate.equals("histogram"))
				{
					System.out.println("Object histogram:");
					
					Seq s = mod.getSeq(object);
					
					for (int j = 1; j <= s.size(); j++)
					{
						String label = null;
						String items = null;
						String histogramClass = s.getSeq(j).asNode().toString(false);
						
						StmtIterator it = mod.listStatements(ResourceFactory.createResource(histogramClass),
															 (Property)null,
															 (RDFNode)null);
						
						while (it.hasNext())
						{
							Statement st = it.nextStatement();
							String p = st.getPredicate().getLocalName();
							String o = st.getObject().toString();
							
							if (p.equals("label"))
								label = o;
							else if (p.equals("items"))
								items = o;
						}
						
						System.out.println(items + "\t" + label);
					}
				}
			}
		}
		
		System.out.println("\r\n");
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
