package ch.unizh.ifi.isparql.indexing.indexer;

import ch.unizh.ifi.isparql.indexing.api.impl.Indexer;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.StrategyFactory;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.vocabulary.OWL;
import java.util.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * The specific similarity indexer. The class extends
 * the abstract indexer and implements the indexer interface.
 * The similarity indexer, calculates the similarities
 * in respect to a similarity strategy (one of AbstractStrategy)
 * given ontology resources (URI of subjects). The first resource 
 * has to be specified, the second may be specified. Similarities are stored 
 * persistently to a MySQL database. This indexer is mainly intended
 * for a similarity index of resources which are compared by
 * similarity strategies based on Lin/Resnik measures. The purpose 
 * of the indexer was to proof the concept of a similarity index
 * for the thesis, and was developed for Lin/Resnik measures 
 * because for such measures we could expect a very significant
 * performance improvement. The similarity index requires
 * a database named isparql with a table named si, which includes
 * the attributes, resource1 varchar(), resource2 varchar(), strategy1 double.
 * 
 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
 * @see ch.unizh.ifi.isparql.indexing.api.impl.Indexer
 * @author		Markus Stocker
 * @version 	$Id: SimilarityIndexer.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class SimilarityIndexer extends Indexer 
{	
	/** MySQL connection */
	private Connection conn;
	/** The strategy used to compute the similarity of resources */
	private AbstractStrategy strategy;
	
	/**
	 * Constructor. It initializes the JDBC driver, while the 
	 * abstract indexer takes the file name and loads the persistent
	 * ontology from the file name into an in-memory model.
	 * 
	 * @param inFileName
	 */
	public SimilarityIndexer(String inFileName)
	{
		super(inFileName);
		
		try 
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {}
	}
	
	/**
	 * Interface implementation. We init the strategy object required for similarity comparison
	 * and the database connection. Further, we create the similarity index.
	 */
	public void run()
	{	
		// Init strategy
		strategy = StrategyFactory.create(strategyName, model.getGraph());
		
		// Setting params
		strategy.setOntologyURI1(ontologyURI);
		strategy.setOntologyRootURI1(ontologyRootURI);
		strategy.setOntologyBaseURI1(ontologyBaseURI);
		strategy.setOntologyAltURI(ontologyAltURI);
		
		try 
		{
			conn = DriverManager.getConnection("jdbc:mysql://localhost/isparql?user=" + databaseUID + "&password=" + databasePWD);
			
			createSimilarityIndex();
			
			conn.close();
			
		} catch (SQLException ex) {
			System.out.println("SQLException: " + ex.getMessage());
			System.out.println("SQLState: " + ex.getSQLState());
			System.out.println("VendorError: " + ex.getErrorCode());
		}
	}
	
	// Create the similarity index
	private void createSimilarityIndex()
	{
		int count = 1;
		ResIterator resIter = model.listSubjects();
		List<String> resources1 = new ArrayList<String>();
		List<String> resources2 = new ArrayList<String>();
		
		if (resource1 == null)
		{
			System.out.println("Reference resource missing (resource1)");
			System.exit(0);
		} 
		else 
			resources1.add(resource1);
		
		if (resource2 == null)
		{
			while (resIter.hasNext())
			{
				Resource res = (Resource)resIter.nextResource();
				Statement stmt = res.getProperty(RDF.type);
				
				if (stmt.getObject().asNode().getURI().equals(OWL.Class.toString()))
					resources2.add(res.getURI());
			}
		}
		else
			resources2.add(resource2);
		
		for (Iterator iter = resources1.iterator(); iter.hasNext(); )
		{
			String resource1 = (String)iter.next();
			
			for (Iterator it = resources2.iterator(); it.hasNext(); )
			{
				String resource2 = (String)it.next();
				
				strategy.setArgument1(Node.createURI(resource1));
				strategy.setArgument2(Node.createURI(resource2));
				
				Double similarity = strategy.similarity();
				
				String sql = "INSERT INTO si SET " +
							 "resource1 = '" + resource1 + "', " +
							 "resource2 = '" + resource2 + "', " +
							 "strategy1 = " + similarity;
				
				// Just print out
				System.out.println(count + " " + similarity + " " + resource1 + " " + resource2);
				
				// DB STORE
				insert(sql);
				
				count++;
			}
		}
	}
	
	// Add a new row to the database
	private void insert(String sql)
	{
		java.sql.Statement stmt = null;
		java.sql.ResultSet rs = null;
		
		try 
		{
			stmt = conn.createStatement();
			stmt.execute(sql);

		}  catch (SQLException e) { System.out.println(e.getMessage()); }
		
		finally 
		{
			if (rs != null) 
			{
				try 
				{
					rs.close();
				} catch (SQLException sqlEx) { }
				rs = null;
			}
			if (stmt != null) 
			{
				try 
				{
					stmt.close();
				} catch (SQLException sqlEx) { }
				stmt = null;
			}
		}
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
