package ch.unizh.ifi.isparql.optimizer;

import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractCostModel;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Seq;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.util.FileManager;
import java.util.*;
import java.io.InputStream;

/**
 * This class provides the logic of selectivity estimation cost computation
 *
 * @author		Markus Stocker
 * @version 	$Id: CostModel.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class CostModel extends AbstractCostModel
{	
	/** The number of triples found in the underlying ontology */
	private Integer nrOfTriples;
	/** The average numer of predicates for each resource of the underlying ontology */
	private Double avgNrOfPredicates;
	private InputStream in = null;
	/** The global name space for the statistical ontology */
	private String NS = "http://localhost/";
	/** The Jena model */
	private Model model = ModelFactory.createDefaultModel();
	/**
	 * Store the information about the predicate occurrences, i.e.
	 * how many times a predicates is found in all statements
	 * (string => predicate URI) 
	 */
	private Map<String, Integer> predicatesFrequency = new HashMap<String, Integer>();
	/** Store the object value histogram for each predicate (String => predicate URI) */
	private Map<String, TreeMap<Double, Integer>> predicatesHistogram = new HashMap<String, TreeMap<Double, Integer>>();
	/** File name holding the statistical data */
	private static String inFileName = null;

	/**
	 * Constructor, not file name is required, use this constructor only, if 
	 * a file name for the statistical ontology is previously set, to directly
	 * specified in this class (field inFileName).
	 */
	public CostModel()
	{
		this(inFileName);
	}
	
	/**
	 * Constructor, generally to be used, when a file name for the statistical 
	 * ontology is not previously specified. This constructor loads the 
	 * statistical model into memory.
	 * 
	 * @param costModelFileName
	 */
	public CostModel(String costModelFileName)
	{
		if (costModelFileName == null)
			throw new IllegalArgumentException("Please provide a file name for the statistical cost model");
		
		if (costModelFileName.length() > 0)
		{
			in = FileManager.get().open( costModelFileName );
		
			if (in == null)
				throw new IllegalArgumentException("File: " + costModelFileName + " not found");

			model.read(in, "");
		}
		
		/*
		 * Init information form model to optimized data structures
		 * Avoid to iterate more than once over the model statements
		 */
		setNrOfTriples();
		setAvgNrOfPredicates();
		setPredicatesFrequency();
		setPredicatesHistogram();
	}
	
	/**
	 * Constructor, generally to be used, when a Jena model for the statistical 
	 * ontology is already available. We can set the previously load Jena
	 * model of the statistical ontology as parameter and load the 
	 * cost model object this way.
	 * 
	 * @see com.hp.hpl.jena.rdf.model.Model
	 * @param model
	 */
	public CostModel(Model model)
	{
		this.model = model;
		
		/*
		 * Init information form model to optimized data structures
		 * Avoid to iterate more than once over the model statements
		 */
		setNrOfTriples();
		setAvgNrOfPredicates();
		setPredicatesFrequency();
		setPredicatesHistogram();
	}
	
	/**
	 * The cost for a subject is calculated as follows:
	 * - in case of a variable, the cost resulting is 1 which corresponds to the 
	 *   formula, # of stmt / # of stmt (unknown subject matches each statement)
	 * - in case of a URI, the cost resulting is calculated as ratio between the 
	 *   average # of predicates for a resource and the # of statements (this assumes
	 *   that each resource is defined only once in an ontology: we know the average
	 *   # of predicates for resources and use this as selectivity estimation if a 
	 *   subject has a defined URI)
	 *   
	 * @return Double
	 */
	protected Double costForSubject()
	{
		if (subject.isVariable())
			return 1d;
		else
			if (nrOfTriples > 0)
				return avgNrOfPredicates / nrOfTriples;
			else 
				return 1d;
	}
	
	/**
	 * The cost for a predicate is calculated as follows:
	 * - in case of a variable, the cost resulting is 1 which corresponds to the 
	 *   formula, # of stmt / # of stmt (unknown predicate matches each statement)
	 * - in case of a URI, the cost resulting is calculated as ratio between the 
	 *   predicate frequency and the # of statements
	 *  
	 * @return Double
	 */
	protected Double costForPredicate()
	{
		if (predicate.isVariable())
			return 1d;
		else
		{
			try
			{
				Double frequency = predicatesFrequency.get(predicate.getURI()).doubleValue();
				return frequency / nrOfTriples;
			} catch (NullPointerException e)
			{
				//System.out.println("[OPTIMIZER INFO] Predicate not found: " + predicate.toString(false));
				return 1d;
			}
		}
	}
	
	/**
	 * The cost for a predicate is calculated as follows:
	 * - in case of a variable, the cost resulting is 1 which corresponds to the 
	 *   formula, # of stmt / # of stmt (unknown predicate matches each statement)
	 * - in case of a literal (URI, Numbers, Strings) we evaluate the frequency based
	 *   on the histogram corresponding to the predicate
	 *   * if the predicate is known, we consider the histogram related to the predicate
	 *   * if the predicate is unknown, we consider all histograms of the same data type
	 *     (URI, alphabetic, numeric)
	 *   * if the object is constrained by a FILTER (ex. <=, >=), we consider
	 *     this information when evaluating the cost (ex. for age > 20, we consider all
	 *     items of age predicate falling into histogram classes > 20).
	 * 
	 * @return Double
	 */
	protected Double costForObject()
	{
		if (objectVariableIsBound)
			return 0.1d;
		else if (object.isVariable())
			return 1d;
		else 
		{
			try
			{
				if (predicate.isURI() && object.isLiteral())
				{	
					return getHistogramCostFor(predicate.getURI(), object.getLiteralValue().toString());
				}
				else
				{
					return 1d;
				}
			} catch (NullPointerException e)
			{
				//System.out.println("[OPTIMIZER INFO] Object histogram not found: " + object.toString(false));
				return 1d;
			}
		}
	}
	
	// Extract the cost for an object by looking for the frequency inside of the corresponding histogram
	private Double getHistogramCostFor(String predicate, String object)
	{
		TreeMap histogram = predicatesHistogram.get(predicate);
		
		for (Iterator iter = histogram.keySet().iterator(); iter.hasNext(); )
		{	
			// Check whether the histogram class key is a double
			Double classLowerBound = (Double)iter.next();
				
			if (classLowerBound >= object.hashCode())
					return (Double)histogram.get(classLowerBound) / nrOfTriples;
		}
		
		return 1d / nrOfTriples;
	}
	
	/* Read the nr of triples information from the statistical model and store the 
	 * integer value to the corresponding field (nrOfTriples) */
	private void setNrOfTriples()
	{
		StmtIterator iter = model.listStatements(ResourceFactory.createResource(NS + "statistics"),
				  								 ResourceFactory.createProperty(NS + "nrOfTriples"),
				  								 (RDFNode)null);

		if (iter.hasNext())
		{
			Node object = iter.nextStatement().getObject().asNode();
			nrOfTriples = new Integer(object.toString(false));
		}
	}
	
	/* Read the average number of predicates information from the statistical model and
	 * store the double value to the corresponding field (avgNrOfPredicates) */
	private void setAvgNrOfPredicates()
	{
		StmtIterator iter = model.listStatements(ResourceFactory.createResource(NS + "statistics"),
					 							 ResourceFactory.createProperty(NS + "avgNrOfPredicates"),
					 							 (RDFNode)null);

		if (iter.hasNext())
		{
			Node object = iter.nextStatement().getObject().asNode();
			avgNrOfPredicates = new Double(object.toString(false));
		}
	}
	
	/* Read the information about the predicates frequency from the model and create
	 * a HashMap storing the predicate URI as key and the frequency as integer value
	 * For each histogram predicate found, initialize the histogram tree maps data 
	 * structure for the predicate URI. */
	private void setPredicatesFrequency()
	{
		Seq seq = model.getSeq(NS + "predicates");
		
		for (int i = 1; i <= seq.size(); i++)
		{
			String subject = seq.getSeq(i).asNode().toString(false);

			StmtIterator iter = model.listStatements(ResourceFactory.createResource(subject),
									  				 (Property)null,
									  				 (RDFNode)null);
			
			while (iter.hasNext())
			{
				Statement stmt = iter.nextStatement();
				String predicate = stmt.getPredicate().getLocalName();
				String object = stmt.getObject().toString();
				
				if (predicate.equals("frequency"))
					predicatesFrequency.put(subject, new Integer(object));
				else if (predicate.equals("histogram"))
					// We add the histogram URI and init the tree map for the histogram
					predicatesHistogram.put(subject, new TreeMap<Double, Integer>());
			}
		}
	}
	
	/* Interate over all (empty) predicates histogram and construct the histograms
	 * as an in-memory representation from the underlying statistical ontology */
	private void setPredicatesHistogram()
	{
		for (Iterator iter = predicatesHistogram.keySet().iterator(); iter.hasNext(); )
		{
			String histogramURI = (String)iter.next();
			
			// Get the tree map representing the histogram of this predicate
			TreeMap<Double, Integer> histogram = (TreeMap<Double, Integer>)predicatesHistogram.get(histogramURI);
			
			// Check the histogram resource, it's a sequence of class URI resources
			Seq seq = model.getSeq(histogramURI);
			
			// Step through the sequence and the query all class URIs found for the histogram
			for (int i = 1; i <= seq.size(); i++)
			{
				Double classLowerBound = null;
				Integer counter = null;
				String histogramClassURI = seq.getSeq(i).asNode().toString(false);
				
				// Visit the histogramClassURI to get the label and items predicates
				StmtIterator it = model.listStatements(ResourceFactory.createResource(histogramClassURI),
													   (Property)null,
													   (RDFNode)null);
				
				while (it.hasNext())
				{
					Statement stmt = it.nextStatement();
					String p = stmt.getPredicate().getLocalName();
					String o = stmt.getObject().toString();
					
					if (p.equals("label"))
						classLowerBound = new Double(o);
					else if (p.equals("items"))
						counter = new Integer(o);
				}
				
				histogram.put(classLowerBound, counter);
			}
		}
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
