package ch.unizh.ifi.isparql.optimizer;

import ch.unizh.ifi.isparql.optimizer.CostModel;
import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import com.hp.hpl.jena.query.*;
import com.hp.hpl.jena.query.core.ElementGroup;
import java.util.*;

/**
 * ARQ optimizer module. This module is instantiated when an ARQ query object
 * should be optimized using the rules implements in iSPARQL optimizer. 
 * The ARQ query object which needs to be optimized is given as
 * parameter to the optimize() method, which returns a new (optimized)
 * ARQ query object.
 * 
 * Please make following changes to arq.query: we need to init a ModOptimizer object
 * and in queryExecLocal() after creating the query and dataset, optimize
 * the query by executing ModOptimizer.optimize()
 *
 * @author		Markus Stocker
 * @version 	$Id: ModOptimizer.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ModOptimizer
{
	/** The flag allows turning on/off the optimizer */
	private boolean optimizer = true;
	/** The new element group of the query object */
	private ElementGroup outElementGroup = null;
	/** The registry of rules executed during optimization process */
	private RuleRegistry registry = new RuleRegistry();
	
	/**
	 * Init the optimizer without optimization cost model
	 * 
	 * @param query
	 * @return Query
	 */
	public Query optimize(Query query)
	{		
		return optimize(query, new CostModel());
	}
	
	/**
	 * Optimize the input query based on a rule registry
	 * 
	 * @param query
	 * @param cm
	 * @return Query
	 */
	public Query optimize(Query query, CostModel cm)
	{		
		// We optimize only for select type
		if (optimizer && query.isSelectType())
		{
			List<AbstractRule> rules = registry.getAll();
			outElementGroup = (ElementGroup)query.getQueryPattern();

			for (Iterator iter = rules.iterator(); iter.hasNext(); )
			{
				AbstractRule rule = (AbstractRule)iter.next();
				
				/*
				 * The input query is given as param to the rule execution
				 * The method returns an optimized query based on the rule logic
				 */
				outElementGroup = rule.exec(query, outElementGroup, cm);
			}
			
			OptimizedQuery opQuery = new OptimizedQuery(query, outElementGroup);
			
			return opQuery.construct();
		}
		
		return query;
	}
	
	/**
	 * Return the rule registry object
	 * 
	 * @return RuleRegistry
	 */
	public RuleRegistry ruleRegistry()
	{
		return registry;
	}

	/**
	 * Allows checking if the optimizer is on
	 * 
	 * @return boolean
	 */
	public boolean isOn()
	{
		return optimizer;
	}
	
	/** Allows turning the optimizer on */
	public void on()
	{
		optimizer = true;
	}
	
	/** Allows turning the optimizer off */
	public void off()
	{
		optimizer = false;
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
