package ch.unizh.ifi.isparql.optimizer;

import java.util.Iterator;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.SortCondition;
import com.hp.hpl.jena.query.core.ElementGroup;

/**
 * This class represents the optimized query object (ARQ ElementGroup). 
 * It mainly constructs a new ARQ query object starting from the
 * original query object by visiting the elements.
 *
 * @author		Markus Stocker
 * @version 	$Id: OptimizedQuery.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class OptimizedQuery
{
	/** The input query */
	private Query inQuery = null;
	/** The optimized query (output) */
	private Query outQuery = null;
	/** The transformed element group */
	private ElementGroup outElementGroup = null;
	
	/**
	 * Constructor. It takes the original query object and the 
	 * optimized query element group (SPARQL WHERE clause)
	 * 
	 * @param inQuery
	 * @param outElementGroup
	 */
	public OptimizedQuery(Query inQuery, ElementGroup outElementGroup)
	{
		this.inQuery = inQuery;
		this.outElementGroup = outElementGroup;
	}
	
	/**
	 * Construct the optimized query. This method constructs the 
	 * new ARQ query object, starting from the original query object
	 * and the new (optimized) ARQ element group.
	 * 
	 * @return Query
	 */
	public Query construct()
	{		
		// Init the transformed query
		outQuery = QueryFactory.make();
		
		/* Construct the transformed query
		 * Exept of element group, all optimized query elements are the same */
		prolog();
		select();
		// Set the transformed element group to the output query
		elementGroup(outElementGroup);
		orderBy();
		offset();
		limit();
		
		//System.out.println(outElementGroup);
		
		return outQuery;
	}
	
	/* Read the base URI and PREFIX information from the input query
	 * and set them to the output query */
	private void prolog()
	{
		// BASE URI reduces performance
		outQuery.setBaseURI(inQuery.getBaseURI());
		outQuery.setPrefixMapping(inQuery.getPrefixMapping());
		outQuery.setQuerySelectType();
	}
	
	/* Construct the SELECT with all variables defined in input query
	 * and set them to the transformed (output) query */
	private void select()
	{
		// Check whether the input query has projection variables or not
		if (inQuery.isQueryResultStar())
		{
			outQuery.setQueryResultStar(true);
		} else 
		{
			// Add all projection variables to the optimized query
			for (Iterator iter = inQuery.getResultVars().iterator(); iter.hasNext(); )
			{
				outQuery.addResultVar((String)iter.next());
			}
		}
	}
	
	// Set the transformed SELECT element group to the output query
	private void elementGroup(ElementGroup outElementGroup)
	{
		outQuery.setQueryPattern(outElementGroup);
	}
	
	// Read the order by (if defined) and set it to the output query
	private void orderBy()
	{
		if (inQuery.hasOrderBy())
		{
			for (Iterator iter = inQuery.getOrderBy().iterator(); iter.hasNext(); )
			{
				outQuery.addOrderBy((SortCondition)iter.next());
			}
		}
	}
	
	// Read the offset (if defined) and set it to the output query
	private void offset()
	{
		if (inQuery.hasOffset())
			outQuery.setOffset(inQuery.getOffset());
	}
	
	// Read the limit (if defined) and set it to the output query
	private void limit()
	{
		if (inQuery.hasLimit())
			outQuery.setLimit(inQuery.getLimit());
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
