package ch.unizh.ifi.isparql.optimizer.api.impl;

import ch.unizh.ifi.isparql.optimizer.api.ICostModel;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.ElementTriplePattern;

/**
 * Abstract cost model. The framework allows the implementation of different
 * cost model, which all extends this abstract cost model class.
 * The class implements the cost model interface.
 *
 * @see			ch.unizh.ifi.isparql.optimizer.api.ICostModel
 * @author		Markus Stocker
 * @version 	$Id: AbstractCostModel.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public abstract class AbstractCostModel implements ICostModel
{
	/** The triple pattern subject, for which the cost is calculated */
	protected Node subject;
	/** The triple pattern predicate, for which the cost is calculated */
	protected Node predicate;
	/** The triple pattern object, for which the cost is calculated */
	protected Node object;
	/** A flag, to check whether the object variable is previously bound in query */
	protected boolean objectVariableIsBound = false;
	
	/** 
	 * Interface implementation. The cost for a triple pattern is calculated
	 * as the multiplications of the costs for subject, predicate and object.
	 * 
	 * @param el
	 * @return Double
	 */
	public Double costForTriplePattern(ElementTriplePattern el)
	{		
		subject = el.getTriple().getSubject();
		predicate = el.getTriple().getPredicate();
		object = el.getTriple().getObject();
		
		return costForSubject() * costForPredicate() * costForObject();
	}
	
	/**
	 * Set the flag for previously bound objects in query. For such objects
	 * we implement a hack, since for them we return a cost of 0.1 instead
	 * of 1.0.
	 * 
	 * @param value
	 */
	public void objectVariableIsBound(boolean value)
	{
		objectVariableIsBound = value;	
	}
	
	/** To implement by specific cost model */
	protected abstract Double costForSubject();
	
	/** To implement by specific cost model */
	protected abstract Double costForPredicate();
	
	/** To implement by specific cost model */
	protected abstract Double costForObject();
	
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
