package ch.unizh.ifi.isparql.optimizer.rule1.visitor;

import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import ch.unizh.ifi.isparql.query.URIRegistry;
import ch.unizh.ifi.isparql.query.StrategyProperty;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.SortCondition;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import com.hp.hpl.jena.query.expr.ExprFunction;
import com.hp.hpl.jena.graph.Node;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the prepare stage. Basically we need to extract information
 * about the query variable statistics and the equals (=) expressions in 
 * FILTER elements, required on transform stage to optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupPrepare extends ElementVisitorBase
{	
	/** The input query */
	private Query inQuery = null;
	/** Object for query variable statistics */
	private QueryVarStats queryVarStats = null;
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** All FILTER expressions joined by logical AND with function equals (=) */
	private Map<String, Node> exprEqualsMap = new HashMap<String, Node>();
	/** All FILTER expressions which are not joined by logical AND */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	/** Flag for isparql blocks */
	private boolean isImprecise = false;
	/** The registry of iSPARQL URI-references */
	private URIRegistry urireg = new URIRegistry();
	
	/**
	 * Constructor.
	 * Init the object for query variable statistics
	 * 
	 * @param inQuery
	 * @param inElementGroup
	 */
	public ElementGroupPrepare(Query inQuery, ElementGroup inElementGroup)
	{	
		this.inQuery = inQuery;
		this.inElementGroup = inElementGroup;
		this.queryVarStats = new QueryVarStats();
	}
	
	/**
	 * Process logic when preparing the input query.
	 * First the projection variables are inspected.
	 * Then the input query element group is processed (var stats and FILTER processing).
	 * Last the ORDER BY clause variables are processed.
	 */
	public void exec()
	{
		processProjection();
		processElementGroup();
		processOrderBy();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get the sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * Check whether the S/P/O are variables and make statistics.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{	
		Node subject = el.getTriple().getSubject();
		Node predicate = el.getTriple().getPredicate();
		Node object = el.getTriple().getObject();
			
		// Check whether it is an imprecise block
		// Please note, this requires that iSPARQL was already reordered
		if (predicate.getURI().equals(urireg.get(StrategyProperty.name)))
			isImprecise = true;
	
		if (! isImprecise)
		{
			if (subject.isVariable())
				queryVarStats.inc(subject.getName());
			
			if (predicate.isVariable())
				queryVarStats.inc(predicate.getName());
			
			if (object.isVariable())
				queryVarStats.inc(object.getName());
		}
		
		// Mark the end of a imprecise block
		if (predicate.getURI().equals(urireg.get(StrategyProperty.similarity)))
			isImprecise = false;
	}
	
	/**
	 * Visit a FILTER element.
	 * Create a ExpressionPrepare visitor to visit the FILTER expression.
	 * Extract the list of expressions with a equals function (=).
	 * Extract the list of expressions which are not joined by logical AND.
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{	
		// Visitor for the filter expression
		FilterExprPrepare visitor = new FilterExprPrepare(el.getExpr().getFunction());
		
		visitor.exec();
	
		// Get map of all filter expressions which can't be substituted
		exprNotLogicalAnd = visitor.getExprNotLogicalAnd();
		
		// Get map of all filter expressions which can be substituted
		exprEqualsMap = visitor.getExprEqualsMap();
	}
	
	// Make statistics about the projection variables
	private void processProjection()
	{	
		// Record stats for projection variables
		for (Iterator iter = inQuery.getResultVars().iterator(); iter.hasNext(); )
			queryVarStats.inc((String)iter.next());
	}
	
	// Walk throught the query element group
	private void processElementGroup()
	{
		// Record stats for variables found in WHERE clause
		ElementWalker.walk(inElementGroup, this);
	}
	
	// Make statistics about the order by variables
	private void processOrderBy()
	{
		// Record order by variables
		if (inQuery.hasOrderBy())
		{
			for (Iterator iter = inQuery.getOrderBy().iterator(); iter.hasNext(); )
			{
				SortCondition sortCondition = (SortCondition)iter.next();
				queryVarStats.inc(sortCondition.getExpression().getVarName());
			}
		}
	}
	
	/**
	 * Return the map of FILTER expressions with equals function
	 * 
	 * @return Map<String, String>
	 */
	public Map<String, Node> getExprEqualsMap()
	{
		return exprEqualsMap;
	}
	
	/**
	 * Return the list of FILTER expressions which are not joined by a logical AND
	 * 
	 * @return List<ExprFunction>
	 */
	public List<ExprFunction> getExprNotLogicalAnd()
	{
		return exprNotLogicalAnd;
	}
	
	/**
	 * Return the object which holds the statistical information about the 
	 * variables found in query
	 * 
	 * @return QueryVarStats
	 */
	public QueryVarStats getQueryVarStats()
	{
		return queryVarStats;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
