package ch.unizh.ifi.isparql.optimizer.rule1.visitor;

import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import ch.unizh.ifi.isparql.query.StrategyProperty;
import ch.unizh.ifi.isparql.query.URIRegistry;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import com.hp.hpl.jena.query.expr.ExprFunction;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the transform stage. We use the information extracted from
 * the previous prepare stage to transform and optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupTransform extends ElementVisitorBase
{	
	/** Object holding the statistics about the query variables */
	private QueryVarStats queryVarStats = null;
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** The output query element group (optimized) */
	private ElementGroup outElementGroup = null;
	/** The map holding equals FILTER expressions */
	private Map<String, Node> exprEqualsMap = null;
	/** The list holding all other expressions */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	/** The list with eventually new triples to add (in case the substituted variable is used else where in query) */
	private List<ElementTriplePattern> additionalTriplePattern = new ArrayList<ElementTriplePattern>();
	/** Flag for isparql blocks */
	private boolean isImprecise = false;
	/** The registry of iSPARQL URI-references */
	private URIRegistry urireg = new URIRegistry();
	
	/**
	 * Constructor. Takes just a list of params.
	 * 
	 * @param queryVarStats
	 * @param inElementGroup
	 * @param outElementGroup
	 * @param exprEqualsMap
	 * @param exprNotLogicalAnd
	 */
	public ElementGroupTransform(QueryVarStats queryVarStats, ElementGroup inElementGroup, ElementGroup outElementGroup, Map<String, Node> exprEqualsMap, List<ExprFunction> exprNotLogicalAnd)
	{
		this.inElementGroup = inElementGroup;
		this.exprEqualsMap = exprEqualsMap;
		this.queryVarStats = queryVarStats;
		this.outElementGroup = outElementGroup;
		this.exprNotLogicalAnd = exprNotLogicalAnd;
	}
	
	/**
	 * Process logic when transforming the input query.
	 * First process the element group (substitute variables, modify FILTER).
	 * Add new triple pattern if required.
	 */
	public void exec()
	{
		processElementGroup();
		processAdditionalTriplePattern();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * In case the triple pattern object variable matches with one from FILTER
	 * extracted in previous prepare stage, substitute the variable and check
	 * whether the variable is used elsewhere in the query. If true, store
	 * a corresponding triple pattern for later addition to a list.
	 * 
	 * If the triple pattern object isn't a variable, add the pattern as is
	 * to the output element group.
	 * 
	 * If the triple pattern object is a variable, but don't match with a
	 * FILTER variable which can be substituted, add the pattern as is to 
	 * the output element group.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{	
		Node subject = el.getTriple().getSubject();
		Node predicate = el.getTriple().getPredicate();
		Node object = el.getTriple().getObject();
		
		// Check whether it is an imprecise block
		// Please note, this requires that iSPARQL was already reordered
		if (predicate.getURI().equals(urireg.get(StrategyProperty.name)))
			isImprecise = true;
		
		/*
		 * Triple pattern without variable as object need just to be added
		 * to the transform element group
		 */
		if (! object.isVariable())
		{
			outElementGroup.addElementTriplePattern(el);
			return;
		}
		
		// This can be done only if object is a variable, else null pointer exception
		String key = el.getTriple().getObject().getName();
		
		if (exprEqualsMap.containsKey(key))
		{
			/* 
			 * The triple pattern object matches with a filter variable name
			 * Create a new TriplePattern element and substitute the object 
			 * variable with the value set on FILTER (exprEqualsMap)
			 */
			outElementGroup.addElementTriplePattern(new ElementTriplePattern(new Triple(subject, 
																						predicate, 
																						Node.createLiteral(exprEqualsMap.get(key).getLiteral()))));
			// Since we removed the object variable, we need to decrement the 
			// corresponding stats, only for SPARQL, not iSPARQL
			// We do not increment iSPARQL vars, see ElementGroupPrepare
			if (!isImprecise)
				queryVarStats.dec(key);
			
			/*
			 * Check if the variable eliminated on substitution is needed 
			 * somewhere (like for projection). In this case, create a new
			 * TriplePattern element with the variable as object, ie
			 * ?s :p ?o . FILTER (?o = "abc") 
			 * ==>
			 * ?s :p "abc" . ?s :p ?o
			 * 
			 * If the stats for this variable are > than 0, we need to add
			 * a triple pattern with the variable as object, since the value
			 * is reused maybe in projection, as subject or in ORDER BY
			 */
			if (!isImprecise && queryVarStats.get(key) > 0) {
				additionalTriplePattern.add(new ElementTriplePattern(new Triple(subject, 
																				predicate, 
																				Node.createVariable(key))));
				// We add a new triple pattern with a variable as object, increment the corresponding stat
				queryVarStats.inc(key);
			}
		} 
		else
			outElementGroup.addElementTriplePattern(el);
		
		// Mark the end of a imprecise block
		if (predicate.getURI().equals(urireg.get(StrategyProperty.similarity)))
			isImprecise = false;
	}
	
	/**
	 * Visit an FILTER element.
	 * Create an ExpressionTransform visitor and process the FILTER element.
	 * We need to add all not substituted variables to as FILTER to the
	 * output element group.
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{	
		FilterExprTransform visitor = new FilterExprTransform(exprNotLogicalAnd, outElementGroup);
		
		visitor.exec();
		
		outElementGroup = visitor.getOutElementGroup();
	}
	
	/**
	 * Return the output element group
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}

	// Walk throught the input element group
	private void processElementGroup()
	{
		ElementWalker.walk(inElementGroup, this);
	}
	
	// Add additional triple patterns if required
	private void processAdditionalTriplePattern()
	{
		/*
		 * If a variable was substituted but it is used in projection, imprecise
		 * or order by, we need to add a triple pattern holding this variable. 
		 * We add all this triple patterns at the end of non imprecise sparql
		 * since potentially the query
		 * 
		 * ?s :p1 "abc" .
		 * ?s :p2 "def" .
		 * ?s :p1 ?o .
		 * 
		 * is more efficient than
		 * 
		 * ?s :p1 "abc" .
		 * ?s :p1 ?o .
		 * ?s :p2 "def" .
		 */
		for (Iterator iter = additionalTriplePattern.iterator(); iter.hasNext(); )
			outElementGroup.addElementTriplePattern((ElementTriplePattern)iter.next());
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
