package ch.unizh.ifi.isparql.optimizer.rule1.visitor;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.expr.ExprFunction;
import com.hp.hpl.jena.query.expr.ExprVisitorBase;
import com.hp.hpl.jena.query.expr.NodeValue;
import com.hp.hpl.jena.query.expr.NodeVar;
import java.util.*;

/**
 * This class provides the logic to visit a FILTER expression during 
 * prepare stage. For our purpose we need the figure out every expression 
 * that could be substituted in a triple pattern.
 * 
 * This is the case for expressions that are joined by logical AND (&&) and 
 * where the operator is equal (=). Expressions that don't meet this conditions
 * are stored in a special List for reuse during the transform stage, since
 * they are required to build the new FILTER element for the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: FilterExprPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class FilterExprPrepare extends ExprVisitorBase
{	
	/** The ARQ FILTER expression */
	private ExprFunction exprFunction = null;
	/** List of FILTER expressions that are concatenated by a logical AND */
	private List<ExprFunction> exprEquals = new ArrayList<ExprFunction>();
	/** List of FILTER expressions that are not concatenated by a logical AND */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	
	/**
	 * Constructor, just takes an argument
	 * 
	 * @param exprFunction
	 */
	public FilterExprPrepare(ExprFunction exprFunction)
	{
		this.exprFunction = exprFunction;
	}
	
	/** Visit the FILTER expression during prepare stage */
	public void exec()
	{
		visit(exprFunction);
	}
	
	/**
	 * Get the filter expression function and visit it
	 * There are two special cases which has to be considered, 
	 * (a) In case the operator is a logical AND, we visit the sub expressions, 
	 * (b) In case the operator is equal, we add the expression to a list,
	 * (c) In all other cases, we add the expression to another list. This list
	 * ist required on transforming stage to rebuild the FILTER element with
	 * all expressions that couldn't be substituted.
	 * 
	 * @param func
	 */
	public void visit(ExprFunction func)    
	{		
		if (func.getOpName().equals("&&"))
		{
			for (Iterator iter = func.getArgs().iterator(); iter.hasNext(); )
			{
				ExprFunction subFunc = (ExprFunction)iter.next();
				subFunc.visit(this);
			}
		}
		else if (func.getOpName().equals("="))
			exprEquals.add(func);
		else
			exprNotLogicalAnd.add(func);
	}
	
	/**
	 * Get a key/value mapping of all equals expressions found in FILTER element.
	 * This map is then used on transform stage, where variables are matched
	 * with the keys of this map to get potentially substitutions, ie
	 * ?s :p ?o . FILTER (?o = "abc") 
	 * ==> 
	 * ?s :p "abc"
	 * 
	 * @return Map<String, String>
	 */
	public Map<String, Node> getExprEqualsMap()
	{		
		// We need the node to extract the literal and the language too
		Map<String, Node> exprEqualsMap = new HashMap<String, Node>();
		
		for (Iterator iter = exprEquals.iterator(); iter.hasNext(); )
		{
			ExprFunction exprNode = (ExprFunction)iter.next();
			NodeVar nodeVar = (NodeVar)exprNode.getArgs().get(0);
			NodeValue nodeValue = (NodeValue)exprNode.getArgs().get(1);
			exprEqualsMap.put(nodeVar.getVarName(), nodeValue.asNode());
		}
		
		return exprEqualsMap;
	}
	
	/**
	 * Return the expression functions list of FILTER element which are not
	 * joined by a logical AND (&&). We do only substitute expression functions
	 * joined by logical AND since triple pattern matches are AND joins.
	 * 
	 * This list ist hold for the transformation stage, since there we are 
	 * interested in all expression functions which could not be substituted.
	 * They have to be added in a new FILTER element.
	 * 
	 * @return List<ExprFunction>
	 */
	public List<ExprFunction> getExprNotLogicalAnd()
	{
		return exprNotLogicalAnd;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
