package ch.unizh.ifi.isparql.optimizer.rule2.visitor;

import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import com.hp.hpl.jena.query.expr.ExprFunction;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the prepare stage. Basically we need to extract information
 * about FILTER expressions joined with a logical AND. Those can be executed
 * earlier, i.e. move them close to the triple pattern object variable used
 * in FILTER expression.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupPrepare extends ElementVisitorBase
{	
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** Visitor for the filter expression */
	private FilterExprPrepare filterExprVisitor = new FilterExprPrepare();
	
	/**
	 * Constructor, just take a parameter
	 * 
	 * @param inElementGroup
	 */
	public ElementGroupPrepare(ElementGroup inElementGroup)
	{
		this.inElementGroup = inElementGroup;
	}
	
	/**
	 * Process logic when preparing the input query.
	 * The input query element group will be processed
	 */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get the sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a FILTER element.
	 * Execute the FILTER expression visitor for the FILTER element found.
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{	
		filterExprVisitor.visit(el.getExpr().getFunction());
	}
	
	// Walk throught the query element group
	private void processElementGroup()
	{
		// Record stats for variables found in WHERE clause
		ElementWalker.walk(inElementGroup, this);
	}
	
	/**
	 * Return the map of FILTER expressions with equals function
	 * 
	 * @return Map<String, String>
	 */
	public List<ExprFunction> getLogicalAndExpr()
	{
		return filterExprVisitor.getLogicalAndExpr();
	}
	
	/**
	 * Return the list of FILTER expressions which are not joined by a logical AND
	 * 
	 * @return List<ExprFunction>
	 */
	public List<ExprFunction> getNotLogicalAndExpr()
	{
		return filterExprVisitor.getNotLogicalAndExpr();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
