package ch.unizh.ifi.isparql.optimizer.rule2.visitor;

import com.hp.hpl.jena.query.expr.NodeVar;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import com.hp.hpl.jena.query.expr.ExprFunction;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the transform stage. We use the information extracted from
 * the previous prepare stage to transform and optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupTransform extends ElementVisitorBase
{
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** The output query element group (optimized) */
	private ElementGroup outElementGroup = null;
	/** The map holding equals FILTER expressions */
	private List<ExprFunction> exprLogicalAnd = new ArrayList<ExprFunction>();
	/** The list holding all other expressions */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();	
	
	/**
	 * Constructor, takes just a list of params
	 * 
	 * @param inElementGroup
	 * @param outElementGroup
	 * @param exprEqualsMap
	 * @param exprNotLogicalAnd
	 */
	public ElementGroupTransform(ElementGroup inElementGroup, ElementGroup outElementGroup, List<ExprFunction> exprLogicalAnd, List<ExprFunction> exprNotLogicalAnd)
	{
		this.inElementGroup = inElementGroup;
		this.outElementGroup = outElementGroup;
		this.exprLogicalAnd = exprLogicalAnd;
		this.exprNotLogicalAnd = exprNotLogicalAnd;
	}
	
	/**
	 * Process logic when transforming the input query.
	 * First process the element group (substitute variables, modify FILTER).
	 */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * In case the triple pattern object variable matches with one from FILTER
	 * extracted in previous prepare stage, substitute the variable and check
	 * whether the variable is used elsewhere in the query. If true, store
	 * a corresponding triple pattern for later addition to a list.
	 * 
	 * If the triple pattern object isn't a variable, add the pattern as is
	 * to the output element group.
	 * 
	 * If the triple pattern object is a variable, but don't match with a
	 * FILTER variable which can be substituted, add the pattern as is to 
	 * the output element group.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{	
		ExprFunction matchedExpr = null;
		
		// In each case, the input element triple pattern is added to the output element group
		outElementGroup.addElementTriplePattern(el);
		
		// Do nothing for triple pattern without a object variable
		if (! el.getTriple().getObject().isVariable())
			return;
		
		// Else step through all FILTER expressions found in previous stage
		for (Iterator iter = exprLogicalAnd.iterator(); iter.hasNext(); )
		{
			ExprFunction expr = (ExprFunction)iter.next();
			NodeVar nodeVar = (NodeVar)expr.getArgs().get(0);
			String objectVarName = el.getTriple().getObject().getName();
			String exprVarName = nodeVar.getVarName();
			
			/*
			 * Check whether the triple pattern object variable name
			 * matches with the FILTER expression variable
			 */
			if (objectVarName.equals(exprVarName))
			{
				// Mark expression for deletion
				matchedExpr = expr;
				
				// Add the FILTER expression to the element group
				outElementGroup.addElementFilter(new ElementFilter(expr));
			}
		}
		
		// Remove the matched expression from list, avoid adding the same filter again!
		exprLogicalAnd.remove(matchedExpr);
	}
	
	/**
	 * Visit an FILTER element.
	 * Create an ExpressionTransform visitor and process the FILTER element.
	 * We need to add all not substituted variables to as FILTER to the 
	 * output element group.
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{	
		FilterExprTransform visitor = new FilterExprTransform(exprNotLogicalAnd, outElementGroup);
		
		visitor.exec();
		
		outElementGroup = visitor.getOutElementGroup();
	}
	
	/**
	 * Return the output element group
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}

	// Walk throught the input element group
	private void processElementGroup()
	{
		ElementWalker.walk(inElementGroup, this);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
