package ch.unizh.ifi.isparql.optimizer.rule2.visitor;

import com.hp.hpl.jena.query.expr.ExprFunction;
import com.hp.hpl.jena.query.expr.ExprVisitorBase;
import java.util.*;

/**
 * This class provides the logic to visit a FILTER expression during 
 * prepare stage. For our purpose we need the figure out every expression 
 * that could be substituted in a triple pattern.
 * 
 * This is the case for expressions that are joined by logical AND (&&) and 
 * where the operator is equal (=). Expressions that don't meet this conditions
 * are stored in a special List for reuse during the transform stage, since
 * they are required to build the new FILTER element for the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: FilterExprPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class FilterExprPrepare extends ExprVisitorBase
{
	/** The list of FILTER expression elements which are concatenated by logical AND */
	private List<ExprFunction> exprLogicalAnd = new ArrayList<ExprFunction>();
	/** THe list of FILTER expression elements which are not concatenated by logical AND */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	
	/**
	 * Get the filter expression function and visit it
	 * There are two special cases which has to be considered,
	 * (a) In case the operator is a logical AND, we visit the sub expressions,
	 * (b) In case the operator is equal, we add the expression to a list,
	 * (c) In all other cases, we add the expression to another list. This list
	 * ist required on transforming stage to rebuild the FILTER element with
	 * all expressions that couldn't be substituted.
	 * 
	 * @param func
	 */
	public void visit(ExprFunction func)    
	{		
		String operator = func.getOpName();
		
		if (operator.equals("&&"))
		{
			for (Iterator iter = func.getArgs().iterator(); iter.hasNext(); )
			{
				ExprFunction subFunc = (ExprFunction)iter.next();
				subFunc.visit(this);
			}
		}
		else if (operator.equals("<") || operator.equals(">") || operator.equals("<=") || operator.equals(">="))
			exprLogicalAnd.add(func);
		else
			exprNotLogicalAnd.add(func);
	}
	
	/**
	 * Return the expression functions list of FILTER element which are
	 * joined by a logical AND (&&).
	 * 
	 * @return List<ExprFunction>
	 */
	public List<ExprFunction> getLogicalAndExpr()
	{		
		return exprLogicalAnd;
	}
	
	/**
	 * Return the expression functions list of FILTER element which are NOT
	 * joined by a logical AND (&&).
	 * 
	 * @return List<ExprFunction>
	 */
	public List<ExprFunction> getNotLogicalAndExpr()
	{
		return exprNotLogicalAnd;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
