package ch.unizh.ifi.isparql.optimizer.rule3;

import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule3.visitor.ElementGroupPrepare;
import ch.unizh.ifi.isparql.optimizer.rule3.visitor.ElementGroupTransform;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import java.util.*;

/**
 * Reorder iSPARQL (imprecise) triple pattern. The rule reorders imprecise
 * statements (virtual triples) in order that an arbitrary order of imprecise
 * statements can be given in the original query. Since imprecise statements
 * requires some rules of ordering in order that ARQ executes the query
 * without exceptions, we add this rule of reordering iSPARQL, first 
 * to allow to the user an arbirary listing of imprecise statements in query
 * (which is naturally for SPARQL) and second to reorder similarity strategies
 * agains their selectivity, which is specified by the threshold statement.
 * In fact, the higher the threshold, the more are the resulting tuples
 * potentially constrained.
 *
 * @author		Markus Stocker
 * @version 	$Id: ReorderImprecise.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ReorderImprecise extends AbstractRule
{	
	/** Flag the element group as iSPARQL */
	private boolean isImprecise = false;
	/** List of imprecise variables, ie ?strategyN */
	private Set<String> impreciseSubjects;
	/** List of imprecise statements */
	private List<ElementTriplePattern> impreciseTriplePattern;
	
	/** 
	 * The prepare stage for this rule, first check whether the query is a iSPARQL query.
	 * If true, process the stage, and extract the list of imprecise subjects and statements.
	 */
	public void prepare() 
	{
		ElementGroupPrepare visitor = new ElementGroupPrepare(inElementGroup);
		
		visitor.exec();
		
		isImprecise = visitor.isImprecise();
		
		if (isImprecise)
		{
			impreciseSubjects = visitor.getImpreciseSubjects();
			impreciseTriplePattern = visitor.getImpreciseTriplePattern();
		}
	}
	
	/**
	 * The transform stage for this rule.
	 * 
	 * @param ElementGroup
	 */
	public ElementGroup transform()
	{			
		ElementGroupTransform visitor = new ElementGroupTransform(inElementGroup, 
				  												  outElementGroup,
				  												  impreciseSubjects,
				  												  impreciseTriplePattern,
				  												  isImprecise);

		visitor.exec();

		return visitor.getOutElementGroup();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
