package ch.unizh.ifi.isparql.optimizer.rule3.visitor;

import ch.unizh.ifi.isparql.query.URIRegistry;
import ch.unizh.ifi.isparql.query.StrategyProperty;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import java.util.*;

/**
 * This class provides the logic to find the imprecise subjects and statements
 * in an iSPARQL query, and allows checking whether the query is imprecise or not.
 * The information collected in this stage is required during transform stage, 
 * to process the iSPARQL query reordering.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupPrepare extends ElementVisitorBase
{	
	/** The registry of iSPARQL URI-references */
	private URIRegistry urireg = new URIRegistry();
	/** List of URI-references of isparql properties, eg. threshold, similarity, name, ... */
	private List<String> strategyPropertyURI = new ArrayList<String>();
	/** Store the subject strategy variable as key for each imprecise sparql triple block found in query */
	private Map<String, List<ElementTriplePattern>> impreciseBlocks = new TreeMap<String, List<ElementTriplePattern>>();
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** Semaphore to process imprecise RDF lists (arguments, scores, weights ... properties) */
	private boolean isImpreciseRDFList = false;
	/** Strategy triple pattern */
	private List<ElementTriplePattern> elts;
	/** Hold the strategy subject variable name */
	private String impreciseTriplePatternSubject;
	/** Hold the imprecise triple pattern strategy subject names */
	private Set<String> impreciseTriplePatternSubjects = new TreeSet<String>();
	/** Hold the optimized imprecise block keys (the strategy subject variables) */
	private List<String> impreciseOptimizedBlockKeys = new ArrayList<String>();
	/** Reorder blocks because of between / threshold properties */
	private boolean processReorderImpreciseBlocks = false;
	/** Check whether it is an imprecise sparql query */
	private boolean isImprecise = false;
	
	/**
	 * Constructor, just takes some arguments, and initialize the iSPARQL URI-references registry.
	 * 
	 * @param inElementGroup
	 */
	public ElementGroupPrepare(ElementGroup inElementGroup)
	{
		URIRegistry urireg = new URIRegistry();
		
		this.inElementGroup = inElementGroup;
		this.strategyPropertyURI = urireg.getStrategyPropertyURI();
	}
	
	/**
	 * Process logic when preparing the input query.
	 * The input query element group will be processed.
	 */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get the sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * Check whether the triple pattern is a belongs to an imprecise sparql block.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{
		String predicate = null;
		
		if (el.getTriple().getPredicate().isURI())
			predicate = el.getTriple().getPredicate().getURI();
		
		Node object = el.getTriple().getObject();
		
		// The element triple pattern predicate is a strategy property URI
		if (strategyPropertyURI.contains(predicate))
		{
			// We found a imprecise pattern, mark the query as imprecise
			isImprecise = true;
			
			// Get the strategy subject variable name
			impreciseTriplePatternSubject = el.getTriple().getSubject().toString();
			String subject = impreciseTriplePatternSubject;
			
			elts = impreciseBlocks.get(impreciseTriplePatternSubject);
			
			if (elts == null)
				elts = new ArrayList<ElementTriplePattern>();
			
			// Add the triple pattern to the imprecise block
			elts.add(el);
			// Add the imprecise block elements to the map
			impreciseBlocks.put(impreciseTriplePatternSubject, elts);
			// Add the subject to the set
			impreciseTriplePatternSubjects.add(subject);
			
			/* Check if treshold or between predicate, to know if we need to optimize
			 * Do the check only if the flag is still false */
			if (processReorderImpreciseBlocks == false 
					&& predicate.equals(urireg.get(StrategyProperty.threshold)))
				processReorderImpreciseBlocks = true;
		} 
		else if (isImpreciseRDFList)
		{
			/* If the semaphore is set to true, we found a triple pattern 
			 * with an RDF list as object. Add the triple pattern to the same
			 * element triple pattern list as the last element visited
			 * We exploit the fact that an imprecise RDF list is always listed
			 * after the opening triple pattern of the RDF list visited, e.g. 
			 * for arguments property, the list (?var1 ?var2) is always visited
			 * after the arguments property element, thus the semaphore is set
			 * true. */
			String subject = el.getTriple().getSubject().toString();
			elts.add(el);
			// Add the subject of the RDF list to the set
			impreciseTriplePatternSubjects.add(subject);
		}
		
		// If we don't have a imprecise sparql query, just return
		if (! isImprecise)
			return;
			
		// For this properties, we need to set the RDF list semaphore to true
		if (predicate.equals(urireg.get(StrategyProperty.arguments))
				|| predicate.equals(urireg.get(StrategyProperty.between))
				|| predicate.equals(urireg.get(StrategyProperty.ontologyBaseURI))
				|| predicate.equals(urireg.get(StrategyProperty.ontologyRootURI))
				|| predicate.equals(urireg.get(StrategyProperty.ontologyURI))
				|| predicate.equals(urireg.get(StrategyProperty.scores))
				|| predicate.equals(urireg.get(StrategyProperty.weights)))
			isImpreciseRDFList = true;
		
		if (object.isURI())
		{
			// If the predicate of the element triple pattern is RDF.nil we set the semaphore to false
			if (object.getURI().equals(RDF.nil.toString()))
				isImpreciseRDFList = false;
		}
	}
	
	// Walk throught the query element group
	private void processElementGroup()
	{
		// Record stats for variables found in WHERE clause
		ElementWalker.walk(inElementGroup, this);
	}
	
	/** 
	 * Return the list of ARQ triple pattern including the imprecise statements
	 * which are now reordered, in order to allow a correct execution.
	 * 
	 * @return List<ElementTriplePattern>
	 */
	public List<ElementTriplePattern> getImpreciseTriplePattern()
	{
		/* Reorder the imprecise sparql blocks by threshold property 
		 * This is an optimization: blocks defining a high threshold
		 * (i.e. less possible similarities) should be executed first,
		 * since they constrain the results. If a similarity calculated by a strategy
		 * doesn't fit into the threshold constrains, the binding 
		 * is dropped, thus all following triple pattern are ignored by ARQ */
		reorderImpreciseBlocks();
		
		/* Reorder the imprecise sparql blocks triple pattern, 
		 * ie property name & similarity and score aggregator strategy */
		return reorderImpreciseTriplePattern();
	}
	
	/**
	 * A set of all imprecise subject variable names, i.e. all the strategies
	 * 
	 * @return Set<String>
	 */
	public Set<String> getImpreciseSubjects()
	{
		return impreciseTriplePatternSubjects;
	}
	
	// Implements the logic to reorder imprecise statements
	private List<ElementTriplePattern> reorderImpreciseTriplePattern()
	{
		boolean isAggregatorStrategy = false;
		List<ElementTriplePattern> tempAggregatorPattern = new ArrayList<ElementTriplePattern>();
		List<ElementTriplePattern> impreciseTriplePattern = new ArrayList<ElementTriplePattern>();
		
		// Reorder imprecise blocks
	    for (Iterator iter = impreciseOptimizedBlockKeys.iterator(); iter.hasNext(); )
		{
	    	ElementTriplePattern elName = null;
			ElementTriplePattern elSimilarity = null;
			List<ElementTriplePattern> temp = new ArrayList<ElementTriplePattern>();
			
			/* Extract the block from the map in a previously defined optimal order */
			List<ElementTriplePattern> elts = impreciseBlocks.get((String)iter.next());
			
			for (Iterator it = elts.iterator(); it.hasNext(); )
			{
				ElementTriplePattern el = (ElementTriplePattern)it.next();
				String predicate = el.getTriple().getPredicate().getURI();
				String object = el.getTriple().getObject().toString(false);
				
				if (predicate.equals(urireg.get(StrategyProperty.name)))
				{
					// The element predicate is a strategy name property, mark it
					elName = el;
					// Check if the object is an aggregator strategy, this has to be at the bottom
					if (object.equals(AbstractStrategy.scoreAggregator))
						isAggregatorStrategy = true;
				}
				else if (predicate.equals(urireg.get(StrategyProperty.similarity)))
					// The element predicate is a strategy similarity property, mark it
					elSimilarity = el;
				else
					// Add the element to a temporary list
					temp.add(el);
			}
			
			if (isAggregatorStrategy)
			{
				/* If we found an aggregator strategy, save the patterns to a 
				 * temporary list. Add this triple patterns at the bottom */
				tempAggregatorPattern.add(elName);
				
				for (Iterator it = temp.iterator(); it.hasNext(); )
					tempAggregatorPattern.add((ElementTriplePattern)it.next());
				
				tempAggregatorPattern.add(elSimilarity);
				isAggregatorStrategy = false;
			}
			else 
			{
				// Add the strategy name property first
				impreciseTriplePattern.add(elName);
			
				for (Iterator it = temp.iterator(); it.hasNext(); )
					impreciseTriplePattern.add((ElementTriplePattern)it.next());
			
				// Add the strategy similarity property last
				impreciseTriplePattern.add(elSimilarity);
			}
		}
		
		// Add the aggregator pattern at the bottom
		for (Iterator iter = tempAggregatorPattern.iterator(); iter.hasNext(); )
			impreciseTriplePattern.add((ElementTriplePattern)iter.next());
		
		return impreciseTriplePattern;
	}
	
	// Check whether we have to reorder strategies against the threshold
	private void reorderImpreciseBlocks()
	{
		if (processReorderImpreciseBlocks)
			// We need to reorder the imprecise sparql blocks by threshold
			impreciseOptimizedBlockKeys.addAll(processReorderImpreciseBlocks());
		else
			// Nothing to optimize, there is no threshold property defined
			impreciseOptimizedBlockKeys.addAll(impreciseBlocks.keySet());
	} 
	
	// Implements the logic to reorder strategies against the threshold, reduce the selectivity
	private List<String> processReorderImpreciseBlocks()
	{
		Map<Double, List<String>> map = new TreeMap<Double, List<String>>();
		
		for (Iterator iter = impreciseBlocks.keySet().iterator(); iter.hasNext(); )
		{
			Double threshold = new Double(0);
			String key = (String)iter.next();
			// Imprecise block triple elements
			List<ElementTriplePattern> elts = impreciseBlocks.get(key);
		
			for (Iterator it = elts.iterator(); it.hasNext(); )
			{
				ElementTriplePattern el = (ElementTriplePattern)it.next();
				String predicate = el.getTriple().getPredicate().getURI();
				
				// If the predicate is threshold property, save the specific threshold
				if (predicate.equals(urireg.get(StrategyProperty.threshold)))
					threshold = new Double(el.getTriple().getObject().getLiteralValue().toString());
			}
			
			// The the list of keys for this threshold
			List<String> list = map.get(threshold);
			
			if (list == null)
				list = new ArrayList<String>();
			
			// Add the new key to the list
			list.add(key);
			
			// Add the list to the map of threshold => strategy subject variables
			map.put(threshold, list);
		}
	
		// The ordered set of imprecise 
		List<String> list = new ArrayList<String>();
		List<Double> keyList = new ArrayList<Double>();
		
		// Get a list of items to traverse in reverse order
		for (Iterator iter = map.keySet().iterator(); iter.hasNext(); )
			keyList.add((Double)iter.next());
		
		// Traverse the list in reverse order
		for (int i = keyList.size() - 1; i >= 0; i--)
			// Add the lists with strategy subject variables for each threshold
			list.addAll((List<String>)map.get(keyList.get(i)));
		
		return list;
	}
	
	/**
	 * Check whether the query is an iSPARQL query
	 * 
	 * @return boolean
	 */
	public boolean isImprecise()
	{
		return isImprecise;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
