package ch.unizh.ifi.isparql.optimizer.rule3.visitor;

import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the transform stage. We use the information extracted from
 * the previous prepare stage to transform and optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupTransform extends ElementVisitorBase
{
	/** Mark the query as imprecise sparql */
	private boolean isImprecise = false;
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** The output query element group (optimized) */
	private ElementGroup outElementGroup = null;
	/** List of distinct imprecise statement subjects, ie ?strategyN variables */
	private Set<String> impreciseSubjects;
	/** List of imprecise statements of the query */
	private List<ElementTriplePattern> impreciseTriplePattern;
	
	/**
	 * Constructor, takes just a list of params
	 * 
	 * @param inElementGroup
	 * @param outElementGroup
	 * @param impreciseSubjects
	 * @param impreciseTriplePattern
	 * @param isImprecise
	 */
	public ElementGroupTransform(ElementGroup inElementGroup, ElementGroup outElementGroup, Set<String> impreciseSubjects, List<ElementTriplePattern> impreciseTriplePattern, boolean isImprecise)
	{	
		this.inElementGroup = inElementGroup;
		this.outElementGroup = outElementGroup;
		this.impreciseSubjects = impreciseSubjects;
		this.impreciseTriplePattern = impreciseTriplePattern;
		this.isImprecise = isImprecise;
	}
	
	/** Process logic when transforming the input query */
	public void exec()
	{
		processElementGroup();
		
		if (isImprecise)
			// Do this only if the query is marked as imprecise sparql
			processImpreciseTriplePattern();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * In case we find an imprecise element triple pattern, we do not add it
	 * to the output element group. All imprecise element triple pattern are
	 * added at the bottom after.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{
		// Add first all element triple patterns that are not of imprecise block
		if (!isImprecise || !impreciseSubjects.contains(el.getTriple().getSubject().toString()))
			outElementGroup.addElementTriplePattern(el);
	}
	
	/** Iterate over imprecise statements and add them to the ouput ARQ element group */
	public void processImpreciseTriplePattern()
	{
		for (Iterator it = impreciseTriplePattern.iterator(); it.hasNext(); )
			outElementGroup.addElementTriplePattern((ElementTriplePattern)it.next());
	}
	
	/**
	 * If an ElementFilter is found, just add it to the output element group
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{
		outElementGroup.addElementFilter(el);
	}
	
	/**
	 * Return the output ARQ element group
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}

	// Walk throught the input element group
	private void processElementGroup()
	{
		ElementWalker.walk(inElementGroup, this);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
