package ch.unizh.ifi.isparql.optimizer.rule4.visitor;

import ch.unizh.ifi.isparql.optimizer.CostModel;
import ch.unizh.ifi.isparql.query.URIRegistry;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import com.hp.hpl.jena.graph.Node;
import java.util.*;

/**
 * The prepare stage of reorder by selectivity rule. We mainly calculate the cost
 * for each SPARQL triple pattern found in query. The cost is used as a ranking
 * in transform stage to reorder SPARQL patterns by their selectivity.
 * 
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupPrepare extends ElementVisitorBase
{	
	/** The cost for a triple pattern */
	private Double cost;
	/** The cost model object */
	private CostModel cm;
	/** Flag iSPARQL queries as imprecise */
	private boolean isImprecise = false;
	/** List of iSPARQL imprecise statements property URI-references */
	private List<String> strategyPropertyURI = new ArrayList<String>();
	/** List of distinct cost stages with corresponding triple pattern elements */
	private Map<Double, List<Element>> stages = new TreeMap<Double, List<Element>>();
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** List of variables which were found previously in the query */
	private List<String> previousVariables = new ArrayList<String>();
	
	/**
	 * Constructor, just take some param
	 * 
	 * @param inElementGroup
	 * @param cm
	 */
	public ElementGroupPrepare(ElementGroup inElementGroup, CostModel cm)
	{
		URIRegistry urireg = new URIRegistry();
		
		// The cost model
		this.cm = cm;
		this.inElementGroup = inElementGroup;
		this.strategyPropertyURI = urireg.getStrategyPropertyURI();
	}
	
	/**
	 * Process logic when preparing the input query
	 * The input query element group is processed
	 */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get the sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * Check whether the predicate is an imprecise SPARQL property. For
	 * SPARQL triple patterns, the cost of the element (triple pattern)
	 * is calculated.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{
		String predicate = null;
		Node subject = el.getTriple().getSubject();
		Node object = el.getTriple().getObject();	
		
		if (el.getTriple().getPredicate().isURI())
			predicate = el.getTriple().getPredicate().getURI();
		
		// For the first imprecise sparql pattern, set the flag to mark that
		// from now, every triple pattern is for iSPARQL
		if (!isImprecise && strategyPropertyURI.contains(predicate))
			isImprecise = true;
		
		if (isImprecise)
			// For iSPARQL we set the cost to max (at least > 1.0)
			cost = Double.MAX_VALUE;
		else
		{
			if (object.isVariable())
			{
				// If the object is a variable, check if it is already bound
				if (previousVariables.contains(object.getName()))
					cm.objectVariableIsBound(true);
			}
			
			cost = cm.costForTriplePattern(el);
		}

		// Get the elements list for this stage cost
		List<Element> elements = stages.get(cost);
		
		// If there isn't a elements list for this stage, create it
		if (elements == null)
			elements = new ArrayList<Element>();
		
		// Add this element to the element list
		elements.add(el);
		
		// Add the subject/object variable to the list, in order to remember 
		// variables that are already bound
		if (subject.isVariable())
			previousVariables.add(subject.getName());
		if (object.isVariable())
			previousVariables.add(object.getName());
		
		cm.objectVariableIsBound(false);
		
		stages.put(cost, elements);	
	}
	
	/**
	 * Process a FILTER element
	 */
	public void visit(ElementFilter el)
	{
		/*
		 * The FILTER has the same cost as the last visited element triple pattern
		 * to make sure, that the FILTER is at the right place while reordering
		 */
		List<Element> elements = stages.get(cost);
		elements.add(el);
	}
	
	// Walk throught the query element group
	private void processElementGroup()
	{	
		// Record stats for variables found in WHERE clause
		ElementWalker.walk(inElementGroup, this);
	}
	
	/**
	 * Return a map of cost stages with all elements for this stage.
	 * Query triple patterns with the same cost, are contained in the same stage.
	 * Thus the map contains the double value (cost) as key and as value 
	 * a list of all triple pattern elements within this cost stage.
	 * This map is used during transform stage of this rule, to reorder
	 * triple patterns against the cost stage, ordered by increasing cost.
	 * 
	 * @return Map<Double, List<Element>>
	 */
	public Map<Double, List<Element>> getCostStages()
	{	
		return stages;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
