package ch.unizh.ifi.isparql.optimizer.rule5.visitor;

import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.SortCondition;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import java.util.*;

/**
 * This class provides the logic used in prepare stage of the 
 * remove dispensable pattern rule. We iterate over the query, to 
 * gather information about the occurrences of variables.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupPrepare.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupPrepare extends ElementVisitorBase
{	
	/** The input query */
	private Query inQuery = null;
	/** Object for query variable statistics */
	private QueryVarStats queryVarStats = null;
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	
	/**
	 * Constructor
	 * 
	 * @param inQuery
	 * @param inElementGroup
	 */
	public ElementGroupPrepare(Query inQuery, ElementGroup inElementGroup)
	{
		this.inQuery = inQuery;
		this.inElementGroup = inElementGroup;
		this.queryVarStats = new QueryVarStats();
	}
	
	/**
	 * Process logic when preparing the input query.
	 * First the projection variables are inspected.
	 * Then the input query element group is processed (var stats and FILTER processing).
	 * Last the ORDER BY clause variables are processed.
	 */
	public void exec()
	{
		processProjection();
		processElementGroup();
		processOrderBy();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get the sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern. Increment the statistics 
	 * of S/P/O variables which may be found in the element.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{	
		Node subject = el.getTriple().getSubject();
		Node predicate = el.getTriple().getPredicate();
		Node object = el.getTriple().getObject();
			
		if (subject.isVariable())
			queryVarStats.inc(subject.getName());
			
		if (predicate.isVariable())
			queryVarStats.inc(predicate.getName());
			
		if (object.isVariable())
			queryVarStats.inc(object.getName());
	}
	
	/**
	 * Visit a FILTER element. Increment variable stats for variables found in FILTER.
	 * 
	 * @param el
	 */
	public void visit(ElementFilter el) 
	{	
		for (Iterator iter = el.varNamesMentioned().iterator(); iter.hasNext(); )
			queryVarStats.inc((String)iter.next());
	}

	// Make statistics about the projection variables
	private void processProjection()
	{	
		// Record stats for projection variables
		for (Iterator iter = inQuery.getResultVars().iterator(); iter.hasNext(); )
			queryVarStats.inc((String)iter.next());
	}
	
	// Walk throught the query element group
	private void processElementGroup()
	{
		// Record stats for variables found in WHERE clause
		ElementWalker.walk(inElementGroup, this);
	}
	
	// Make statistics about the order by variables
	private void processOrderBy()
	{
		// Record order by variables
		if (inQuery.hasOrderBy())
		{
			for (Iterator iter = inQuery.getOrderBy().iterator(); iter.hasNext(); )
			{
				SortCondition sortCondition = (SortCondition)iter.next();
				queryVarStats.inc(sortCondition.getExpression().getVarName());
			}
		}
	}
	
	/**
	 * Return the object which holds the statistical information about the 
	 * variables found in query.
	 * 
	 * @return QueryVarStats
	 */
	public QueryVarStats getQueryVarStats()
	{
		return queryVarStats;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
