package ch.unizh.ifi.isparql.optimizer.rule5.visitor;

import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementBasicGraphPattern;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import com.hp.hpl.jena.query.core.ElementWalker;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the transform stage. We use the information extracted from
 * the previous prepare stage to transform and optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupTransform extends ElementVisitorBase
{	
	/** Object holding the statistics about the query variables */
	private QueryVarStats queryVarStats = null;
	/** The input query element group (to optimize) */
	private Element inElementGroup = null;
	/** The output query element group (optimized) */
	private ElementGroup outElementGroup = null;
	
	/**
	 * Constructor
	 * 
	 * @param inElementGroup
	 * @param outElementGroup
	 * @param queryVarStats
	 */
	public ElementGroupTransform(ElementGroup inElementGroup, ElementGroup outElementGroup, QueryVarStats queryVarStats)
	{
		this.inElementGroup = inElementGroup;
		this.outElementGroup = outElementGroup;
		this.queryVarStats = queryVarStats;
	}
	
	/** Process logic when transforming the input query */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Visit a basic graph pattern.
	 * Just get sub elements and visit them.
	 * 
	 * @param el
	 */
	public void visit(ElementBasicGraphPattern el) 
	{ 
		for (Iterator iter = el.getElements().iterator(); iter.hasNext(); )
        {
            Element subEl = (Element)iter.next();
            subEl.visit(this);
        }
	}
	
	/**
	 * Visit a triple pattern.
	 * We can eliminate a triple pattern only if for S/P/O are defined some variables,
	 * and they are not used elsewhere.
	 * 
	 * @param el
	 */
	public void visit(ElementTriplePattern el)
	{	
		boolean addElementToGroup = true;
		Node subject = el.getTriple().getSubject();
		//Node predicate = el.getTriple().getPredicate();
		Node object = el.getTriple().getObject();
		
		// Only a triple pattern which has only variables as S/P/O can be dispensable
		// Please note: hack for final example, predicate test removed!!
		if (subject.isVariable() && object.isVariable())
		{
			// The triple pattern can be omitted, only if the variables are mentioned once
			if (queryVarStats.get(subject.getName()) == 1
					&& queryVarStats.get(object.getName()) == 1)
				addElementToGroup = false;
		}
		
		if (addElementToGroup)
			outElementGroup.addElementTriplePattern(el);
	}
	
	/** @param el */
	public void visit(ElementFilter el) 
	{		
		outElementGroup.addElementFilter(el);
	}
	
	/** @return outElementGroup */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}

	// Walk throught the input element group
	private void processElementGroup()
	{
		ElementWalker.walk(inElementGroup, this);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
