package ch.unizh.ifi.isparql.query;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.util.iterator.ClosableIterator;
import com.hp.hpl.jena.graph.GraphUtil;
import com.hp.hpl.jena.graph.Triple;
import java.util.*;

/**
 * Pattern class. A pattern is a pair of nodes which can be of complex type or simple.
 * The purpose is to identify if the nodes are URI-references. For simple nodes
 * we just add the two nodes as triple elements. For a complex pattern, we parse
 * the undelying graph to identify the predicates of the complex pattern. We save
 * all triples found to the elements list. The class is used for LevenshteinResCmp
 * similarity strategy, to recurse over resources and calculate a similarity
 * which considers not only a single predicate, but all predicates of a resource, 
 * and moreover predicates which specifies a URI-reference as object are 
 * recursively resolved and the predicates of the referenced resource are
 * considered for the similarity computation.
 *
 * @author		Markus Stocker
 * @version 	$Id: Pattern.java,v 1.2 2006-11-21 12:41:56 ms Exp $
 */

public class Pattern 
{
	private List<Triple> elements1 = new ArrayList<Triple>();
	private List<Triple> elements2 = new ArrayList<Triple>();
	private Graph graph = null;
	private Node node1 = null;
	private Node node2 = null;
	private boolean isComplex = false;
	
	public Pattern(Graph graph, Node node1, Node node2)
	{
		this.graph = graph;
		this.node1 = node1;
		this.node2 = node2;
		
		if (this.node1.isURI() && this.node2.isURI())
			isComplex = true;
		
		this.parse();
	}
	
	// Parse the undelying graph depending whether the pattern is complex or not
	private void parse()
	{
		if (isComplex) 
			parseComplexPattern();
		else 
			parseSimplePattern();
	}
	
	// Parse a complex pattern, find all triples for the nodes of the pattern
	private void parseComplexPattern()
	{
		ClosableIterator it = GraphUtil.findAll( graph );
		while (it.hasNext()) 
		{
			Triple triple = (Triple)it.next();
			Node subject = triple.getSubject();
			if (subject.equals(node1))
				elements1.add(triple);
			if (subject.equals(node2))
				elements2.add(triple);
		}
	}

	// Parse simple pattern, just add a triple for both nodes
	private void parseSimplePattern()
	{
		isComplex = false;
		
		Node node = Node.createAnon();
		
		elements1.add(new Triple(node,node,node1));
		elements2.add(new Triple(node,node,node2));
	}
	
	/**
	 * This method provides a way to find the corresponding triple of the second
	 * (complex) node given the predicate of the first node. There are multiple ways
	 * how to implement this logic. We search here for exact predicate matches. 
	 * We could also do a Levenshtein sequence match between a predicate and all 
	 * other predicates of the second node and return the triple which highest score.
	 * This way would match also predicates like tel and telephone between two 
	 * resources.
	 * 
	 * @param predicate1
	 * @return Triple
	 */
	public Triple findTripleForPredicate(Node predicate1)
	{		
		for (Iterator iter = elements2.iterator(); iter.hasNext(); )
		{
			Triple triple2 = (Triple)iter.next();
			Node predicate2 = triple2.getPredicate();
			if (predicate1.toString().equals(predicate2.toString()))
				return triple2;
		}
		
		return null;
	}
	
	/**
	 * Get the first node of the pattern
	 * 
	 * @return Node
	 */
	public Node getNode1()
	{	
		return node1;
	}
	
	/**
	 * Get the second node of the pattern
	 * 
	 * @return Node
	 */
	public Node getNode2()
	{
		return node2;
	}
	
	/**
	 * Return the triples array for a node
	 * 
	 * @param node
	 * @return List
	 */
	public List getTriplesFor(Node node)
	{
		if (node.equals(node1))
			return elements1;
		if (node.equals(node2))
			return elements2;
		
		return null;
	}
	
	/** 
	 * Mark if the pattern is complex. A pattern is complex, 
	 * when both node1 and node2 are URI-references.
	 * 
	 * @return boolean 
	 */
	public boolean isComplex()
	{
		return isComplex;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
