package ch.unizh.ifi.isparql.query;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.exception.StrategyNotDefined;
import ch.unizh.ifi.isparql.query.strategy.*;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Graph;

/**
 * Factory for strategies. This class returns an instance of the iSPARQL
 * similarity strategy which corresponds to the specified strategy
 * keyword (name). Strategy names are organized in AbstractStrategy class.
 * If a new strategy is added to the framework, we need to add it to 
 * this factory class and to the AbstractStrategy class, where the 
 * corresponding name is specified as a public field. Moreover, strategies
 * are identified by a URI-reference, thus a corresponding URI-reference
 * has to be created in URIRegistry class. Please note the two different
 * create() methods. 
 * 
 * @see ch.unizh.ifi.isparql.query.strategy
 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
 * @see ch.unizh.ifi.isparql.query.URIRegistry
 *
 * @author		Markus Stocker
 * @version 	$Id: StrategyFactory.java,v 1.3 2006-11-21 18:31:46 ms Exp $
 */

public class StrategyFactory 
{	
	/**
	 * This method is used to instantiate similarity strategies during iSPARQL query processing. 
	 * In fact, during query processing, we have an ARQ binding, S/P/O and ARQ execution context,
	 * which are all relevant for iSPARQL strategy processing. In this case, the strategy name
	 * (keyword, which must correspond to one defined in AbstractStrategy) must be 
	 * stored as string in the object node of a imprecise statement (virtual triple).
	 * For example ?strategy1 isparql:name "LevenshteinStrCmp" is the directive to the iSPARQL
	 * framework to create a LevenshteinStrCmp similarity strategy. The Java class 
	 * which is invoqued by this imprecise statement (isparql:name), instantiates
	 * the similarity strategy using this StrategyFactory. Please note, that the 
	 * LevenshteinStrCmp specified by the object, must be configured by the AbstractStrategy.
	 * If an unknown strategy name is specified in query, this create() method throws
	 * an error.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return AbstractStrategy
	 */
	public static AbstractStrategy create(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{	
		// Extract the strategy name from triple object (member)
		String strategy = object.toString(false);
		
		if (strategy.equals(AbstractStrategy.levenshteinStrCmp))
			return new LevenshteinStrCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.levenshteinResCmp))
			return new LevenshteinResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.linResCmp))
			return new LinResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.resnikResCmp))
			return new ResnikResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.shortestPathResCmp))
			return new ShortestPathResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.MITPHLevenshteinResCmp))
			return new MITPHLevenshteinResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.MITPHTreeEditDistanceResCmp))
			return new MITPHTreeEditDistanceResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.subGraphIsomorphismResCmp))
			return new SubgraphIsomorphismResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.scoreAggregator))
			return new ScoreAggregator(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.cosineStrCmp))
			return new CosineStrCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.jensenShannonStrCmp))
			return new JensenShannonStrCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.owlstcServicesCmp))
			return new OWLSTCServicesCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.MITPHLoLResCmp))
			return new MITPHLoLResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.MITPHTFIDFResCmp))
			return new MITPHTFIDFResCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.LOLStrCmp))
			return new LOLStrCmp(binding, subject, predicate, object, execCxt);
		if (strategy.equals(AbstractStrategy.diceStrCmp))
			return new DiceStrCmp(binding, subject, predicate, object, execCxt);
		
		// The requested strategy is not implemented, get an error and exit
		new StrategyNotDefined("StrategyFactory", strategy);
		
		return null;
	}
	
	/**
	 * This method is slightly different to the method above. This one is used
	 * when a similarity index is created. In order to reference a specific similarity
	 * strategy which should be used to create the index, we create another create()
	 * factory method for similarity strategies. Since during similarity indexing
	 * we do not have a ARQ binding, S/P/O and query execution context, the
	 * param used are not the same. Nevertheless, the name of the strategy (must
	 * be one configured in AbstractStrategy), and the underlying graph is
	 * required to calculate the similarity against a strategy. Please note, 
	 * that not all available similarity strategy can be invoqued yet. Since,
	 * we used only a few strategy for similarity index, only the following 
	 * can be instantiated over this method: linResCmp, diceStrCmp, LOLStrCmp.
	 * If a strategy name is given which is not specified in this factory method,
	 * the class throws an error.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
	 * @param strategy
	 * @param graph
	 * @return AbstractStrategy
	 */
	public static AbstractStrategy create(String strategy, Graph graph)
	{
		if (strategy.equals(AbstractStrategy.linResCmp))
			return new LinResCmp(graph);
		if (strategy.equals(AbstractStrategy.diceStrCmp))
			return new DiceStrCmp(graph);
		if (strategy.equals(AbstractStrategy.LOLStrCmp))
			return new LOLStrCmp(graph);
		
		// The requested measure is not implemented, get an error and exit
		new StrategyNotDefined("StrategyFactory", strategy);
		
		return null;
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
