package ch.unizh.ifi.isparql.query;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import java.util.HashMap;
import java.util.Map;

/**
 * Registry for similarity strategies. The registry is saved to the engine execution context.
 * This registry is required to allow the configuration and execution of similarity strategies
 * in different stages of ARQ iSPARQL query processing. Since, multiple imprecise statements are 
 * specified to configure and execute a similarity strategy, we need to store somewhere
 * the strategy configuration during query processing. We selected the ARQ query execution 
 * context, since this context is assigned to each ARQ property function (isparql:*).
 * Thus, we can extract the similarity strategy from this registry on each 
 * imprecise statement directive, and store a new configuration param when the corresponding
 * imprecise statement is invoqued, or execute the similarity strategy when
 * isparql:similarity is executed by ARQ. The similarity strategy is referenced by
 * a unique URI-reference, which is stored in the subject variable of a imprecise statement
 * (virtual triple). The first imprecise statement (ispraql:name) instantiates and 
 * binds this URI-reference to the corresponding variable to ARQ bindings. For instance,
 * ?strategy1 isparql:name "LevenshteinResCmp", creates a URI-reference 
 * http://www.ifi.unizh.ch/isparql/strategy/LevenshteinResCmp#ID0 which is then bind
 * to the variable ?strategy1. Afterwards, when for example the imprecise statement
 * ?strategy1 isparql:threshold 0.5 is processed by ARQ, the URI-reference previously
 * bound to the variable ?strategy1 is read by the isparql:threshold Java class, the 
 * StrategyRegistry class (this) is extracted from the ARQ query execution context, then
 * the similarity strategy corresponding to the URI-reference (?strategy1) is extratected 
 * from the StrategyRegistry. Thus, we have the previously instantiated similarity strategy
 * object (isparql:name instantiates similarity strategies) and we add the 0.5 threshold
 * to the similarity strategy object. Similarity when ARQ processes the
 * ?strategy1 isparql:similarity ?similarity1 imprecise statement, the corresponding 
 * strategy is extracted from the StrategyRegistry and the similarity is calculated in 
 * respect for the configuration, by using the strategy specified in isparql:name.
 * Please note, that this explication, explicates why imprecise statements for
 * similarity strategies must follow a specific ordering (ie, isparql:name must be 
 * executed first, and isparql:similarity last).
 *
 * @author		Markus Stocker
 * @version 	$Id: StrategyRegistry.java,v 1.2 2006-11-21 12:41:56 ms Exp $
 */

public class StrategyRegistry 
{
	/**
	 * Map URI-references with similarty strategies.
	 * (eg, http://www.ifi.unizh.ch/isparql/strategy/LevenshteinResCmp#ID0).
	 * URI-references for strategy are unique (# fragment).
	 * Mapping: URI-reference (string) => AbstractStrategy
	 */
	Map<String, AbstractStrategy> registry = new HashMap<String, AbstractStrategy>();
	
	public StrategyRegistry() { }
	
	/**
	 * Put a strategy identified by a unique URI to the registry
	 * 
	 * @param uri
	 * @param AbstractStrategy
	 */
	public void put(String uri, AbstractStrategy strategy)
	{
		registry.put(uri, strategy);
	}
	
	/**
	 * Get the strategy from the registry by URI
	 * 
	 * @param uri
	 * @return Strategy
	 */
	public AbstractStrategy get(String uri)
	{		
		return registry.get(uri);
	}
	
	/**
	 * This method allows creation of new similarity strategy URI-references,
	 * or more precisely, the # fragment. Basically the method identifies the
	 * size of the registry which is incremented by 1. The new integer is 
	 * then used as ID fragment for the newly created URI-reference which
	 * is afterwards bound to the subject variable of the isparql:name 
	 * imprecise statement.
	 * 
	 * @see ch.unizh.ifi.isparql.query.property.name
	 * 
	 * @return String
	 */
	public String nextMeasureId()
	{
		int id = registry.size();
		
		return "#ID" + id++;
	}
	
	/** The method allows clearing the registry */
	public void clear()
	{
		registry.clear();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
