package ch.unizh.ifi.isparql.query.api.impl;

import ch.unizh.ifi.isparql.query.api.IAggregator;
import ch.unizh.ifi.isparql.query.optimizer.AggregatorOptimizer;
import java.util.*;

/**
 * Abstract class of aggregation methods for similarity scores.
 * This class defines all available aggregation methods as public static final strings.
 * The class implements the interface.
 *
 * @see 		ch.unizh.ifi.isparql.query.aggregator
 * @see			ch.unizh.ifi.isparql.query.api.IAggregator
 * @author		Markus Stocker
 * @version 	$Id: AbstractAggregator.java,v 1.3 2006-11-21 18:31:46 ms Exp $
 */

public abstract class AbstractAggregator implements IAggregator 
{
	/** The URI-reference for the aggregation method, deprecated */
	protected String uri = null;
	/** The name used for the aggregation method */
	protected String name = null;
	/** The optimizer for aggregation lists, not used, irrelevant */
	protected AggregatorOptimizer optimizer = new AggregatorOptimizer();
	/** The list of similarity scores */
	private List<Double> scores = new ArrayList<Double>();
	/** The list of score weights, to consider on score aggragation */
	private List<Double> weights = new ArrayList<Double>();
	
	// Definition of all available aggregators
	// Add new aggregators to the method isDefined!
	/** The name of simple aggregation method, simple */
	public static final String simple = new String("simple");
	/** The name of average aggregation method, avg */
	public static final String avg = new String("avg");
	/** The name of max aggregation method, max */
	public static final String max = new String("max");
	/** The name of min aggregation method, min */
	public static final String min = new String("min");
	/** The name of sum aggregation method, sum */
	public static final String sum = new String("sum");
	/** The name of median aggregation method, median */
	public static final String median = new String("median");
	/** The name of noisy-and aggregation method, noisyand */
	public static final String noisyand = new String("noisyand");
	/** The name of noisy-or aggregation method, noisyor */
	public static final String noisyor = new String("noisyor");
	/** The name of geometric aggregation method, geometric */
	public static final String geometric = new String("geometric");
	/** The name of harmonic aggregation method, harmonic */
	public static final String harmonic = new String("harmonic");
	
	/**
	 * Constructor. Init the aggregator optimizer, 
	 * should be deleted, deprecated and not used.
	 * I leave it for historic reason. The score list 
	 * aggregation optimizer was never used, nor evaluated.
	 */
	public AbstractAggregator() 
	{
		optimizer.setAggregatorName(getName());
		optimizer.setAggregatorDefaultValue(defaultValue());
	}
	
	/** 
	 * This method is used to check if an aggregation method
	 * is defined. As parameter, a AbstractAggregator.* should
	 * be used, eg. AbstractAggregator.max
	 * 
	 * @param aggregator 
	 */
	public static boolean isDefined(String aggregator)
	{
		if (aggregator.equals(simple))
			return true;
		if (aggregator.equals(avg))
			return true;
		if (aggregator.equals(max))
			return true;
		if (aggregator.equals(min))
			return true;
		if (aggregator.equals(sum))
			return true;
		if (aggregator.equals(median))
			return true;
		if (aggregator.equals(noisyand))
			return true;
		if (aggregator.equals(noisyor))
			return true;
		if (aggregator.equals(geometric))
			return true;
		if (aggregator.equals(harmonic))
			return true;
		
		return false;
	}
	
	/** @param value */
	public void setURI(String value)
	{
		uri = value;
	}
	
	/** @return String */
	public String getURI()
	{
		return uri;
	}
	
	/** @param value */
	public void setName(String value)
	{
		name = value;
	}
	
	/**
	 * Add a score to the list of scores 
	 * which are aggregated
	 * 
	 * @param value
	 */
	public void addScore(Double value)
	{
		scores.add(value);
	}
	
	/**
	 * Add a weight to the list of weights
	 * which are considered during score aggregation.
	 * 
	 * @param value
	 */
	public void addWeight(Double value)
	{	
		weights.add(value);
	}
	
	/**
	 * Allows adding a list of values for the
	 * scores that are aggregated with the 
	 * selected aggregation method.
	 * 
	 * @param values
	 */
	public void addScores(List<Double> values)
	{
		scores.addAll(values);
	}
	
	/**
	 * Allows adding a list of values for the
	 * weights that are used when a corresponding
	 * list of scores is aggregated with the
	 * selected aggregation method.
	 * 
	 * @param values
	 */
	public void addWeights(List<Double> values)
	{
		weights.addAll(values);
	}
	
	/** Clear the scores / weights list */
	public void clear()
	{
		scores.clear();
		weights.clear();
	}
	
	/**
	 * Aggregate a list of previously setted scores.
	 * Consider the weights, if the size of the list
	 * of scores corresponds to the size of the list
	 * of weights.
	 * 
	 * @return Double
	 */
	public Double aggregate()
	{
		List<Double> weighted = new ArrayList<Double>();
		
		// Optimization: if a score i 0 for some aggregators (ex. geometric)
		// we don't need to aggregate, just return 0
		for (int i = 0; i < scores.size(); i++)
		{
			if (scores.size() == weights.size()) {
				weighted.add(weights.get(i) * scores.get(i));
			} else
				weighted.add(scores.get(i));
		}
		
		return aggregate(weighted);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
