package ch.unizh.ifi.isparql.query.api.impl;

import ch.unizh.ifi.isparql.query.URIRegistry;
import ch.unizh.ifi.isparql.query.api.IStrategy;
import ch.unizh.ifi.isparql.query.optimizer.SimilarityIndexMem;
import com.hp.hpl.jena.mem.faster.GraphMemFaster;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.graph.Node;
import java.util.*;

/**
 * Abstract strategy class. Each specific strategy (strategy.*) 
 * extends this abstract strategy class. This class implements the 
 * IStrategy interface. The class provides all required setter 
 * and getter methods to configure similarity strategies and
 * specifies the available similarity strategy names. Moreover,
 * the constructor sets the ARQ binding, S/P/O and ARQ execution
 * context to the strategy. This information is the used by
 * specific strategies to implement the similarity computation logic.
 *
 * @see			ch.unizh.ifi.isparql.query.strategy
 * @see 		ch.unizh.ifi.isparql.query.api.impl.IStrategy
 * @author		Markus Stocker
 * @version 	$Id: AbstractStrategy.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public abstract class AbstractStrategy implements IStrategy 
{
	/** The URI-reference used for the strategy */
	private String uri = null;
	/** Strategy property value, isparql:name */
	private String name = null;
	/** Strategy property value, isparql:ignorecase */
	private Boolean ignorecase = null;
	/** Strategy property value, isparql:aggregator */
	private String aggregator = null;
	/** Strategy property value, isparql:threshold */
	private Double threshold = null;
	/** Strategy property value, isparql:between, not used, but implemented */
	private Double lowerBound = null;
	/** Strategy property value, isparql:between, not used, but implemented */
	private Double upperBound = null;
	/** Strategy property value, isparql:arguments, isparql:argument1 */
	private Node argument1 = null;
	/** Strategy property value, isparql:arguments, isparql:argument2 */
	private Node argument2 = null;
	/** Strategy property value, isparql:ontologyURI */
	private String ontologyURI1 = null;
	/** Strategy property value, isparql:ontologyBaseURI */
	private String ontologyBaseURI1 = null;
	/** Strategy property value, isparql:ontologyRootURI */
	private String ontologyRootURI1 = null;
	/** Strategy property value, isparql:ontologyURI */
	private String ontologyURI2 = null;
	/** Strategy property value, isparql:ontologyBaseURI */
	private String ontologyBaseURI2 = null;
	/** Strategy property value, isparql:ontologyRootURI */
	private String ontologyRootURI2 = null;
	/** Strategy property value, isparql:ontologyAltURI */
	private String ontologyAltURI = null;
	/** List of scores, for aggregation strategies */
	private List<Double> scores = new ArrayList<Double>();
	/** List of weights, for aggregation strategies */
	private List<Double> weights = new ArrayList<Double>();
	/** The Jena graph object for the underlying ontology */
	protected Graph graph = null;
	/** The subject of the virtual triple pattern (imprecise statement) */
	protected Node subject = null;
	/** The predicate of the virtual triple pattern (imprecise statement) */
	protected Node predicate = null;
	/** The object of the virtual triple pattern (imprecise statement) */
	protected Node object = null;
	/** The ARQ binding */
	protected Binding binding = null;
	/** The field holding the computed similarity strategy value */
	protected Double similarity = null;
	/** The ARQ execution context */
	protected ExecutionContext execCxt = null;
	/** The registry with iSPARQL specific URI-references */
	protected URIRegistry urireg = new URIRegistry();
	/** 
	 * The similarity index can be used to query for the similarity in a persistent store
	 * (MySQL databse) or in a hash (in-memory) instead to compute it. 
	 * This requires that the strategy resource similarities are 
	 * previously indexed in a specific database. 
	 * The similarity index can be used in each strategy.
	 */
	protected SimilarityIndexMem si;
	
	// Add the strategy to isDefined method!
	/** The ScoreAggregator strategy name, to be used in isparql:name */
	public static final String scoreAggregator = new String("ScoreAggregator");
	/** The LevenshteinStrCmp strategy name, to be used in isparql:name */
	public static final String levenshteinStrCmp = new String("LevenshteinStrCmp");
	/** The LevenshteinResCmp strategy name, to be used in isparql:name */
	public static final String levenshteinResCmp = new String("LevenshteinResCmp");
	/** The LinResCmp strategy name, to be used in isparql:name */
	public static final String linResCmp = new String("LinResCmp");
	/** The ResnikResCmp strategy name, to be used in isparql:name */
	public static final String resnikResCmp = new String("ResnikResCmp");
	/** The ShortestPathResCmp strategy name, to be used in isparql:name */
	public static final String shortestPathResCmp = new String("ShortestPathResCmp");
	/** The MITPHLevenshteinResCmp strategy name, to be used in isparql:name */
	public static final String MITPHLevenshteinResCmp = new String("MITPHLevenshteinResCmp");
	/** The MITPHTreeEditDistanceResCmp strategy name, to be used in isparql:name */
	public static final String MITPHTreeEditDistanceResCmp = new String("MITPHTreeEditDistanceResCmp");
	/** The SubGraphIsomorphismResCmp strategy name, to be used in isparql:name */
	public static final String subGraphIsomorphismResCmp = new String("SubGraphIsomorphismResCmp");
	/** The CosineStrCmp strategy name, to be used in isparql:name */
	public static final String cosineStrCmp = new String("CosineStrCmp");
	/** The JensenShannonStrCmp strategy name, to be used in isparql:name */
	public static final String jensenShannonStrCmp = new String("JensenShannonStrCmp");
	/** The OWLSTCServicesCmp strategy name, to be used in isparql:name */
	public static final String owlstcServicesCmp = new String("OWLSTCServicesCmp");
	/** The MITPHLoLResCmp strategy name, to be used in isparql:name */
	public static final String MITPHLoLResCmp = new String("MITPHLoLResCmp");
	/** The MITPHTFIDFResCmp strategy name, to be used in isparql:name */
	public static final String MITPHTFIDFResCmp = new String("MITPHTFIDFResCmp");
	/** The LOLStrCmp strategy name, to be used in isparql:name */
	public static final String LOLStrCmp = new String("LOLStrCmp");
	/** The DiceStrCmp strategy name, to be used in isparql:name */
	public static final String diceStrCmp = new String("DiceStrCmp");
	
	/**
	 * The constructor is used for SimilarityIndexer
	 * where strategies are invoqued only with the graph object
	 * 
	 * @see ch.unizh.ifi.isparql.indexing.indexer.SimilarityIndexer
	 * @param graph
	 */
	public AbstractStrategy(Graph graph)
	{
		this.graph = graph;
		
		// Defaults
		setAggregator(AbstractAggregator.avg);
		setThreshold(new Double(0));
		setLowerBound(new Double(0));
		setUpperBound(Double.MAX_VALUE);
		setIgnorecase(new Boolean("true"));
	}
	
	/**
	 * This constructure is used in strategies
	 * to init strategy name defined in SPARQL queries (isparql:name).
	 * ARQ assigns the binding, S/P/O and execution context to the strategy.
	 * 
	 * @see ch.unizh.ifi.ispaql.query.strategy
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public AbstractStrategy(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt) 
	{	
		this.subject = subject;
		this.predicate = predicate;
		this.object = object;
		this.binding = binding;
		this.execCxt = execCxt;
		
		// Defaults
		setAggregator(AbstractAggregator.avg);
		setThreshold(new Double(0));
		setLowerBound(new Double(0));
		setUpperBound(Double.MAX_VALUE);
		setIgnorecase(new Boolean("false"));
		
		this.graph = (GraphMemFaster)execCxt.getActiveGraph();
	}
	
	/**
	 * Set the similarity index object if queries are executed inside of 
	 * a java framework where the similarity index object can be instantiated once.
	 * 
	 * @param si
	 */
	public void setSimilarityIndex(SimilarityIndexMem si)
	{
		this.si = si;
	}
	
	/**
	 * Check whether a given strategy is defined or not.
	 * 
	 * @param strategy
	 * @return boolean
	 */
	public static boolean isDefined(String strategy)
	{
		if (strategy.equals(levenshteinStrCmp))
			return true;
		if (strategy.equals(levenshteinResCmp))
			return true;
		if (strategy.equals(linResCmp))
			return true;
		if (strategy.equals(resnikResCmp))
			return true;
		if (strategy.equals(shortestPathResCmp))
			return true;
		if (strategy.equals(MITPHLevenshteinResCmp))
			return true;
		if (strategy.equals(MITPHTreeEditDistanceResCmp))
			return true;
		if (strategy.equals(subGraphIsomorphismResCmp))
			return true;
		if (strategy.equals(scoreAggregator))
			return true;
		if (strategy.equals(cosineStrCmp))
			return true;
		if (strategy.equals(jensenShannonStrCmp))
			return true;
		if (strategy.equals(owlstcServicesCmp))
			return true;
		if (strategy.equals(MITPHLoLResCmp))
			return true;
		if (strategy.equals(MITPHTFIDFResCmp))
			return true;
		if (strategy.equals(LOLStrCmp))
			return true;
		if (strategy.equals(diceStrCmp))
			return true;
		
		return false;
	}
	
	/**
	 * Default implementation to calculate the structure similarity between two
	 * complex resources. The method takes two integers which corresponds to the 
	 * number of triples (predicates) of both resources.
	 * 
	 * Default formula: div(mod(intersecOf(mod(int1), mod(int2))), 
	 * mod(int1) + mod(int2) - mod(intersecOf(mod(int1), mod(int2))))
	 *     
	 * @param int1
	 * @param int2
	 * @return Double
	 */
	protected Double getSimilarityStructure(Integer int1, Integer int2)
	{
		int intersec = Math.min(int1, int2);
		int div1 = intersec;
		int div2 = int1 + int2 - intersec;
		
		if (div2 != 0)
			return new Double(div1 / div2);
			
		return new Double(1);
	}
	
	/** @param value */
	public void setURI(String value)
	{
		uri = value;
	}
	
	/** @return String */
	public String getURI()
	{
		return uri;
	}
	
	/** @param value */
	public void setName(String value)
	{
		name = value;
	}
	
	/** @return String */
	public String getName()
	{
		return name;
	}
	
	/** @param value */
	public void setThreshold(String value)
	{
		setThreshold(new Double(value));
	}
	
	/** @param value */
	public void setThreshold(Double value)
	{
		threshold = value;
	}
	
	/** @return Double */
	public Double getThreshold()
	{	
		return threshold;
	}
	
	/** @param value */
	public void setIgnorecase(String value)
	{
		setIgnorecase(new Boolean(value));
	}
	
	/** @param value */
	public void setIgnorecase(Boolean value)
	{
		ignorecase = value;
	}
	
	/** @return Boolean */
	public Boolean getIgnorecase()
	{
		return ignorecase;
	}
	
	/** @param value */
	public void setArgument1(Node value)
	{
		if (value.isVariable())
			value = Node.create(binding.get(value.getName()).getLiteralValue().toString());
		
		// Make sure to remove XSD datatype information
		if (value.isLiteral())
			value = Node.createLiteral(value.getLiteralValue().toString());
		else
			value = Node.create(value.getURI());
		
		argument1 = value;
	}
	
	/** @return Node */
	public Node getArgument1()
	{	
		return argument1;
	}
	
	/** @param value */
	public void setArgument2(Node value)
	{
		if (value.isVariable())
			value = Node.create(binding.get(value.getName()).getLiteralValue().toString());
		
		// Make sure to remove XSD datatype information
		if (value.isLiteral())
			value = Node.createLiteral(value.getLiteralValue().toString());
		else
			value = Node.create(value.getURI());
		
		argument2 = value;
	}
	
	/** @return Node */
	public Node getArgument2()
	{
		return argument2;
	}
	
	/** @param value */
	public void setLowerBound(Double value)
	{
		lowerBound = value;
	}
	
	/** @return Double */
	public Double getLowerBound()
	{
		return lowerBound;
	}
	
	/** @param value */
	public void setUpperBound(Double value)
	{
		upperBound = value;
	}
	
	/** @return Double */
	public Double getUpperBound()
	{
		return upperBound;
	}
	
	/** @param value */
	public void setAggregator(String value)
	{
		aggregator = value;
	}
	
	/** @return String */
	public String getAggregator()
	{
		return aggregator;
	}
	
	/** @param value */
	public void setOntologyBaseURI1(String value)
	{
		ontologyBaseURI1 = value;
	}
	
	/** @return String */
	public String getOntologyBaseURI1()
	{
		return ontologyBaseURI1;
	}
	
	/** @param value */
	public void setOntologyRootURI1(String value)
	{
		ontologyRootURI1 = value;
	}
	
	/** @return String */
	public String getOntologyRootURI1()
	{
		return ontologyRootURI1;
	}
	
	/** @param value */
	public void setOntologyURI1(String value)
	{
		ontologyURI1 = value;
	}
	
	/** @return String */
	public String getOntologyURI1()
	{
		return ontologyURI1;
	}
	
	/** @param value */
	public void setOntologyBaseURI2(String value)
	{
		ontologyBaseURI2 = value;
	}
	
	/** @return String */
	public String getOntologyBaseURI2()
	{
		return ontologyBaseURI2;
	}
	
	/** @param value */
	public void setOntologyRootURI2(String value)
	{
		ontologyRootURI2 = value;
	}
	
	/** @return String */
	public String getOntologyRootURI2()
	{
		return ontologyRootURI2;
	}
	
	/** @param value */
	public void setOntologyURI2(String value)
	{
		ontologyURI2 = value;
	}
	
	/** @return String */
	public String getOntologyURI2()
	{
		return ontologyURI2;
	}
	
	/** @param value */
	public void setOntologyAltURI(String value)
	{
		ontologyAltURI = value;
	}
	
	/** @return String */
	public String getOntologyAltURI()
	{
		return ontologyAltURI;
	}
	
	/** @param value */
	public void addScore(Double value)
	{
		scores.add(value);
	}
	
	/** @param value */
	public void addWeight(Double value)
	{
		weights.add(value);
	}
	
	/** @return List<Double> */
	public List<Double> getScores()
	{
		return scores;
	}
	
	/** @return List<Double> */
	public List<Double> getWeights()
	{
		return weights;
	}
	
	/** @param value */
	public void setSimilarity(Double value)
	{
		similarity = value;
	}
	
	/** @return Double */
	public Double getSimilarity()
	{
		return similarity;
	}
	
	/** @return ExecutionContext */
	public ExecutionContext getExecutionContext()
	{
		return execCxt;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
