package ch.unizh.ifi.isparql.query.measure.weightingscheme;

import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.MeasureRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.measure.weightingscheme.StringTFIDF;
import simpack.util.corpus.StringUtils;
import com.hp.hpl.jena.query.util.Symbol;
import com.wcohen.ss.BasicStringWrapper;
import com.wcohen.ss.api.StringWrapper;
import java.util.*;

/**
 * SimPack wrapper for StringTFIDF similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.weightingscheme.StringTFIDF
 * @author		Markus Stocker
 * @version 	$Id: MeasureStringTFIDF.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureStringTFIDF extends AbstractMeasure 
{	
	private StringTFIDF measure = null;
	private boolean isSetCorpus = false;
	private MeasureRegistry measurereg = null;
	private ArrayList<StringWrapper> corpus = new ArrayList<StringWrapper>();
	private String uri = urireg.get(AbstractMeasure.stringTFIDF);
	
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation. The constructor
	 * extracts the measure object from the measure registry to
	 * avoid documents traning more than once.
	 * 
	 * @param strategy
	 */
	public MeasureStringTFIDF(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(stringTFIDF));
		setName(stringTFIDF);
		measurereg = (MeasureRegistry)strategy.getExecutionContext().getContext().get(new Symbol(Constants.MEASURE_REGISTRY));
	
		if (measurereg == null)
		{	
			// Create a measure registry
			measurereg = new MeasureRegistry();
			// Store the registry to the execution context
			strategy.getExecutionContext().getContext().set(new Symbol(Constants.MEASURE_REGISTRY), measurereg);
		}
		else 
		{
			// Get the measure from the registry by specifying his URI
			measure = (StringTFIDF)measurereg.get(uri);
			
			if (measure != null)
				isSetCorpus = true;
		}
	}
	
	/**
	 * Compute the similarity, using the SimPack StringTFIDF class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{
		String argument1 = strategy.getArgument1().getLiteralValue().toString();
		String argument2 = strategy.getArgument2().getLiteralValue().toString();
		
		return similarity(argument1, argument2);
	}
	
	/**
	 * Compute the similarity, using the SimPack StringTFIDF class.
	 * If the measure could not be found in the registry, we create 
	 * a new SimPack StringTFIDF with the corpus set by setCorpus()
	 * and the training is executed. The StringTFIDF object is then
	 * saved to the measure registry.
	 * 
	 * @see ch.unizh.ifi.isparql.query.MeasureRegistry
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(String argument1, String argument2)
	{		
		// Measure is not found in registry, create it
		if (measure == null) 
			measurereg.put(uri, new StringTFIDF(corpus));

		measure = (StringTFIDF)measurereg.get(uri);
		
		return measure.getSimilarity(StringUtils.clean(argument1), 
									 StringUtils.clean(argument2));
	}
	
	/**
	 * Allows setting the corpus, which is a list of documents
	 * 
	 * @param documents
	 */
	public void setCorpus(List<String> documents)
	{	
		for (Iterator iter = documents.iterator(); iter.hasNext(); )
			corpus.add(new BasicStringWrapper(StringUtils.clean((String)iter.next())));
	
		isSetCorpus = true;
	}
	
	/**
	 * Verify if the corpus is already set to the measure, ie the
	 * measure already executed the documents training.
	 * 
	 * @return boolean
	 */
	public boolean isSetCorpus()
	{
		return isSetCorpus;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
