package ch.unizh.ifi.isparql.query.optimizer;

import ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer;
import ch.unizh.ifi.isparql.query.Constants;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * Wrapper to iSPARQL similarity index persitent store.
 * Database access layer to query the strategy similarity index.
 *
 * @author		Markus Stocker
 * @version 	$Id: SimilarityIndexDB.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class SimilarityIndexDB extends AbstractOptimizer
{
	/** The MySQL database connection */
	private Connection conn;
	/** The similarity value */
	private Double similarity = null;
	/** The first resource for similarity lookup */
	private String resource1 = null;
	/** The second resource for similarity lookup */
	private String resource2 = null;
	/** The table attribute name, where the similarity value is stored */
	private String strategy;
	
	/**
	 * Constructor, create the JDBC instance, 
	 * do it only if the optimizer is activated.
	 *
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer
	 */
	public SimilarityIndexDB()
	{
		if (! optimizer)
			return;
		
		try 
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {}
		
		try 
		{
			conn = DriverManager.getConnection("jdbc:mysql://localhost/isparql?user=" + Constants.MYSQL_DB_UID + "&password=" + Constants.MYSQL_DB_PWD);	
		} catch (SQLException ex) {
			System.out.println("SQLException: " + ex.getMessage());
			System.out.println("SQLState: " + ex.getSQLState());
			System.out.println("VendorError: " + ex.getErrorCode());
		}
	}
	
	/** 
	 * Interface implementation.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.IOptimizer
	 * @return Double
	 */
	public Double lookup()
	{
		return lookup(resource1, resource2, strategy);
	}
	
	/**
	 * Interface implementation.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.IOptimizer
	 * @param resource1
	 * @param resource2
	 * @return Double
	 */
	public Double lookup(String resource1, String resource2)
	{
		return lookup(resource1, resource2, "strategy1");
	}

	/**
	 * Interface implementation.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.IOptimizer
	 * @param resource1
	 * @param resource2
	 * @param strategy
	 * @return Double
	 */
	public Double lookup(String resource1, String resource2, String strategy)
	{
		java.sql.Statement stmt = null;
		java.sql.ResultSet rs = null;
		
		if (!optimizer || resource1 == null || resource2 == null)
			return similarity;
		
		String sql = "SELECT " + strategy + " FROM si WHERE " +
		 			 "resource1 = '" + resource1 + "' AND " +
		 			 "resource2 = '" + resource2 + "'";

		try 
		{
			stmt = conn.createStatement();
			rs = stmt.executeQuery(sql);
			
			while (rs.next())
			{
				similarity = rs.getDouble(1);
			}

		}  catch (SQLException e) { System.out.println(e.getMessage()); }
		
		finally 
		{
			if (rs != null) 
			{
				try 
				{
					rs.close();
				} catch (SQLException sqlEx) { }
				rs = null;
			}
			if (stmt != null) 
			{
				try 
				{
					stmt.close();
				} catch (SQLException sqlEx) { }
				stmt = null;
			}
		}
		
		return similarity;
	}
	
	/**
	 * Allows setting the resources which are used
	 * for the similarity lookup() in the persistent store.
	 * 
	 * @param value1
	 * @param value2
	 */
	public void setResources(String value1, String value2)
	{
		resource1 = value1;
		resource2 = value2;
	}
	
	/**
	 * Allows setting the table attribute name,
	 * where the similarity value is stored.
	 * 
	 * @param value
	 */
	public void setStrategy(String value)
	{
		strategy = "strategy1";
	}
	
	/** Close the MySQL database connection */
	public void finalize()
	{
		if (!optimizer)
			return;
		
		try 
		{
			conn.close();
		} catch (SQLException e) {}
	}

}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
