package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.AggregatorFactory;
import ch.unizh.ifi.isparql.query.Pattern;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLevenshtein;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import java.util.*;

/**
 * Levenshtein resource comparison strategy
 *
 * @author		Markus Stocker
 * @version 	$Id: LevenshteinResCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class LevenshteinResCmp extends AbstractStrategy 
{
	private List<Pattern> plan = new ArrayList<Pattern>();
	/** The framework SimPack wrapper */
	private MeasureLevenshtein measure = new MeasureLevenshtein(this);
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public LevenshteinResCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.levenshteinResCmp);
	}
	
	/**
	 * This method implements the logic what is to do with both similarity resources
	 * defined in query. For Levenshtein, we search for Patterns, ie get all atomic 
	 * object similarity comparisions of complex resources, or create a single pattern
	 * if the nodes are already simple (not URIs).
	 * 
	 * @param node1
	 * @param node2
	 */
	private void planArguments(Node node1, Node node2)
	{			
		findPatterns(new Pattern(graph, node1, node2));
	}
	
	/**
	 * Method which can be recursively executed to find all patterns of two resources
	 * The goal is to check if two nodes are complex or not. For complex nodes, all
	 * associated triples are extracted from the underlying graph and stored to this
	 * pattern. This method iterates over all triples found for node1 and extracts
	 * the corresponding triple from node2 (findTripleForPredicate). If both objects
	 * of two inspected triples are URIs, the method is executed again over this 
	 * objects.
	 * 
	 * @param pattern
	 */
	private void findPatterns(Pattern pattern)
	{	
		for (Iterator iter = pattern.getTriplesFor(pattern.getNode1()).iterator(); iter.hasNext(); )
		{
			Triple triple1 = (Triple)iter.next();
			// Problem: what if there are two or more equally named predicates!?
			Triple triple2 = pattern.findTripleForPredicate(triple1.getPredicate());
			
			if (triple1.getObject().isURI() && triple2.getObject().isURI())
				findPatterns(new Pattern(graph, triple1.getObject(), triple2.getObject()));
		}
		plan.add(pattern);
	}
	
	/**
	 * The structure similarity considers the number of triples of each complex 
	 * resource and is calculated only for complex resources. The formula used is
	 * 
	 * div(mod(intersecOf(mod(triples1), mod(triples2))), 
	 *     mod(triples1) + mod(triples2) - mod(intersecOf(mod(triples1), mod(triples2))))
	 *     
	 * @param pattern
	 * @return Double
	 */
	private Double getSimilarityStructure(Pattern pattern)
	{
		int nrOfTriples1 = pattern.getTriplesFor(pattern.getNode1()).size();
		int nrOfTriples2 = pattern.getTriplesFor(pattern.getNode2()).size();
		int intersec = Math.min(nrOfTriples1, nrOfTriples2);
		int div1 = intersec;
		int div2 = nrOfTriples1 + nrOfTriples2 - intersec;
		
		if (div2 != 0) {
			int simStructure = div1 / div2;
			return new Double(simStructure);
		} else {
			return new Double(1);
		}
	}
	
	/**
	 * This method calculates the similarity between all not URIs triples of two
	 * pattern resources. Iterate over all triples of the first resource (node1),
	 * search for the corresponding triple of the second resource and calculated
	 * the similarity between the two triple objects. Aggregate the similarities
	 * by means of the defined or default aggregator.
	 * 
	 * @param pattern
	 * @return Double
	 */
	protected Double getSimilarityMeasure(Pattern pattern)
	{
		AbstractAggregator aggregator = AggregatorFactory.create(getAggregator());
		
		for (Iterator iter = pattern.getTriplesFor(pattern.getNode1()).iterator(); iter.hasNext(); )
		{
			Triple triple1 = (Triple)iter.next();
			Triple triple2 = pattern.findTripleForPredicate(triple1.getPredicate());
			
			if (triple1 != null && triple2 != null)
			{
				if (!triple1.getObject().isURI() && !triple2.getObject().isURI())
				{
					String object1 = triple1.getObject().toString(false);
					String object2 = triple2.getObject().toString(false);
		
					aggregator.addScore(measure.similarity(object1, object2));
				}
			}
		}
		
		return aggregator.aggregate();
	}
	
	/**
	 * Interface implementation
	 * 
	 * @see ch.unizh.ifi.isparql.query.measure.sequence.MeasureLevenshtein
	 * @return Double
	 */
	public Double similarity()
	{	
		AbstractAggregator overallAggregator = AggregatorFactory.create(getAggregator());
		AbstractAggregator patternAggregator = AggregatorFactory.create(getAggregator());
		
		// Get recursively all patterns of a complex resource
		planArguments(getArgument1(), getArgument2());
		
		// Get structure / measure similarity for all patterns
		for (Iterator iter = plan.iterator(); iter.hasNext(); )
		{
			Pattern pattern = (Pattern)iter.next();
			
			// Consider similarity structure only for complex patterns
			if (pattern.isComplex())
				patternAggregator.addScore(getSimilarityStructure(pattern));
			
			patternAggregator.addScore(getSimilarityMeasure(pattern));
			
			// Aggregate similarity measure and structure
			overallAggregator.addScore(patternAggregator.aggregate());
			patternAggregator.clear();
		}
		
		return overallAggregator.aggregate();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
