package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.AggregatorFactory;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLOL;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.graph.Node;
import java.util.*;

/**
 * MIT Process Handbook Strategy.
 * This strategy is specific for MIT Process Handbook. The purpose is to
 * get all Parts of two resources, resolve them to get their names and find 
 * for each name of the first process the highest Levenshtein score when compared
 * to all names of the second process.
 *
 * @author		Markus Stocker
 * @version 	$Id: MITPHLoLResCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MITPHLoLResCmp extends AbstractStrategy 
{
	/** The framework SimPack wrapper */
	private MeasureLOL measure = new MeasureLOL(this);
	private String baseURI = "http://www.ifi.unizh.ch/ddis/ph/2006/08/ProcessHandbook.owl#";
	
	/** 
	 * Constructor, used for similarity indexing
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param graph
	 */
	public MITPHLoLResCmp(Graph graph)
	{
		super(graph);
		
		setName(AbstractStrategy.MITPHLoLResCmp);
	}
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public MITPHLoLResCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.MITPHLoLResCmp);
	}
	
	/**
	 * Interface implementation
	 * 
	 * @see ch.unizh.ifi.isparql.query.measure.sequence.MeasureLOL
	 * @return Double
	 */
	public Double similarity()
	{	
		AbstractAggregator pattern = AggregatorFactory.create(getAggregator());
		
		List process1PartNames = getPartNamesForProcess(getArgument1());
		List process2PartNames = getPartNamesForProcess(getArgument2());
		
		/* Get the best Levenshtein similarity between each hasPart of process1
		* and all hasParts of process2 */
		for (Iterator iter = process1PartNames.iterator(); iter.hasNext(); ) {
			pattern.addScore(bestSimilarityFor((String)iter.next(), process2PartNames));
		}
		
		return pattern.aggregate();
	}
	
	private Double bestSimilarityFor(String part, List parts)
	{
		Double best = new Double(0);
		
		for (Iterator iter = parts.iterator(); iter.hasNext(); )
		{
			Double similarity = measure.similarity(part, (String)iter.next());
			if (similarity > best)
				best = similarity;
		}
		
		return best;
	}
	
	private List getPartNamesForProcess(Node uri)
	{
		List<String> partNames = new ArrayList<String>();
		
		Iterator partURIs = graph.queryHandler().objectsFor(uri, Node.createURI(baseURI + "hasPart"));
	
		while (partURIs.hasNext())
		{
			Node node = (Node)partURIs.next();
			Iterator partName = graph.queryHandler().objectsFor(node, Node.createURI(baseURI + "name"));
			
			while (partName.hasNext()) 
			{
				Node node1 = (Node)partName.next();
				partNames.add(node1.toString(false));
			}
		}
		
		return partNames;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
