package ch.unizh.ifi.isparql.query.strategy;

/**
 * MIT Process Handbook Strategy
 *
 * @author		Markus Stocker
 * @version 	$Id: MITPHTFIDFResCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

import ch.unizh.ifi.isparql.query.AggregatorFactory;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLevenshtein;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLOL;
import ch.unizh.ifi.isparql.query.measure.tree.MeasureTreeEditDistance;
import ch.unizh.ifi.isparql.query.measure.weightingscheme.MeasureStringTFIDF;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.graph.Node;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;

import simpack.api.ITreeNode;
import simpack.util.tree.TreeNode;

public class MITPHTFIDFResCmp extends AbstractStrategy 
{
	private String baseURI = "http://www.ifi.unizh.ch/ddis/ph/2006/08/ProcessHandbook.owl#";
	private String descriptionURI = "http://www.ifi.unizh.ch/ddis/ph/2006/08/ProcessHandbook.owl#description";
	private String nameURI = "http://www.ifi.unizh.ch/ddis/ph/2006/08/ProcessHandbook.owl#name";
	private String corpusFileName = "D:\\cvsroot\\diplomarbeit\\sources\\isparql\\query-examples\\mitph-corpus.txt";
	
	/** 
	 * Constructor, used for similarity indexing
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param graph
	 */
	public MITPHTFIDFResCmp(Graph graph)
	{
		super(graph);
		
		setName(AbstractStrategy.MITPHTFIDFResCmp);
	}
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public MITPHTFIDFResCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.MITPHTFIDFResCmp);
	}
	
	/**
	 * Interface implementation
	 *
	 * @return Double
	 */
	public Double similarity()
	{	
		Double partsSimilarity;
		AbstractAggregator aggregator = AggregatorFactory.create(this.getAggregator());
		String name1 = getObjectFor(getArgument1(), Node.create(nameURI));
		String name2 = getObjectFor(getArgument2(), Node.create(nameURI));
		String textDescription1 = getObjectFor(getArgument1(), Node.create(descriptionURI));
		String textDescription2 = getObjectFor(getArgument2(), Node.create(descriptionURI));
		
		List<String> process1PartNames = getPartNamesForProcess(getArgument1());
		List<String> process2PartNames = getPartNamesForProcess(getArgument2());
		
		Double nameSimilarity = getNameSimilarity(name1, name2);
		Double descriptionSimilarity = getDescriptionSimilarity(textDescription1, textDescription2);
		
		if (process1PartNames.size() <= process2PartNames.size())
			partsSimilarity = getPartsSimilarity(process1PartNames, process2PartNames);
		else
			partsSimilarity = getPartsSimilarity(process2PartNames, process1PartNames);
		
		aggregator.addScore(nameSimilarity);
		aggregator.addScore(descriptionSimilarity);
		aggregator.addScore(partsSimilarity);
		
		//System.out.println(nameSimilarity + " " + descriptionSimilarity + " " + partsSimilarity);
		
		return aggregator.aggregate();
	}
	
	private Double getPartsSimilarity(List<String> process1, List<String> process2)
	{
		MeasureTreeEditDistance measure = new MeasureTreeEditDistance(this);
		
		int level = 0;
		ITreeNode tree1 = new TreeNode(getArgument1().getURI());
		ITreeNode tree2 = new TreeNode(getArgument2().getURI());
	
		tree1 = getTreeNodeFor(tree1, getArgument1(), level);
		tree2 = getTreeNodeFor(tree2, getArgument2(), level);
		
		measure.setTree1(tree1);
		measure.setTree2(tree2);
		
		return measure.similarity();
		/**
		MeasureLevenshteinOfLevenshtein measure = new MeasureLevenshteinOfLevenshtein(this);
		AbstractAggregator aggregator = AggregatorFactory.create(this.getAggregator());
	
		for (int i = 0; i < process1.size(); i++ )
		{
			aggregator.addScore(measure.similarity((String)process1.get(i),
											  	   (String)process2.get(i)));
		}
		
		return aggregator.aggregate();
		*/
	}
	
	private ITreeNode getTreeNodeFor(ITreeNode tree, Node uri, int level)
	{	
		level++;
		
		// We create trees of recurse level 10
		if (level == 10)
			return tree;
		
		Iterator hasPartPredicates = graph.queryHandler().objectsFor(uri, Node.createURI(baseURI + "hasPart"));
		
		/*
		 * Step through all hasPart predicates for this uri 
		 * and create a TreeNode for each one
		 * Add the hasPart TreeNodes to the parent TreeNode
		 */
		while (hasPartPredicates.hasNext())
		{
			// Get the URI for the new hasPart predicate
			uri = (Node)hasPartPredicates.next();
			tree.add(getTreeNodeFor(new TreeNode(uri.getURI()), uri, level));
		}
		
		return tree;
	}
	
	private Double getNameSimilarity(String name1, String name2)
	{
		MeasureLOL measure = new MeasureLOL(this);
		
		return measure.similarity(name1, name2);
	}
	
	private Double getDescriptionSimilarity(String textDescription1, String textDescription2)
	{
		//MeasureStringTFIDF measure = new MeasureStringTFIDF(this);
		
		// Set the corpus only the first time
		//if (measure.isSetCorpus() == false)
		//	measure.setCorpus(getCorpus());
		
		MeasureLOL measure = new MeasureLOL(this);
			
		textDescription1 = textDescription1.trim();
		textDescription1 = textDescription1.replaceAll("[\r\n]"," ");
		textDescription1 = textDescription1.replaceAll("<.*?>","");
		
		textDescription2 = textDescription2.trim();
		textDescription2 = textDescription2.replaceAll("[\r\n]"," ");
		textDescription2 = textDescription2.replaceAll("<.*?>","");
		
		return measure.similarity(textDescription1, textDescription2);
	}
	
	private String getObjectFor(Node subject, Node predicate)
	{
		Node node = null;
		
		ExtendedIterator it = graph.queryHandler().objectsFor(subject, predicate);
		
		while (it.hasNext())
		{
			 node = (Node)it.next();
		}
		
		String object = node.getLiteralValue().toString();
		object = object.trim();
		object = object.replaceAll("[\r\n]"," ");
		
		return object;
	}
	
	private List<String> getPartNamesForProcess(Node uri)
	{
		List<String> partNames = new ArrayList<String>();
		
		Iterator partURIs = graph.queryHandler().objectsFor(uri, Node.createURI(baseURI + "hasPart"));
	
		while (partURIs.hasNext())
		{
			Node node = (Node)partURIs.next();
			Iterator partName = graph.queryHandler().objectsFor(node, Node.createURI(baseURI + "name"));
			
			while (partName.hasNext()) 
			{
				Node node1 = (Node)partName.next();
				partNames.add(node1.toString(false));
			}
		}
		
		return partNames;
	}
	
	private List<String> getCorpus()
	{
		String line;
		BufferedReader br = null;
		List<String> documents = new ArrayList<String>();
		
		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(corpusFileName)));
		} catch (FileNotFoundException e) { System.out.println(e); }
		
		try
		{
			while((line = br.readLine()) != null) 
			{
				if (line.length() > 0)
					documents.add(line);
			}
		} catch (IOException e) {}
		
		return documents;
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
