package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.AggregatorFactory;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLevenshtein;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLOL;
import ch.unizh.ifi.isparql.query.measure.simmetrics.MeasureJaccard;
import ch.unizh.ifi.isparql.query.measure.simmetrics.MeasureCosine;
import ch.unizh.ifi.isparql.query.measure.vector.MeasureAlignmentComparison;
import ch.unizh.ifi.isparql.query.measure.weightingscheme.MeasureStringTFIDF;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import java.util.*;

/**
 * Specific strategy to compare OWLS-TC2 services
 *
 * @author		Markus Stocker
 * @version 	$Id: OWLSTCServicesCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class OWLSTCServicesCmp extends AbstractStrategy 
{
	private final String serviceNameURI = "http://www.daml.org/services/owl-s/1.1/Profile.owl#serviceName";
	private final String textDescriptionURI = "http://www.daml.org/services/owl-s/1.1/Profile.owl#textDescription";
	private final String inputURI = "http://www.daml.org/services/owl-s/1.1/Profile.owl#hasInput";
	private final String outputURI = "http://www.daml.org/services/owl-s/1.1/Profile.owl#hasOutput";
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public OWLSTCServicesCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.owlstcServicesCmp);
	}
	
	/**
	 * Interface implementation
	 * 
	 * @return Double
	 */
	public Double similarity()
	{			
		String serviceName1 = getObjectFor(this.getArgument1(), Node.create(serviceNameURI));
		String serviceName2 = getObjectFor(this.getArgument2(), Node.create(serviceNameURI));
		String textDescription1 = getObjectFor(this.getArgument1(), Node.create(textDescriptionURI));
		String textDescription2 = getObjectFor(this.getArgument2(), Node.create(textDescriptionURI));
	
		String input1 = getInputOutputFor(this.getArgument1(), Node.create(inputURI));
		String input2 = getInputOutputFor(this.getArgument2(), Node.create(inputURI));
		String output1 = getInputOutputFor(this.getArgument1(), Node.create(outputURI));
		String output2 = getInputOutputFor(this.getArgument2(), Node.create(outputURI));
		
		/*
		Map<Object, Object> input1 = getInputOutputFor(this.getArgument1(), Node.create(inputURI));
		Map<Object, Object> input2 = getInputOutputFor(this.getArgument2(), Node.create(inputURI));
		Map<Object, Object> output1 = getInputOutputFor(this.getArgument1(), Node.create(outputURI));
		Map<Object, Object> output2 = getInputOutputFor(this.getArgument2(), Node.create(outputURI));
		*/
		
		AbstractAggregator aggregator = AggregatorFactory.create(this.getAggregator());
		Double serviceNameSimilarity = getServiceNameSimilarity(serviceName1, serviceName2);
		Double textDescriptionSimilarity = getTextDescriptionSimilarity(textDescription1, textDescription2);
		Double inputSimilarity = getInputOutputSimilarity(input1, input2);
		Double outputSimilarity = getInputOutputSimilarity(output1, output2);
		
		//System.out.println(serviceNameSimilarity + " " + textDescriptionSimilarity + " " + inputSimilarity + " " + outputSimilarity);
		
		aggregator.addScore(serviceNameSimilarity);
		aggregator.addScore(textDescriptionSimilarity);
		aggregator.addScore(inputSimilarity);
		aggregator.addScore(outputSimilarity);
		
		return aggregator.aggregate();
	}
	
	private Double getServiceNameSimilarity(String serviceName1, String serviceName2)
	{
		MeasureLevenshtein measure = new MeasureLevenshtein(this);
	
		return measure.similarity(serviceName1, serviceName2);
	}
	
	private Double getTextDescriptionSimilarity(String textDescription1, String textDescription2)
	{
		//MeasureStringTFIDF measure = new MeasureStringTFIDF(this);
		//measure.setCorpus(getCorpus());
		MeasureLOL measure = new MeasureLOL(this);
		
		return measure.similarity(textDescription1, textDescription2);
	}
	
	private Double getInputOutputSimilarity(String value1, String value2)
	{
		MeasureJaccard measure = new MeasureJaccard(this);
		//MeasureCosine measure = new MeasureCosine(this);
		
		return measure.similarity(value1, value2);
	}
	
	/*
	private Double getInputOutputSimilarity(Map<Object, Object> argument1, Map<Object, Object> argument2)
	{
		MeasureAlignmentComparison measure = new MeasureAlignmentComparison(this);
		
		return measure.similarity(argument1, argument2);
	}
	*/
	
	private String getObjectFor(Node subject, Node predicate)
	{
		Node node = null;
		
		ExtendedIterator it = graph.queryHandler().objectsFor(subject, predicate);
		
		while (it.hasNext())
		{
			 node = (Node)it.next();
		}
		
		String object = node.getLiteralValue().toString();
		object = object.trim();
		object = object.replaceAll("[\r\n]"," ");
		
		return object;
	}
	
	private String getInputOutputFor(Node subject, Node predicate)
	{
		ExtendedIterator it = null;
		List<Node> nodes = new ArrayList<Node>();
		Map<Object, Object> map = new HashMap<Object, Object>();
		String inputOutputString = new String("");
		Node parameterTypePredicate = Node.create("http://www.daml.org/services/owl-s/1.1/Process.owl#parameterType");
		
		// Extract the hasInput or hasOutput objects
		it = graph.queryHandler().objectsFor(subject, predicate);
		
		while (it.hasNext())
		{
			nodes.add((Node)it.next());
		}
		
		// Get the parameter type URI specified as object of the hasInput / hasOutput
		for (Iterator iter = nodes.iterator(); iter.hasNext(); )
		{
			subject = (Node)iter.next();
			it = graph.queryHandler().objectsFor(subject, parameterTypePredicate);
			
			while (it.hasNext())
			{
				Node node = (Node)it.next();
				String parameter = node.getLiteralValue().toString();
				parameter = parameter.replace("http://127.0.0.1/ontology/","");
				map.put(subject.getURI().substring(subject.getURI().indexOf("#") + 1, subject.getURI().length()), parameter);
				inputOutputString += parameter + " ";
			}
		}
		
		//return map;
		return inputOutputString;
	}
	
	private List<String> getCorpus()
	{
		List<String> documents = new ArrayList<String>();
		
		documents.add("The service returns a list of flights that fulfill the desired travel conditions.");
		documents.add("This service returns the funding for research on the weapon provided by the given government.");
		documents.add("This service retruns a certain digital video camera with its maximum price availbility at a certain shopping mall");
		documents.add("This service returns not only information of new comedy films, but also highly rated action films.");
		documents.add("This service returns a camera and its price available in a Wallmart.");
		documents.add("This service provides the price of breakfast, specific coffee and Sadwich.");
		documents.add("This service returns food of HEB grocery compangy. Beginning in 1905, HEB Grocery Company is now one of the nation's largest independently owned food retailers with stores throughout Texas and Mexico.");
		documents.add("This service informs the price of certain food, used for human being.");
		documents.add("This service returns current weatherfront of Germany.");
		documents.add("This service adds Monograph as an item in the session of a authorised person.");
		documents.add("Guddu service is used to receive cola for a price.");
		documents.add("This service returns town for hiking and surfing");
		documents.add("This service returns price of a monograph.");
		documents.add("This is the recommended service to know about the rural area for surfing.");
		documents.add("Checks the availability of the required equipment for an operation.");
		documents.add("This site recommends price for grind machine.");
		documents.add("This service returns price of a media player.");
		documents.add("USWhiskeyCoffee service returns a coffee and a whiskey for the given recommended price.");
		documents.add("This service returns a map of Germany. This map is useful to find a route for traveling between its two cities, for example from Frankfurt to Berlin etc.");
		documents.add("This service gives you information for the lending offered by the given government.");
		documents.add("This is the very first service that returns the destination where both games hiking and surfing are available.");
		documents.add("This service returns the offered scholarship from the given government, it is normally dedicated to the persons who have high profile in education.");
		documents.add("This service returns available cola for the given maximum price.");
		documents.add("This service informs you about the certain type of tea which is available in the certain grocerystore and price.");
		documents.add("It is the best service that returns price of a given science-fiction novel to the certain user.");
		documents.add("This service returns Lecturers of the Saarland university.");
		documents.add("Are you a regular reader? We offer a service to return you recommended price of a book.");
		documents.add("This service returns mixerycola and irishcoffee for a given price.");
		documents.add("This service returns an address of an employee of a university.");
		documents.add("This service returns price of a set of MP3Player, a CD Player, and microwave oven.");
		documents.add("GRWF service is used to get weatherfront of a particular geogrphical region.");
		documents.add("This service informs the price of certain apple.");
		documents.add("TaxPCW Service offers a whiskey and a coffe for a tax free price.");
		documents.add("This service offer Cola and Beer for the given maximum price.");
		documents.add("This service checks credit card account of an authorised person and adds the selected book in his shopping cart.");
		documents.add("This service returns report on a certain type of Drought.");
		documents.add("This service informs you selling of a certain butter in a given grocerystore.");
		documents.add("Car Price quality");
		documents.add("Camera Price");
		documents.add("This service is used to login in our system.");
		documents.add("This service returns food available at the certain retail store and its quality also.");
		documents.add("This service returns accomodation information, hotel, resturant etc in the city of the country.");
		documents.add("This service returns author of a certain book such as short story or encyclopedia.");
		documents.add("This service returns price of printed material, either it is newspaper or a fiction novel.");
		documents.add("Creates a customer account which is required for booking a transport.");
		documents.add("This service provides information of recommended price in Euro for a Renault car whose model year is given.");
		documents.add("This service returns sports positions available in a certain country national team, for instance coach of German national football team.");
		documents.add("KAHN is a recommended service to find high valuable books with their price on the given title.");
		documents.add("This service returns town for sports.");
		documents.add("HOM2 is the mostly used service to return an address of a given researcher.");
		documents.add("This service returns recommended price of preparedfood and avilable in a certain Grocery shop.");
		documents.add("This service returns current warmfront condition of Italy.");
		documents.add("This service provides a bread for the maximum price, and also a free offered cola on luck.");
		documents.add("This service returns skilled occupation position in a certain company located in a certain country. If the company input is not given then the output is a job in the whole country.");
		documents.add("This service informs you about available camera, its price and availability time in the certain shoppingmall.");
		documents.add("This service gives you information for the scholarships offered by the given government, it is available to get the degree from any government (university or organization).");
		documents.add("This service finds a book on its publication number.");
		documents.add("This service is used to know about a hotel of a given village.");
		documents.add("MerkelD is a german firm and presents a service that returns not only price of a given preparedfood, but also informs the usage of the food in certain type of diet.");
		documents.add("This service returns a publication on given ISBN.");
		documents.add("This service adds any object as an item in the shopping cart of a authorised person.");
		documents.add("This service returns recommended price of a science-fiction novel for a user.");
		documents.add("This service creates an account for the customer. This is required before a transport can be booked.");
		documents.add("If you are interest to know about available skilled position in a certain country and city, then use this service. If the city input is not given then the output is a job in the whole country.");
		documents.add("This service finds a publication on publication number, it also returns also its author.");
		documents.add("UNO offers a service that returns a certain type of medical doctors needed by UNO.");
		documents.add("This service returns publication references of a research fellow.");
		documents.add("It is a reliable service to know about the funding provided for the missile by the given government.");
		documents.add("This service returns you the funding provided for a Projectile weapon, misiles, by the given government.");
		documents.add("There are some agencies, who attracts customers by providing a car and a gift auto.");
		documents.add("This service returns Icing condition of Germany.");
		documents.add("This service returns some special kind of tea, useful for stomach problems, obtainable from a drugstore.");
		documents.add("This service returns information of all famous hotels of the world.");
		documents.add("Training tutorial and course for windows, frontpage and for Programming with .NET");
		documents.add("This service returns the funding for missiles spent by the asian governments.");
		documents.add("Lecturer is also a person who conducts a course in the university. This service returns name of lecturers who teach the current semmester of the university.");
		documents.add("This service returns an abstract information including all publication references of a researcher.");
		documents.add("This service returns price of a book for a authorised person.");
		documents.add("This is a recommended service to know about the beach for the beach sports.");
		documents.add("The service provides a list of flights that are apropriate to bring the patient from one to another location.");
		documents.add("Book a flight. The customer using this service must have an account for this flight company!");
		documents.add("This is a reliable service to know particular skilled positions in a city.");
		documents.add("It is P2P service to locate films for a title in a network.");
		documents.add("This service returns Lecturers of the Munchen university.");
		documents.add("This service returns the funding, specially for the ballistic missile, provided by the given government,");
		documents.add("Hallo2 service is a refined service to get cola for a given price.");
		documents.add("This service returns a recommended price of an offer of a set of DVD player and MP3Player and also information of its shipping cost.");
		documents.add("This service returns person who wrote the given novel.");
		documents.add("This service returns Lecturers of the given higher educational organization.");
		documents.add("This service varify account of an authorised person and returns recommended price of an object.");
		documents.add("This service returns total price of the items in a shopping basket of the user.");
		documents.add("DPA service returns certain type diagnostic process available in the certain hospital, and its cost also.");
		documents.add("This service informs you about the certain type of apple selling by a certain retail store.");
		documents.add("SaturnVideo is an official service of Saturn store to return list of available video media.");
		documents.add("SOH presents the best service to know about the destination for surfing.");
		documents.add("This service adds a link into database from given one geopolitical entity to the other geopolitical entity.");
		documents.add("GEWS service is used to get season report of a particular geogrphical entity.");
		documents.add("PFP is a reknown service that returns not only price of a given preparedfood, but also its expirydate.");
		documents.add("This service returns price of a prepared food.");
		documents.add("This service returns a publication on its publication number.");
		documents.add("This service returns the weather process condition of a given country.");
		documents.add("This service returns hotel of the capital city of the country, the country is an optional input.");
		documents.add("This service returns price of a romantic novel for a user.");
		documents.add("This service returns prepared food selling by the given whole sale store.");
		documents.add("This service provides a book on the given ISBN.");
		documents.add("RPCW service returns a coffee and a whiskey for the given recommended price, also the brand name and quality of the both products.");
		documents.add("This service returns professor of a university.");
		documents.add("A pioneer service to know about the city for both sports hiking and surfing.");
		documents.add("The main focus of this service to return the review on a science-fiction novel of a user, also the recommended price (it may be not represent the current price of the novel.)");
		documents.add("This service returns information of a hotel of a given city of Germany.");
		documents.add("T-service analysed car market value and deduced that color is one of the main issue in its popularity, so this service provides list of price of car with its color.");
		documents.add("This service informs you funding for the self powered devices, such as street cars, provided by the government organization.");
		documents.add("This service facilitate a client to find a company which offer the same car within range of given amount of money.");
		documents.add("This service returns the information of a book whose title best matches the give string");
		documents.add("HDP2 is a new version of HDP service that returns about a certain diagnostic system avaialbe in a certain hospital, and its time duration.");
		documents.add("This service informs you about internal change, EarthTremor or earthquake, that happened in the earth.");
		documents.add("A book searchengine");
		documents.add("TR service returns price of a novel for a certain person.");
		documents.add("BShop is a recommended service that verifies credit card account of an authorised person and adds the selected book in his shopping cart.");
		documents.add("Bea service checks credit account of an authorised person and adds the selected book in his shopping cart.");
		documents.add("This service returns certain type diagnostic process available in the certain hospital.");
		documents.add("This service stores the input title into a database.");
		documents.add("This service returns an addresses, whether email or postal, of a researcher.");
		documents.add("You can find a recommended price of a german media player, brand name in high quality especially DVD and MP3 player.");
		documents.add("This serive returns Fodder, food for animal, obtainable from the certain grocery store.");
		documents.add("This service returns the weather condition, large-scale atmospheric processes that influence weather in a given region for 2-5 days.");
		documents.add("This service provides all relative paths from Frankfurt to your location.");
		documents.add("This service returns an address of a visiting researcher.");
		documents.add("This service offer a liquid like water for a maximum price.");
		documents.add("This service returns recommended price of an offer of a set of DVD player and MP3Player and also you received a message of its delivery.");
		documents.add("This service returns a coffeewithwhiskey for the given recommended price, also suggestion on usage of this type of coffee.");
		documents.add("Ben service returns recommended price in Euro of a prepared food.");
		documents.add("Our US clients offers some professional jobs, you can find also the region of US in which you have to work.");
		documents.add("SSP service informs you about lense with its price and its range camera, available in the certain shoppingmall.");
		documents.add("This service returns reader of the given book and his/her review also and the last person who already made his comments on that book.");
		documents.add("It is a reliable service to return hotel of the city of the country");
		documents.add("This is a nice service to know about the city for both sports hiking and surfing.");
		documents.add("Book a transport. The service can only be used by having a customer account for that company already.");
		documents.add("This service returns author of the novel and also its genre that shows that the novel is comic, fantasy or science-fiction.");
		documents.add("If you want to travel to Berlin, the we provide a service to return a map indicate all paths from your location to Berlin.");
		documents.add("This service checks credit account of an authorised person and adds the selected book in his shopping cart.");
		documents.add("This service returns a certain motion, where the agent is an Organism and the patient is a BodyPart, and proposition about it.");
		documents.add("This service returns price of the pair of a car and a cycle.");
		documents.add("It is the recommended service to inform you scholarship offered from the given government organization for achievement of the degree.");
		documents.add("It returns DESTINATION of my office.");
		documents.add("This service takes as input account number or product ID and returns cola as gift or biproduct.");
		documents.add("This service adds a publication material as an item in the session of a authorised person.");
		documents.add("This service returns price of a set of a MP3Player and a portable DVD Player.");
		documents.add("This service returns food presented by mercantile organization.");
		documents.add("This service returns the funds used for the missile and mass destruction weapons by the Iraqi government.");
		documents.add("It is a european service which takes price in euro as input and returns a coffee and a whiskey affordable within the given price.");
		documents.add("This service returns skilled occupation position in a certain public company of a certain country. If the public-company input is not given then the output is a job in all public companies of the country.");
		documents.add("This service returns a price of a set of a WMA Player and Wireless player.");
		documents.add("This service returns a coffeewithwhiskey for the given price.");
		documents.add("Cola is used as a drink in most parties. This service returns best cola for the given maximum price.");
		documents.add("It is a recommended service to know about the funding for degree by the given government.");
		documents.add("SPD-Grune emphasis on tax price (in Euros) of a certain prepared food, and its drawback on environment.");
		documents.add("This service returns price of Fish (dried) of our company.");
		documents.add("This service returns information of a hotel, it needs two inputs country and village.");
		documents.add("This service returns the funding provided for the missile by the given government organization.");
		documents.add("One can find the finest prepared food available at the certain retail store.");
		documents.add("This service varify account of an authorised person and returns tax free price of the book.");
		documents.add("This service returns lightening condition of a region.");
		documents.add("JJ is a 3wheeled Audi car, you can find its price here.");
		documents.add("PDS is an political reknown service that returns the destination where both games hiking and surfing are available.");
		documents.add("This service returns DESTINATION of the organization responsible for surfing.");
		documents.add("This service returns a certain flour, butter and dough offered by the given grocerystore. The output things are used to make a cake.");
		documents.add("This service tries to find price of a car within given amount of money.");
		documents.add("This service return name of books published by the publication.");
		documents.add("Ziku is a frequent usable service to return books written by the given author and their prices also.");
		documents.add("This service returns coffee and whiskey (products) for the given recommended price and avaialbe in the particular hotel. If hotel input is not given then it returns coffee and whiskey products for the given price..");
		documents.add("This service returns recommended price of a set of MP3Player and DVD Player.");
		documents.add("This service returns the funding for missiles spent by the pakistani government.");
		documents.add("This service returns the offered scholarship from the given national government.");
		documents.add("This service returns postal address of a researcher.");
		documents.add("This service returns information of a hotel, it needs two inputs country and town.");
		documents.add("This service returns available VHS for a given title.");
		documents.add("This service returns coffee and whiskey (products) for the given recommended price.");
		documents.add("This service varify account of an authorised person and returns recommended price of the book.");
		documents.add("The service selects one hospital out of a list of hospitals");
		documents.add("This service returns destination of Legal agent who is responsible for surfing.");
		documents.add("This service returns the launche taxes on a camera.");
		documents.add("This service returns price of an offer of a set of DVD player and MP3Player and also you received a message of its delivery.");
		documents.add("This service is a helpful service to find an available video media with its recommended price.");
		documents.add("BMW service returns current available skilled occupation in BMW company.");
		documents.add("ABC is a recommended service to locate action films for a title.");
		documents.add("This service is a recommended service to know about  Heidelburg's, a famous city of Germany, luxury hotel.");
		documents.add("This service returns lecturer of the given educational organization.");
		documents.add("This service returns a recommended price and quality of a set of Portable DVD player and MP3 player.");
		documents.add("The service creates an customer account for further flight bookings.");
		documents.add("This service returns recommended price of the car in Dollars.");
		documents.add("It is a very nice and one of the best services which return coffee and whiskey for the given recommended price.");
		documents.add("This is a recommended service to know about the national park for the sports.");
		documents.add("This service returns the best coffee and whiskey for a given price.");
		documents.add("Checks a cost and healing plan for a proposed treatment. If accomplished, the treatment can take place.");
		documents.add("This service returns investigating facility in the given hospital and its postal address.");
		documents.add("This is the best service to know about the destination for surfing.");
		documents.add("This service returns price of a set of MP3Player and a CD Player.");
		documents.add("Hallo service is a primere service to get cola for a given price.");
		documents.add("This service returns the funding, specially for the missiles, provided by the given government,");
		documents.add("This is the best service to know about the beach for surfing.");
		documents.add("This service varify account of an authorised person and returns current price of the book.");
		documents.add("This service is dedicated to return price of Phillips digital camera available in a mercentile organization.");
		documents.add("This service returns both price and taxed price of a book.");
		documents.add("This service returns Academic Support Staff of the university.");
		documents.add("GEWS service is used to get season report and proposition about a weather status of a particular geogrphical region.");
		documents.add("ODGC service informs for diagnosting system and its cost, available in the given organization, such like hospital.");
		documents.add("This service returns giving aid by the government for missiles technology.");
		documents.add("The service checks, whether the hospital has capabilities to treat a patient on the given date with the desired treatment.");
		documents.add("This service informs a certain user  price of a given science-fiction novel.");
		documents.add("This service returns lecturer of the given organization.");
		documents.add("SAAR is a well known service that returns destination of the organization responsible for surfing.");
		documents.add("It is an attractive service to know about the scholarship offered for the academic degree by the given government.");
		documents.add("This service informs you financing for the Missiles by the US government.");
		documents.add("This service returns a book on the given publication number, also its publisher and authors.");
		documents.add("This service provides a whiskey and colabeer for the maximum price");
		documents.add("This service finds skilled occupation in a given geopolitical entity, like Germany, and also returns company who offers that job.");
		documents.add("TREE service returns an addresses, whether email or postal, of a researcher in academia.");
		documents.add("This service finds a comedy film for a title.");
		documents.add("This service returns certain type diagnostic process available in the certain hospital, if the other input is given then then the output should fulfills requirements of the user question.");
		documents.add("This service returns an address of a research assistant.");
		documents.add("This service returns information of a hotel of a given city.");
		documents.add("This service returns prepared food with its price obtainable from a certain grocery store.");
		documents.add("This service provides a bread/biscuit for the maximum price and a cola (a small package for advertisement of another company)..");
		documents.add("This service supply a critical report of analysis of a car, including its price also.");
		documents.add("Academic Book Number booksearch");
		documents.add("MMVideo is a recommended service return available video media for a title in Media Market store.");
		documents.add("This service checks the room availability for a desired treatment");
		documents.add("This service returns a coffee and a whiskey for the given recommended price, also it returns proposition of the products.");
		documents.add("This service returns researcher of a university");
		documents.add("This service returns weatherfront of a given city.");
		documents.add("This service offer a delicious drink for a maximum price, for example cola.");
		documents.add("This service returns set income tax on the certain profession in a given country.");
		documents.add("This service informs you about the predicting process, that is used for complex syndrome, avialable in a certain hospital.");
		documents.add("This service informs you financing for the Missiles by the government");
		documents.add("This service searches a comedy film for a title.");
		documents.add("This service informs you about the fodder imported from US.");
		documents.add("This service gives you information for the lending offered by the given government, it is available to get the degree from any university of the government.");
		documents.add("This service returns lecturer of the given learned center organization.");
		documents.add("This service returns hotel of the capital city of the country, the country is an optional input, and it returns the destination of the city also.");
		documents.add("This service returns Spainish tax on the given prepared food.");
		documents.add("This service returns price of a book, person is an optional input.");
		documents.add("MAK service returns recommended price in Bollar of a prepared food.");
		documents.add("This service returns a recommended price (in dollars) of an offer of a set of DVD player and MP3Player and also information of its shipping cost.");
		documents.add("The service creates a flight account for the customer");
		documents.add("This service returns the weather condition in a given region.");
		documents.add("This service returns an address of a person, taxi driver etc.");
		documents.add("SFN is an attractive service to get price and author of a science-fiction novel.");
		documents.add("This service returns author of the given science-fiction book.");
		documents.add("return price of a book");
		documents.add("This service returns city of a reputable generic agent who is responsible for sports especially surfing. The registered person is an optional input.");
		documents.add("This service returns author of the given novel, and user review on it.");
		documents.add("This service returns SkilledOccupation needed by the certain country.");
		documents.add("This service returns the student experiments, and its time duration also, organized by the certain organization.");
		documents.add("This service returns author and price of a book, short-story or novel.");
		documents.add("This service return not only price of a car, but also tell how much tax its price includes.");
		documents.add("This service returns recommended price of the pair of a car and bicycle.");
		documents.add("B-co service returns price of animal food, or seeds that is used as animal food.");
		documents.add("An interested service to provide charm price of an expensive car.");
		documents.add("MarkoPS service informs you about price of a prepared food, and also its tax price according to a certain stock market.");
		documents.add("HDP is a reknown service to get to know about a certain diagnostic system avaialbe in a certain hospital, this service returns also summary of the output system.");
		documents.add("This service returns senior researcher fellow of the given higher educational organization.");
		documents.add("It is the most used service for acknowledgement of the hotel in a city of a country.");
		documents.add("This service searches DVD and VHS for a title.");
		documents.add("This service offer a ColaBeer for the given maximum price.");
		documents.add("It is an attractive service to know about the scholarship offered for the award by the german government.");
		documents.add("This service informs you about available digital SLR, its price and availability time in the certain shoppingmall.");
		documents.add("This service returns lecturer of a university.");
		documents.add("Before a treatment can take place, the physician has to set up a cost and healing plan. This plan has to be agreed to by the patient's health insurance.");
		documents.add("This service returns a certain type translocation of chemical weapons.");
		documents.add("This service returns a coffee and a whiskey for the given recommended price, also the brand name of the both products.");
		documents.add("ULi is a reliable service for locating a publication on publication number, it also returns associated currency.");
		documents.add("This service returns price of a novel.");
		documents.add("This service returns author of the novel, and his commitment.");
		documents.add("This service returns the funding, specially for the atomic bomb, provided by the given government,");
		documents.add("This service lets the customer book a flight. A flight number and an account is required.");
		documents.add("GRWF service is used to get weather system of a particular geogrphical region.");
		documents.add("This service tells you that is there any availability of hotel in a city of the country during the given duration, the duration is obligatory input.");
		documents.add("Returns books information to teach and help exams of Oracle networking course");
		documents.add("Delivers the needed drug names for a treatment");
		documents.add("PP service returns the most popular publications.");
		documents.add("This service returns the offered scholarship, specially for high level education, from the given national government, and also phaysical quantity which describes the area of its land.");
		documents.add("This service returns the funding for the missile and other weapons by the government.");
		documents.add("This service is the recommended service to know about the hotel in a geographical region.");
		documents.add("This service returns the best coffee and whiskey for a given price, and also informs you about its quality.");
		documents.add("This service gives you acknowledgement for the lending as a special offer from the given city government.");
		documents.add("Bamzon is a popular service to return recommended price of a book.");
		documents.add("Selects out of a list of flights one flight and returns its flight number");
		documents.add("Tizon is a rival US company of the book providers. It provides a web service to return recommended price of a book.");
		documents.add("This service returns the scholar for degree provided by the given government, and also quantity of the scholarship.");
		documents.add("This service returns author of the publication (novel or short story)");
		documents.add("Uses the ISBN to return price of a book");
		documents.add("It is an attracted service to return cola for a tax free price.");
		documents.add("This service returns a set of icons reprsent roadway signs in the given location, and also photographs of the location.");
		documents.add("This service informs you funding for the ballistic Missile provided by the government, and target range of the missile.");
		documents.add("This is the recommended service to know about the national park for surfing.");
		documents.add("This service returns a recommended price of a CD player.");
		documents.add("SR service returns available camera and its recommended price in the certain shoppingmall.");
		documents.add("This service returns hotel of the city of Saarland state of the germany.");
		documents.add("CityBank needs a certain type of banker, you can contact them through the address.");
		documents.add("This service returns the funding for missiles spent by the North Koria government.");
		documents.add("This service returns the student experiments organized by the certain organization.");
		documents.add("This service returns investigating facility in the given hospital and its address.");
		documents.add("It is one of the most used service to return Lecturers of the given higher educational organization, like universities..");
		documents.add("If a hospital is not capable of treating a patient, his relocation has to be set up. With this service, the hospital physician can make a transport proposal for delivering the patient to the new hospital.");
		documents.add("leynthu provides car rent service in Korea.");
		documents.add("It is the first service that returns senior researcher fellow of the given higher educational organization.");
		documents.add("This service returns destination of generic agent who is responsible for surfing.");
		documents.add("This service returns available profession jobs in a given country.");
		documents.add("This service returns a recommended price of a set of MP3Player and a DVD Player.");
		documents.add("This service is used to get weather process condition of a particular geogrphical region.");
		documents.add("This service informs recommend price of the food.");
		documents.add("This service returns welfares presented by the given government, specially for a high level education.");
		documents.add("This service adds a printed material as an item in the shopping cart of a authorised person.");
		documents.add("This service returns not only a recommended price but also tax price of a set of MP3Player and a portable DVD Player.");
		documents.add("This service is used to get coffee and whiskey (products) for the maximum price.");
		documents.add("A magical price of a red ferrari car.");
		documents.add("This service returns destination of the organization responsible for hiking.");
		documents.add("DFG presents a service that returns the destination where both games hiking and surfing are available.");
		documents.add("This service returns the offered scholarship from the given national government, this scholarship is for high educated persons.");
		documents.add("CCP service informs you for a compact camera and its recommended price in euro, available in the certain shoppingmall.");
		documents.add("This service is used not only to know the hotel in which the person is working, but also about sports  of the city of the country.");
		documents.add("you can acquire recommended price in Euro for a car.");
		documents.add("This service returns information of the used technology in the auto.");
		documents.add("This service returns scholarship offered from the given government, and also some other related organizations.");
		documents.add("If you want to travel from Frankfurt to Berlin, then use this service to provide you a map showing the shortest path between the two cities.");
		documents.add("MedDiag provides a service to return a type of diagnostic investigating in a certain hospital.");
		documents.add("BB service returns price of a book for a certain user.");
		documents.add("This service returns a high comedy film for a title, plus a report including its rate and other features.");
		documents.add("This service returns current weatherfront of Bombay, India.");
		documents.add("This service the certain type investigating facility available in the certain hospital.");
		documents.add("This service returns not only price of a car, but also a short report about its recommendation.");
		documents.add("GEWS service is used to get season report of a particular geogrphical region.");
		documents.add("This service returns price of a set of MP3Player and DVD Player.");
		documents.add("This serive provides the best honey produced by BP company.");
		documents.add("This service returns recommended price in dollars of a science-fiction novel for a user.");
		documents.add("This is a nice service to know about the rural area where you can the sports especially hiking and surfing.");
		documents.add("This service return price of a science fiction book for a user.");
		documents.add("It is an attractive service to know about the scholarship offered for the award by the given government.");
		documents.add("This service returns photographs of the location.");
		documents.add("This is a nice service to know about special welfare for the degree offered by the government.");
		documents.add("This service returns an irish coffee for the given recommended price.");
		documents.add("This service returns recommended price of a set of MP3Player and DVD Player.");
		documents.add("Checks the availability of hospital staff for a desired treatment");
		documents.add("This service informs you about the diagnosticprocess, that is proved unsuccessful, with reasoning.");
		documents.add("A wonderful list of price of auto is available.");
		documents.add("This service returns prepared food that you can buy from the certain grocery store.");
		documents.add("This service returns info of a person, who is working as a proprietor in my shop.");
		documents.add("This is a nice service to know about the rural area where you can find both sports hiking and surfing.");
		documents.add("A client can acquire information of recommended price in Euro for a car whose model year is given also.");
		documents.add("This service gives you information for the lending offered by the given national government.");
		documents.add("This service returns information of a hotel of a given city of Europe. It needs also postal address/code also.");
		documents.add("This service returns the destination of the organization who arrange certain type of sports.");
		documents.add("This service is dedicated to return map of france.");
		documents.add("The service returns a list of transport options that match the given criteria.");
		documents.add("This service returns skilled occupation job in a given company and its time duration.");
		documents.add("This service returns an address of a professor, professor is also a researcher.");
		documents.add("This service informs you for a person who works as co-publisher of a certain book.");
		documents.add("This is a nice service to know about the city where you can find both sports hiking and surfing.");
		documents.add("This service checks credit account of an authorised user and adds the selected book in his shopping cart.");
		documents.add("This service can be used by only a registerd user, it returns the best recommended price of a book.");
		documents.add("Mega service informs about a comedy film for a title.");
		documents.add("If a doctor wants to see a patient's medical records, he must get an authorization before. This service handles physician authorizations.");
		documents.add("This service informs you about the certain type of apple grown under supervising of given corporation.");
		documents.add("This service returns the entrance fee in dollars for a person.");
		documents.add("Geo service returns open offered skilled occupation position in a given geopolitical entity, for instance Russia, Italy etc.");
		documents.add("This service returns recommended price in Euro of the pair of a car and a cycle.");
		documents.add("This service returns professors of the given higher educational organization.");
		documents.add("This service returns a camera and its price available in a shopping-mall.");
		documents.add("It is an attractive service to know about the scholarship offered for the award by the german government.");
		documents.add("Deo service is mostly used service that informs a certain user for a given science-fiction novel.");
		documents.add("This service provides information bearing objects, such as messages, news items etc.");
		documents.add("MedInvestigating provides a service to return a type of medical investigating in a certain medical clinic.");
		documents.add("This service returns an address of a researcher.");
		documents.add("This service informs you financing for the Missiles by the Russian government.");
		documents.add("MianMarkt presents a service to get price of prepared food for a certain customer (agent), which is an optional input.");
		documents.add("Books a transport by taxi, train or bus for delivering a patient to a new hospital");
		documents.add("This is a recommended service to know about the national park for hiking and surfing.");
		documents.add("get price of 1bicycle and a cycle (that is availabe at a low price)");
		documents.add("This service informs you about the current US chief judge.");
		documents.add("CompJ  service returns author of the current computer science journal.");
		documents.add("This service informs the taxed price of a book.");
		documents.add("This service returns author and price of a book, short-story or text book (but no novel).");
		documents.add("ODG service informs for diagnosting system available in the given organization, such like hospital.");
		documents.add("This service add the input book into your shopping cart.");
		documents.add("This service returns the destination where both games hiking and surfing are available.");
		documents.add("This service returns not only price of a book, but also its review.");
		documents.add("This service returns the destination of the legal agent who organize certain type of sports.");
		documents.add("This service returns available video media for a title.");
		documents.add("This service provides a cola for the maximum price and quality. The quality is an optional input.");
		documents.add("This service returns the funding for missiles spent by the indian government.");
		documents.add("This service informs you about the prepared food which is not available in the certain grocerystore and its required quantity.");
		documents.add("Always use this service to know about the destination for surfing.");
		documents.add("This service retruns price of both items separately available in Media markt, digital and standard analog camera, with their prices.");
		documents.add("It is the recommended service that returns price of a set of MP3Player and DVD Player.");
		documents.add("Selects one transport opportunity out of a list of transports and returns its transport number");
		documents.add("If you want to become a certain deacon, CDeacon offers a nice service to return Deacon postion available in the given country.");
		documents.add("Ziko service is mostly used to find coffee and whiskey (products) for the given price.");
		documents.add("This is a recommended service to know about the best destination of the sports.");
		documents.add("This service retrieves the medical records of a patient. An authorization of the physician is required for that task.");
		documents.add("This service reads the input recorded video and returns information of activities and hotels of the given geopolitical entity.");
		documents.add("This service returns Lecturers of the Zambia university. It is the famous university of Zambia.");
		documents.add("This service returns you the funding provided for the missile by the given government.");
		documents.add("The best service that returns destination of the organization who arrange the surfing.");
		documents.add("This services informs a chosen hospital about the (soon) arriving of a patient");
		documents.add("If you buy a certain type of camera, you have also to give the tax price, and this service returns the duty tax too if you buy the same camera outside the country.");
		documents.add("This is a service to find any media, Film, CD etc., for a title.");
		documents.add("This service is used to get coffee and whiskey (products) for the price.");
		documents.add("CF is a very nice service to return the films of both types, comedy and fantasy.");
		documents.add("This service returns price of a car.");
		documents.add("It is a reliable service that states about price of a given science-fiction novel to a user.");
		documents.add("It is the most user service to get the author of the novel.");
		documents.add("It is a recommended service that returns lecturer of a university.");
		documents.add("This service finds skilled occupation in a given geopolitical regions, like Europe, and also returns organization who offers that job.");
		documents.add("Merchant service returns prepared food of the certain store.");
		documents.add("This service returns author of the book the least published book on data mining by the Addison publisher.");
		documents.add("A Service that searchest the cheapest Price for a book");
		documents.add("This service is mostly used to know about special financial help, honorary award, for the degree offered by the government.");
		documents.add("This service returns a map of a given countary. This map is useful to find a route for traveling between two cities.");
		documents.add("BDe is a competitor web service to return recommended price of a monograph in Euro.");
		documents.add("This service informs you financing for the Missiles by the Chinees government.");
		documents.add("ZAF service searches films for the given title, and also returns its price and quality.");
		documents.add("This service offer a hot drink for a maximum price, for coffee, tea etc.");
		documents.add("This service finds letter on the item number.");
		documents.add("Toyota is usually considered as a cheap car. This service returns its price.");
		documents.add("This service find an obtainable media on the given title.");
		documents.add("This is the recommended service to know about the farmland for surfing.");
		documents.add("This service informs you about German polling.");
		documents.add("This service returns not only a recommended price but also tax price of a set of MP3Player and a portable DVD Player.");
		documents.add("This service informs you not onlay an affordable price of the book but also its type (hard-cover or paper back)  and size (small/large/medium).");
		documents.add("FN is a recommended service returns price of a fantasy novel for a given person.");
		documents.add("This service gives good opinion to buy a cheap car.");
		documents.add("This service returns price of Cannon digital camera, available in all markets.");
		documents.add("This service returns recommended price of the car.");
		documents.add("PrCJ service returns skilled occupation position in a certain project.");
		documents.add("Allows booksearch using the academic book number");
		documents.add("This service varify account of an authorised person and adds the selected printed material in his shopping cart.");
		documents.add("ZOO is a reknown service that returns an addresses, whether email or postal, of a researcher.");
		documents.add("IPP service returns intentional psychological process offered in a given hospital. The summary of the output is also obtained, in concise words the intentional psychological process an intentional process that can be realized entirely within the mind or brain of an Organism.");
		documents.add("This service is delegated to return a suitable price of a book, with its review and other recommended books.");
		documents.add("This service is the best service to know about the hotel in a geographical entity.");
		documents.add("This service returns an address of a reader in academia.");
		documents.add("This service returns food available in the certain grocery store.");
		documents.add("This service returns city of a reputable generic agent who is responsible for sports especially surfing.");
		documents.add("This service returns list of video media of Smith Lee production.");
		documents.add("One can discover a film for a title by using this service.");
		documents.add("This service returns price (including taxes also) of the pair of a car and a cycle.");
		documents.add("This service returns reader of the given book and his/her review also.");
		documents.add("This service returns the best coffee and whiskey for a given price, age and quality of the products also.");
		documents.add("This service of a patient's health insurance send the calling patient a telephone number of the nearest emergency center (EMA)");
		documents.add("This service returns price of a given raw food.");
		documents.add("You can search a skilled occupation position in a given company, like BMW etc.");
		documents.add("If an accident or emergency situation happened, the patient (or a witness)      uses this service to send the EMA the GPS position of the location. As a      result the service delivers the arrival time of the ambulance and the      emergency physician.");
		documents.add("It is the recommended service to inform you scholarship offered from the given government organization.");
		documents.add("This service returns the natural films produced by Discovery channel.");
		documents.add("This service returns not only publication references of a researcher, but also its  postal address.");
		documents.add("This service returns financial help, it should be given back after some period, for the degree offered by the government.");
		documents.add("Sends the medical records to the physician, if he's authorized");
		documents.add("This service informs for the certain tea and coffee and comparison reports of their advantages and disadvantages.");
		documents.add("This service is a favourite service that returns hotel info of the given country.");
		documents.add("This service returns author of the novel, and its time of publishing.");
		documents.add("DFKI returns the finest coffee and whiskey for a given price.");
		documents.add("This service returns recommended price and tax price (in euros) of certain bread or biscuit..");
		documents.add("This service returns author of the given science-fiction novel.");
		documents.add("Gel is a wellknown name in hotels, and provide a service to know about  the luxury hotel of the city of the country.");
		documents.add("Some ones are interested to know price and model of an auto, this service is a nice service to fulfill the requirements.");
		documents.add("This service returns the best destination for hiking.");
		documents.add("This service returns not only publication references of a researcher, but also its  postal address.");
		documents.add("This service returns the unliteral giving money that is given by the government organization for missiles.");
		documents.add("This service reserve hotel in a city of the country during the given duration.");
		documents.add("This service varify account of an authorised person and adds the selected monograph in his shopping cart.");
		documents.add("This service returns author of the novel.");
		documents.add("This service returns US tax on the given prepared food.");
		documents.add("This serive provides the finest seed which is produced by the cereal grasses, e.g. rice, corn, wheat, etc. The government agriculture department is responsible to grow them.");
		documents.add("This service informs you about the exportable food.");
		documents.add("This service returns the information of a publication whose publication number matches the given publication-number in a specific format text.");
		documents.add("This service returns prepared food available in the certain store.");
		documents.add("This service informs you funding for the weapons such like Missiles provided by the national government");
		documents.add("This service returns the funding, specially for the PhD academic degree, provided by the given government,");
		documents.add("This service returns an abstract information, including address and publication references, of a researcher.");
		documents.add("This service retruns recommended price of Jockey shirt obtainable in a certain shoppingmall.");
		documents.add("You can use this service to get video media for the given name.");
		documents.add("Self-service solution for video creation and encoding into streaming format. This service finds a software for a title, it also returns its availability as CD and price.");
		documents.add("This service returns publisher of a certain book.");
		documents.add("This service returns highly rated  films, which won oscar awards.");
		documents.add("This service is mostly used to get the review on a science-fiction novel of a user, also the recommended price in Euros.");
		documents.add("This service returns price of an electric device.");
		documents.add("This service returns senior lecturer of a university.");
		documents.add("The emphasis of this service to provide information of the quantity of the food in Aldi, and also approximately the maximum price (not actual) of the food.");
		documents.add("This service returns destination of a reputable generic agent who is responsible for sports especially surfing.");
		documents.add("Why I buy a fast car? this service convince you to buy a fast car and provide its recommended price.");
		documents.add("This service returns food and its quantity obatainable at the given grocery store.");
		documents.add("This service varify account of an authorised person and returns recommended price of a monograph.");
		documents.add("This service retruns information of a certain type of APS-SLR camera in British Museum, and its price in the last century.");
		documents.add("This service returns film for a title, which is not only an action film but also a low comedy film");
		documents.add("This service returns city of a reputable generic agent who is responsible for sports especially surfing.");
		documents.add("This service informs you for a cellphone with camera and its price, available in the certain shoppingmall.");
		documents.add("This service return entrance fee for a certain person.");
		documents.add("This service returns author of the given book.");
		documents.add("GIS service informs you about internal change, like weath condition and soil material, of the given geopolitical entity.");
		documents.add("This service returns town for hiking.");
		documents.add("This service varifay liability account of an authorised person and adds the selected book in his shopping cart.");
		documents.add("This service returns information of person who has booked the given novel.");
		documents.add("This service returns an edited book on its publication number.");
		documents.add("This service returns information of the natural films produced by Discovery channel, also available video media that includes these films.");
		documents.add("This service is used to get price of a vehicle.");
		documents.add("This service returns film for a title, which is not only an action film but also a high comedy film");
		documents.add("This service returns the weather process condition of a given geopolitical entity.");
		documents.add("The service checks the cost and healing plan and accepts or refuses it.");
		documents.add("DiagP service returns diagnostic process system, especially for fever, and the organization who offer this facility.");
		documents.add("This service informs you about available camera, its price and availability time in the certain shoppingmall.");
		documents.add("This service returns price of the pair of a car and 2 person bicycle.");
		documents.add("This service returns price of a microwave oven.");
		documents.add("This service informs you about borrowed (technology or help) from someone by the government for missiles.");
		documents.add("It is a pioneer service that returns professors of the given higher educational organization.");
		documents.add("SOD service returns address of the organization responsible for surfing.");
		documents.add("This service returns author of the given novel.");
		documents.add("This service returns research fellow of the university.");
		documents.add("This service varify account of an authorised person and returns recommended price of a printed materail.");
		documents.add("PTS service informs you about tax free price of a prepared food.");
		documents.add("This service returns weather process condition of Germany.");
		documents.add("This service returns price of an offer of a set of DVD player and MP3Player and also information of its shipping.");
		documents.add("BAT service is one of most reliable service to returns author and book-type of the given book.");
		documents.add("This service returns SkilledOccupation and a related part time positions offering int the certain country.");
		documents.add("Searches for a book if either the academic book number or the ISBN is avaible");
		documents.add("This service returns price and model year of the pair of a car and bicycle.");
		documents.add("CAD is a famous service to inform you about diagnostic process and its length (duration) available in certain care organization.");
		documents.add("AUS is the most used service to know about the destination for surfing.");
		documents.add("This service returns price and tax price (in Dollar) of given prepared food.");
		documents.add("This service informs you about the prepared food which is available in the certain grocerystore and its quantity.");
		documents.add("TR service reserves a novel for a certain person.");
		documents.add("This service returns Video Media of BBC.");
		documents.add("This service is used not only to know the hotel information of the city of the country, but also about its sport facilities.");
		documents.add("Searches for a book if either the academic book number or the ISBN is avaible");
		documents.add("This service is dedicated to return price of Kodak digital camera, avialable in all markets.");
		documents.add("Updates the patient's medical records with info about treatment, used      drugs, diagnosis and date of treatment");
		documents.add("Person Book");
		documents.add("This service returns city of the organization responsible for surfing.");
		documents.add("This service informs you an affordable price of the encyclopedia.");
		documents.add("This service returns drinks, whiskey or coffee, for a given price.");
		documents.add("GEIZ service returns weather season condition of a geopolitical region, and also its timeposition.");
		documents.add("This service searches for a scholarship offered by a given national government for the given degree");
		documents.add("This service returns investigating facility in the Saarland biggest hospital.");
		documents.add("This service returns price of the pair of an auto and 1 person bicycle.");
		documents.add("This service returns the offered scholarship from the given national government for reputable qualified student.");
		documents.add("Given the departure and arrival destination (GPS coordinates) and other required data, this service computes possible transport ways and returns them in a list.");
		documents.add("Ana service returns not only an irish coffee for the given recommended price, but also taste of the coffee.");
		documents.add("It is the best service that returns the scholarships offered by the given government.");
		documents.add("ZAD is mostly used service that returns recommended price in Bollar of a prepared food.");
		documents.add("DJob service returns skilled occupation job in a given company and its time duration. This service can be used only by DJob users (its precondition).");
		documents.add("This service is a recommended service to know about  the luxury hotel of the city of the country.");
		documents.add("This service returns whiskey and coffee for a given price and also informs you that how old the whiskey.");
		documents.add("This service returns author of the book, novel or short story, and its personal notes as a text for the book.");
		documents.add("This service returns Skilled positions available in the certain country.");
		documents.add("MD is the most used service to find price of a set of MP3Player and DVD Player.");
		documents.add("This service retruns price of both items purchased by a shopping mall, digital and analog camera, with their prices.");
		documents.add("PSPP service informs you about purchaseable items, films, cameras etc with its price from the certain shoppingmall.");
		documents.add("This service returns british weatherfront.");

		return documents;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
