package ch.unizh.ifi.isparql.tools;

/**
 * iSPARQL Demo. This demo tool was used to 
 * a university class to show iSPARQL queries.
 *
 * @author		Markus Stocker
 * @version 	$Id: Demo.java,v 1.1 2006-11-20 18:48:55 ms Exp $
 */

import ch.unizh.ifi.isparql.optimizer.ModOptimizer;
import ch.unizh.ifi.isparql.optimizer.CostModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;
import java.io.*;

public class Demo 
{
	/** The iSPARQL optimizer */
	private static ModOptimizer modOptimizer = new ModOptimizer();
	/** The Jena model */
	private static Model model = ModelFactory.createDefaultModel();
	
	public static void main(String[] args)
	{
		boolean verbose = false;
		boolean optimize = false;
		CostModel cm = null;
		String inGraphFileName = null;
		String inQueryFileName = null;
		String costModelFileName = null;
		
		// Eval arguments
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{				
				if (args[i].equals("--query"))
					inQueryFileName = args[i + 1];
				else if (args[i].equals("--graph"))
					inGraphFileName = args[i + 1];
				else if (args[i].equals("--cm"))
					costModelFileName = args[i + 1];
				else if (args[i].equals("--optimize"))
					optimize = true;
				else if (args[i].equals("--verbose"))
					verbose = true;
				else if (args[i].equals("--help"))
					usage();
			}
		} else
			usage();
		
		// Read query from file
		if (verbose)
			System.out.println("Loading query: " + inQueryFileName);
		String qstr = inQueryString(inQueryFileName);
		
		if (verbose)
			System.out.println("Loading dataset: " + inGraphFileName);
		
		InputStream in = FileManager.get().open( inGraphFileName );
		
		if (in == null)
			throw new IllegalArgumentException("File: " + inGraphFileName + " not found");
	
		// Load the model
		model.read(in, "");
		
		// Load the cost model
		if (optimize)
		{
			if (verbose)
				System.out.println("Loading cost model: " + costModelFileName);
			
			modOptimizer.on();
			cm = new CostModel(costModelFileName);
		} else {
			modOptimizer.off();
		}
		
		// Create an ITimer using the Factory class:
		final ITimer timer = TimerFactory.newTimer();
		
		// JIT/hotspot warmup:
		for (int j = 0; j < 3000; ++ j)
		{
			timer.start();
			timer.stop();
			timer.getDuration();
			timer.reset();
		}
		
		if (verbose)
			System.out.println("Optimizer is on: " + modOptimizer.isOn());
		
		if (verbose)
			System.out.println("Execute query ...");
		
		timer.start();
		
		// Create
		Query query = QueryFactory.create(qstr);
		
		// Optimize
		if (optimize)
			query = modOptimizer.optimize(query, cm);
		
		// Prepare
		QueryExecution qe = QueryExecutionFactory.create(query, model);

		// Execute
		ResultSet results = qe.execSelect();
		
		/* Used for DBLP AIFB evaluation
		for ( ; results.hasNext() ; )
	    {
			QuerySolution qs = results.nextSolution();
			String solution = "";
			
			for (Iterator iter = query.getResultVars().iterator(); iter.hasNext(); )
			{
				Node node = qs.get((String)iter.next()).asNode();
				if (node.isURI())
					solution += "\"" + node.getURI() + "\"";
				else if (node.isLiteral())
				{
					solution += "\"" + node.getLiteralValue().toString() + "\"";
				}
				
				solution += "\t";
			}
			
			System.out.println(solution);
	    }
	    */
		
		ResultSetFormatter.out(System.out, results, query);
		
		timer.stop();
		
		if (verbose)
			System.out.println("Query executed in: " + timer.getDuration() + " (msec)");
		
		timer.reset();
	}
	
	// Read the query from file 
	private static String inQueryString(String inQueryFileName)
	{
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";
		
		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(inQueryFileName)));
		} catch (FileNotFoundException e) { System.out.println(e); }
		
		try
		{
			while((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {}
		
		return inQueryString;
	}
	
	private static void usage()
	{
		System.out.println("ch.unizh.ifi.isparql.tools.Demo [options]");
		System.out.println("Options:");
		System.out.println("--query");
		System.out.println("--cm");
		System.out.println("--graph");
		System.out.println("--verbose");
		System.out.println("--help");
		
		System.exit(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
